local AnimationFactory = require("app.factory.AnimationFactory")
local Buff = require("app.actor.Buff")
local scheduler = require(cc.PACKAGE_NAME .. ".scheduler")
local BloodNumView = import("app.views.BloodNumView")
local DebugUtil = require("app.common.DebugUtil")
local CommUtil = require("app.common.CommUtil")
local MathUtil = require("app.common.MathUtil")

local Attacker = class("Attacker",function(attackerProperties,parent,path)
    local node = display.newNode()
    return node
end)

function Attacker:ctor(attackerProperties,parent,path)

    self.attackerProperties = attackerProperties
    self:setPosition(self.attackerProperties.position)
    
    -----------------------------------------------属性初始化----------------------------------------
    if self.attackerProperties.swim == 1 then
        self.canSwim=true
    else
        self.canSwim=false
    end
    self.offset = self.attackerProperties.offset
    self.fullHp = self.attackerProperties.fullHp
    --是否在天上
    if self.attackerProperties.air==1 then
        self.air=true
    else
        self.air=false
    end
    --是否在地下
    if self.attackerProperties.underground==1 then
        self.underground=true
    else
        self.underground=false
    end
    --是否隐身
    if self.attackerProperties.stealth==1 then
        self.stealth=true
    else
        self.stealth=false
    end
    
    self.hit_proc = self.attackerProperties.hit_proc
    self.skill_proc = self.attackerProperties.skill_proc
    self.necrit_proc = self.attackerProperties.necrit_proc
    self.armor = self.attackerProperties.armor
    self.resist = self.attackerProperties.resist
    self.necrit_mul = self.attackerProperties.necrit_mul

    self.wood = self.attackerProperties.wood
    self.water = self.attackerProperties.water
    self.fire = self.attackerProperties.fire
    self.soil = self.attackerProperties.soil
    -----------------------------------------------属性初始化----------------------------------------

    self.buffs = CCArray:create()--怪物中的buff
    self.buffs:retain()

    self.buffSkillIDs = {}

    self.gameScene = parent

    self.move_speed = self.attackerProperties.move_speed
    self.walk_animation_time = self.attackerProperties.walk_animation_time

    self.ifSlowDown = false--是否减速中
    
    --定义主精灵类
    if self.attackerProperties.body_id==70061 then
        self.mainSprite = display.newSprite("#monster_"..self.attackerProperties.body_id.."_walk_front_1.png"):addTo(self)
    else
        self.mainSprite = display.newSprite("#monster_"..self.attackerProperties.body_id.."_walk_front_1.png"):addTo(self)
    end

    self.mainSprite:setAnchorPoint(CCPoint(0.5,0))
    self.mainSprite:setScale(self.attackerProperties.scale)

    local size = self.mainSprite:getCascadeBoundingBox().size
    local anchorPoint = self.mainSprite:getAnchorPoint()
    --local textureCenterX = sposX + (0.5-anchorPoint.x)*size.width
    --local textureCenterY = posY + (0.5-anchorPoint.y)*size.height

    self.hp_bg = display.newSprite("blood_bg.png",-20,size.height-10)
    self.hp = display.newSprite("blood.png",-20,size.height-10)
    self.hp_bg:setAnchorPoint(ccp(0,0.5))
    self.hp:setAnchorPoint(ccp(0,0.5))
    --self.hp = display.newProgressTimer("blood.png",display.PROGRESS_TIMER_BAR)
    self.hp_bg:setScaleX(1)
    self.hp:setScaleX(1)
    self.hp_bg:setScaleY(0.5)
    self.hp:setScaleY(0.5)


    --self.hp:setMidpoint(CCPointMake(0, 0))
    --self.hp:setBarChangeRate(CCPointMake(1, 0))
    --self.hp:setPosition(CCPointMake(0,size.height-10))
    --self.hp:setPercentage(100)
    self:addChild(self.hp_bg)
    self:addChild(self.hp)


    if self.attackerProperties.hideBlood then
        self.hp:setVisible(false)
        self.hp_bg:setVisible(false)
    end
    
    self._waypoints = CCArray:create()--怪物需要行走的路径
    self._waypoints:retain()

    self.wayOffset = ccp(0,0)--角色的运动路径偏移量
    self.curPos=self.attackerProperties.position--游戏主角色的当前位置
    self.fCurLifeValue = self.attackerProperties.hp--游戏角色的生命值
    self.fullHP = self.attackerProperties.hp--游戏角色满血的生命值
    self.fullHP = self.fullHP * 0.8
    self.dead = false
    
    self._enemies=CCArray:create()--增加角色对应的攻击敌人对象数组
    self._enemies:retain()
    self.curWaypoint = 0
    
    self.path = path
    self.reverse = false
    self:setAttackerPath(self.path)
    
    if self.stealth then
        self.mainSprite:setOpacity(80)
        --print("隐身怪")
    end

    if self.underground then
    --if true then
        --print("遁地怪物")
        --self.mainSprite:setVisible(false)
        self.mainSprite:setDisplayFrame(display.newSpriteFrame("monster_70061_walk_front_1.png"))
        local frames = display.newFrames("monster_70061_walk_front_%d.png", 1, 8)
        local animation = display.newAnimation(frames, 1 / 8)
        --self.undergroundSprite = display.newSprite(frames[1]):addTo(self)
        self.mainSprite:setScale(self.attackerProperties.scale)
        --self.mainSprite:setAnchorPoint(CCPoint(0.5,0))
        self.mainSprite:playAnimationForever(animation)
        self.mainSprite:setOpacity(150)
        if self.stealth then
            self.mainSprite:setOpacity(80)
        end
    end

    self.gridIndex = -1
    self.oneToTwoDegrees = {false,false,false,false,false}
    self.degree = 0
    self.way = self.attackerProperties.way
    self.killLabelFlag =  math.random(2)==2 and true or false

    local waypoint = tolua.cast(self._waypoints:objectAtIndex(self.curWaypoint),"WayPoint")
    if not self.canSwim then
        self:doWalkAnimation(waypoint.direction)
    end

    self.walkStatus = true
    self.ifPause = false

    self.dieMusicIndex = math.random(1,6)
    if self.dieMusicIndex==5 then self.dieMusicIndex=6 end

end

--获取贴图中心点在世界坐标的位置
function Attacker:getTextureCenterPointInWorld(posX,posY)
    local size = self.mainSprite:getCascadeBoundingBox().size
    --print("怪物的宽高",size.width, size.height)
    local anchorPoint = self.mainSprite:getAnchorPoint()
    local textureCenterX = posX + (0.5-anchorPoint.x)*size.width
    local textureCenterY = posY + (0.5-anchorPoint.y)*size.height
    return textureCenterX,textureCenterY,size
end

function Attacker:getCenterPointInWorld(posX,posY)
    local size = self.mainSprite:getCascadeBoundingBox().size
    --print("怪物的宽高",size.width, size.height)
    local anchorPoint = self.mainSprite:getAnchorPoint()
    local textureCenterX = posX + (0.5-anchorPoint.x)*size.width
    local textureCenterY = posY + (0.5-anchorPoint.y)*size.height
    return textureCenterX,textureCenterY,size
end

function Attacker:getBox(posX,posY)
    local size = self.mainSprite:getCascadeBoundingBox().size
    --print("怪物的宽高",size.width, size.height)
    local anchorPoint = self.mainSprite:getAnchorPoint()
    local textureCenterX = posX + (0.5-anchorPoint.x)*size.width
    local textureCenterY = posY + (0.5-anchorPoint.y)*size.height
    
    return CCRect(textureCenterX-size.width/2,textureCenterY-size.height/2,size.width,size.height)
end

function Attacker:clean()
    if self.logicHandler then scheduler.unscheduleGlobal(self.logicHandler) self.logicHandler=nil end
    self:stopAllActions()
    if self.gameScene.focusMonster==self then
        self.gameScene.focusMonster = nil
    end
    local array = self:getChildren()
    for i = 0, array:count() - 1 do
        local childNode = tolua.cast(array:objectAtIndex(i), "CCSprite")
        childNode:stopAllActions()
    end
    self.dead = true
    self.gameScene.quadTree:removeMonsterFromGrid(self,self.gridIndex)
    self.buffs:release()
    self._waypoints:release()
    self._enemies:release()
    GameInstance:sharedInstance().monsters:removeObject(self)
    self:removeFromParentAndCleanup(true)
end

function Attacker:get_end_point()
    local lastWaypoint = self._waypoints:count()
    local waypoint = tolua.cast(self._waypoints:objectAtIndex(lastWaypoint-1),"WayPoint")
    return waypoint.position
end

--获取寻找目标时怪物在塔播放动画完成后所在位置点
function Attacker:getMonsterPositionAfterTowerAnimation(delayTime)
    local lastWaypoint = self._waypoints:count()
    local waypoint = nil
    if self.curWaypoint == (lastWaypoint-1) then
        waypoint = tolua.cast(self._waypoints:objectAtIndex(self.curWaypoint),"WayPoint")
    else
        waypoint = tolua.cast(self._waypoints:objectAtIndex(self.curWaypoint+1),"WayPoint")
    end
    
    --local curPosition = ccp(self:getPositionX(),self:getPositionY())
    --local temPosition = ccpMidpoint(curPosition,ccp(CommUtil.getBlankWidth() + waypoint.position.x,waypoint.position.y))
    --local predictPosition = ccpMidpoint(temPosition,ccp(CommUtil.getBlankWidth() + waypoint.position.x,waypoint.position.y))
    tolua.cast(waypoint.position,"CCPoint")
    local vector = ccpSub(waypoint.position,ccp(self:getPositionX(),self:getPositionY()))
    local angle = -1*ccpToAngle(vector)
    local dist = self.move_speed * delayTime --时间根据飞行速度来算

    local sinValue = math.sin(angle)
    local cosValue = math.cos(angle)

    local xDistance = math.abs(dist*cosValue)
    local yDistance = math.abs(dist*sinValue)

    --print("距离是",sinValue,cosValue,xDistance,yDistance)

    local tempX = nil
    local tempY = nil

    if waypoint.position.x > self:getPositionX() then
        tempX = self:getPositionX()+xDistance
    else
        tempX = self:getPositionX()-xDistance
    end

    if waypoint.position.y > self:getPositionY() then
        tempY = self:getPositionY() + yDistance
    else
        tempY = self:getPositionY() - yDistance
    end

    local tempPoint = ccp(tempX,tempY)

    local size = self.mainSprite:getBoundingBox().size
    --print("size的大小是",size.height)
    local anchorPoint = self.mainSprite:getAnchorPoint()
    local x = tempPoint.x + (0.5-anchorPoint.x)*size.width
    local y = tempPoint.y +  (0.5-anchorPoint.y)*size.height
    return x,y,size

end

--预判得到怪物的下一个路径点
function Attacker:getNextPoint(delayTime)
    local lastWaypoint = self._waypoints:count()
    local waypoint = nil
    if self.curWaypoint == (lastWaypoint-1) then
        waypoint = tolua.cast(self._waypoints:objectAtIndex(self.curWaypoint),"WayPoint")
    else
        waypoint = tolua.cast(self._waypoints:objectAtIndex(self.curWaypoint+1),"WayPoint")
    end

    tolua.cast(waypoint.position,"CCPoint")
    local vector = ccpSub(waypoint.position,ccp(self:getPositionX(),self:getPositionY()))
    local angle = -1*ccpToAngle(vector)
    local dist = self.move_speed * delayTime --时间根据飞行速度来算

    local sinValue = math.sin(angle)
    local cosValue = math.cos(angle)

    local xDistance = math.abs(dist*cosValue)
    local yDistance = math.abs(dist*sinValue)

    --print("距离是",sinValue,cosValue,xDistance,yDistance)

    local tempX = nil
    local tempY = nil

    if waypoint.position.x > self:getPositionX() then
        tempX = self:getPositionX()+xDistance
    else
        tempX = self:getPositionX()-xDistance
    end

    if waypoint.position.y > self:getPositionY() then
        tempY = self:getPositionY() + yDistance
    else
        tempY = self:getPositionY() - yDistance
    end

    local tempPoint = ccp(tempX,tempY)

    local size = self.mainSprite:getBoundingBox().size
    local anchorPoint = self.mainSprite:getAnchorPoint()
    local x = tempPoint.x + (0.5-anchorPoint.x)*size.width
    local y = tempPoint.y + 20
    return ccp(x,y)

end


function Attacker:isCloseToEnd()
    local lastWaypoint = self._waypoints:count()
    local waypoint = nil
    if lastWaypoint-self.curWaypoint<2 then
        return true
    else
        return false
    end
end

--取得行星辰箭和扫射箭的终点
function Attacker:getFlyEndPoint(offset)
    local lastWaypoint = self._waypoints:count()
    local waypoint = nil
    if self.curWaypoint == (lastWaypoint-1) then
        waypoint = tolua.cast(self._waypoints:objectAtIndex(self.curWaypoint),"WayPoint")
    else
        waypoint = tolua.cast(self._waypoints:objectAtIndex(self.curWaypoint+1),"WayPoint")
    end

    local vector = ccpSub(waypoint.position,ccp(self:getPositionX(),self:getPositionY()))
    local angle = -1*ccpToAngle(vector)
    local dist = offset

    local sinValue = math.sin(angle)
    local cosValue = math.cos(angle)

    local xDistance = math.abs(dist*cosValue)
    local yDistance = math.abs(dist*sinValue)

    --print("距离是",sinValue,cosValue,xDistance,yDistance)

    local tempX = nil
    local tempY = nil

    if waypoint.position.x > self:getPositionX() then
        tempX = self:getPositionX()-xDistance
    else
        tempX = self:getPositionX()+xDistance
    end

    if waypoint.position.y > self:getPositionY() then
        tempY = self:getPositionY() - yDistance
    else
        tempY = self:getPositionY() + yDistance
    end

    local tempPoint = ccp(tempX,tempY)

    local size = self.mainSprite:getBoundingBox().size
    local anchorPoint = self.mainSprite:getAnchorPoint()
    local x = tempPoint.x + (0.5-anchorPoint.x)*size.width
    local y = tempPoint.y + 20
    return ccp(x,y)

end


--预判得到怪物的下一个路径点
function Attacker:getPredictPosition()
    return ccp(self:getPositionX(),self:getPositionY())
end

function Attacker:getCurWayPoint()
    local waypoint = tolua.cast(self._waypoints:objectAtIndex(self.curWaypoint),"WayPoint")
    return waypoint
end

function Attacker:walk()
    if self.dead then
        return
    end
    if self.fCurLifeValue <= 0 then
        self:clean()
        return
    end
    
    local waypoint1 = tolua.cast(self._waypoints:objectAtIndex(self.curWaypoint),"WayPoint")
    local waypoint2 = tolua.cast(GameInstance:sharedInstance().beautyPoint,"WayPoint")

    if self.gameScene.activityName==nil  and ccpDistance(waypoint1.position,waypoint2.position)<100 then
        self.gameScene:helpBeauty(1)
    end

    local wayPointCount = self._waypoints:count()

    if self.curWaypoint == (wayPointCount-1) then
        --print("最后一个点了")
        self:setSelfVisible(false)
        if self.fCurLifeValue>0 then
            --播放怪物逃跑的音效
            if addingBlood:get()>0 then
                addingBlood:set(addingBlood:get()-1)
            else
                totalHP:set(totalHP:get()-self.attackerProperties.damage)
            end
            CommUtil.playSound("music/tangshenreach.mp3")

            if (gBattle.data.home_scene_id ~= 701  
                and gBattle.data.home_scene_id ~= 702 ) then
                if (totalHP:get() == 9) then
                self.gameScene.beautybg:setScaleY(0.54)
                self.gameScene.beautybg:setScaleX(0.9)
                elseif (totalHP:get() == 8) then
                self.gameScene.beautybg:setScaleY(0.48)
                self.gameScene.beautybg:setScaleX(0.8)
                elseif (totalHP:get() == 7) then
                self.gameScene.beautybg:setScaleY(0.42)
                self.gameScene.beautybg:setScaleX(0.7)
                elseif (totalHP:get() == 6) then
                self.gameScene.beautybg:setScaleY(0.36)
                self.gameScene.beautybg:setScaleX(0.6)
                elseif(totalHP:get() == 5) then
                self.gameScene.beautybg:setScaleY(0.30)
                self.gameScene.beautybg:setScaleX(0.5)
                 elseif(totalHP:get() == 4) then
                self.gameScene.beautybg:setScaleY(0.24)
                self.gameScene.beautybg:setScaleX(0.4)
                elseif(totalHP:get() == 3) then
                self.gameScene.beautybg:setScaleY(0.18)
                self.gameScene.beautybg:setScaleX(0.3)
                elseif(totalHP:get() == 2) then
                self.gameScene.beautybg:setScaleY(0.12)
                self.gameScene.beautybg:setScaleX(0.2)
                elseif(totalHP:get()==1) then
                self.gameScene.beautybg:setVisible(false)
                self.gameScene.beauty:setWhiteShader()
                elseif(totalHP:get()==0) then
                self.gameScene.beauty:stopAllActions()
                end
            end
            if addingBlood:get()==0 then
                self.gameScene:changeBeauty()
            end
            self:getParent().gameHud:updateBeautyBlood()
        end
        self:clean()
        return
    end

    --取得下一个要到达地点的位置
    local currentPosition = ccp(self:getPositionX(),self:getPositionY())
    local waypoint = tolua.cast(self._waypoints:objectAtIndex(self.curWaypoint+1),"WayPoint")
    
    local destPosition = ccp(waypoint.position.x+CommUtil.getBlankWidth(),waypoint.position.y-self.offset)
    
    --计算时间采用设计分辨率800x480
    local dist = ccpDistance(destPosition,ccp(self:getPositionX(),self:getPositionY()))
    local runTime = dist/self.move_speed

    if dist > 0 then
        runTime = dist/self.move_speed *(1 + math.abs(destPosition.y-self:getPositionY())/dist)
    end
    if self.reverse then
        self.mainSprite:setFlipX(not waypoint.isFlipX)
    else
        self.mainSprite:setFlipX(waypoint.isFlipX)
    end
    local action = CCSequence:createWithTwoActions(
            CCMoveTo:create(runTime,destPosition), 
            CCCallFunc:create(function()
                self.curWaypoint = self.curWaypoint + 1
                self:walk()
    end))
    action:setTag(99)
    self:runAction(action)
end

function Attacker:updateGridIndex()
    local posX,posY = self:getPosition()
    local monsterRect,offset,centerPoint,size = self:getRectAndOffset()

    local endIndex = gridRow*gridColumn
    for i=1,endIndex do
        local rect = self.gameScene.quadTree.rects[i]
        if rect~=nil and rect:intersectsRect(monsterRect) then
            if self.gridIndex~=i then
                self.gameScene.quadTree:removeMonsterFromGrid(self,self.gridIndex)
                self.gameScene.quadTree:addMonesterInGrid(self,i)
                self.gridIndex = i
            end
            return
        end
    end
end

function Attacker:doWalkAnimation(direction)
    self.mainSprite:stopAllActions()
    local animation = AnimationFactory.getMonsterAnimation(direction,self.attackerProperties.body_id,self.attackerProperties.walk_animation_time)
    if animation~=nil then 
        transition.execute(self.mainSprite, CCAnimate:create(animation), {
            onComplete = function()
                local wayPointCount = self._waypoints:count()
                local waypoint = tolua.cast(self._waypoints:objectAtIndex((self.curWaypoint+1>wayPointCount-1) and (wayPointCount-1) or (self.curWaypoint+1) ),"WayPoint")
                self:doWalkAnimation(waypoint.direction)
            end,
        })
    end
end


function Attacker:addMissingLabel()
    --print("missing 没有打中")
    local gameScene = self.gameScene
    local sprite = display.newSprite("ui/fighting/fighting_hint_1.png",self:getPositionX(),self:getPositionY()+50)
    sprite:addTo(gameScene,50001)
    transition.moveTo(sprite, {y = self:getPositionY()+80, time = 0.3, easing = "bounce",onComplete=function()
        transition.fadeTo(sprite, {opacity = 0, time = 0.3,onComplete=function() sprite:removeSelf() end})
    end})
    
end

function Attacker:addCritLabel()
    --print("被暴击了")
    local gameScene = self.gameScene
    local sprite = display.newSprite("ui/fighting/fighting_hint_2.png",self:getPositionX(),self:getPositionY()+50)
    sprite:addTo(gameScene,50001)
    transition.moveTo(sprite, {y = self:getPositionY()+80, time = 0.3, easing = "bounce",onComplete=function()
        transition.fadeTo(sprite, {opacity = 0, time = 0.3,onComplete=function() sprite:removeSelf() end})
    end})
end

function Attacker:addKillLabel()
    --print("被暴击了")
    local gameScene = self.gameScene
    local sprite = nil
    if self.killLabelFlag then
        sprite = display.newSprite("ui/fighting/fighting_hint_3.png",self:getPositionX(),self:getPositionY()+50)
    else
        sprite = display.newSprite("ui/fighting/fighting_hint_4.png",self:getPositionX(),self:getPositionY()+50)
    end
    sprite:addTo(gameScene,50001)
    transition.moveTo(sprite, {y = self:getPositionY()+80, time = 0.3, easing = "bounce",onComplete=function()
        transition.fadeTo(sprite, {opacity = 0, time = 0.3,onComplete=function() sprite:removeSelf() end})
    end})
end

function Attacker:getName()
    return self.attackerProperties.name
end
function Attacker:getAttackProperties()
    return self.attackerProperties
end
function Attacker:doEat()
    self.dead = true
    GameInstance:sharedInstance().monsters:removeObject(self)
    self:removeFromParentAndCleanup(true)
end

function Attacker:addDecreaseHPLabel(decreaseHp,if_crit)
    if true then return end
    if decreaseHp<=0 then return end
    local gameScene = self.gameScene
    local numView = BloodNumView.new(string.format("%d",decreaseHp),1):addTo(gameScene,50001)
    if if_crit then numView:setScale(0.7) else numView:setScale(0.4) end
    
    numView:setPosition(self:getPositionX()+20,self:getPositionY())
    transition.moveTo(numView, {y = self:getPositionY()+80, time = 0.3, easing = "bounce",onComplete=function()
        transition.fadeTo(numView, {opacity = 0, time = 0.2,onComplete=function() numView:removeSelf() end})
    end})
    --[[
    local numView = ui.newBMFontLabel({
        text = string.format("%d",decreaseHp),
        font = "fonts/warrior_value.fnt",
    }):addTo(self.gameScene,50002)
    if if_crit then numView:setScale(0.7) else numView:setScale(0.4) end
    numView:setPosition(self:getPositionX()+20,self:getPositionY())
    transition.moveTo(numView, {y = self:getPositionY()+80, time = 0.3, easing = "bounce",onComplete=function()
        transition.fadeTo(numView, {opacity = 0, time = 0.2,onComplete=function() numView:removeSelf() end})
    end})]]

end

function Attacker:doBeHitEffect()
    --[[
    local waypoint = tolua.cast(self._waypoints:objectAtIndex(self.curWaypoint),"WayPoint")
    local direction = waypoint.direction and "front" or "back"
    local action = transition.sequence({
        CCCallFunc:create(function()
            self.mainSprite:stopAllActions()
            local spriteFrame = display.newSpriteFrame("monster_"..self.attackerProperties.body_id.."_hit_"..direction..".png")
            self.mainSprite:setDisplayFrame(spriteFrame)
        end),
        CCDelayTime:create(0.2),
        CCCallFunc:create(function()
            local waypoint = tolua.cast(self._waypoints:objectAtIndex(self.curWaypoint),"WayPoint")
            self:doWalkAnimation(waypoint.direction)
        end),
    })
    self:runAction(action)]]
end

function Attacker:decreaseHpBySkill(decreaseHp,if_crit)
    --print("更新血条显示进度")
    if self.fCurLifeValue<=0 then return end
    self:doBeHitEffect()
    self:addDecreaseHPLabel(decreaseHp,if_crit)
    if if_crit then self:addCritLabel() end
    local newhp = self.fCurLifeValue - decreaseHp
    if newhp <= 0 then
        newhp = 0
    end
    if newhp < self.fCurLifeValue then
        self.fCurLifeValue = newhp
        if newhp == 0 then
            self:runDeadAnimation()
            return
        end
    end
    local percent = self.fCurLifeValue / self.fullHP
    self.hp:setScaleX(percent)
    --self.hp:setPercentage(math.floor(percent))
end

--英雄的技能造成的伤害
function Attacker:decreaseHp(hitType,decreaseHp,if_crit)
    --print("更新血条显示进度")
    if self.fCurLifeValue<=0 then return end
    self:doBeHitEffect()
    self:addDecreaseHPLabel(decreaseHp,if_crit)
    if if_crit then self:addCritLabel() end
    local newhp = self.fCurLifeValue - decreaseHp
    if newhp <= 0 then
        newhp = 0
    end
    if newhp < self.fCurLifeValue then
        self.fCurLifeValue = newhp
        if newhp == 0 then
            --print("怪物被杀死了")
            --CommUtil.playSound("music/die"..math.random(1,4)..".mp3",false)
            --GameInstance:sharedInstance().monsters:removeObject(self)
            --self:removeFromParentAndCleanup(true)
            self:runDeadAnimation()
            return
        end
    end

    self:runBeHitAnimation(hitType)
    
    local percent = self.fCurLifeValue / self.fullHP
    self.hp:setScaleX(percent)
    --self.hp:setPercentage(math.floor(percent))
end

function  Attacker:runBeHitAnimation(hitType)
    if hitType=="hunter" or hitType=="warrior" then
        local frames,animation,offset = AnimationFactory.getBehitPhysicalAnimation()
        local behitEffectSprite = display.newSprite(frames[1])
        behitEffectSprite:setPosition(0,offset)
        self:addChild(behitEffectSprite)

        transition.execute(behitEffectSprite,CCAnimate:create(animation), {
        onComplete = function()
            behitEffectSprite:removeSelf()
        end,
        })
    else
        local frames,animation,offset = AnimationFactory.getBehitMagicAnimation()
        --local frames,animation,offset = AnimationFactory.getBehitPhysicalAnimation()
        local behitEffectSprite = display.newSprite(frames[1])
        behitEffectSprite:setPosition(0,offset)
        self:addChild(behitEffectSprite)

        transition.execute(behitEffectSprite,CCAnimate:create(animation), {
        onComplete = function()
            behitEffectSprite:removeSelf()
        end,
        })
    end
end

function  Attacker:runBeHitEffect(hitType)
    local beHitSprite = display.newSprite(string.format("ui/fighting/fighting_attack_hit%d.png",1),0,70):addTo(self)
    beHitSprite:setScale(0.6)
    beHitSprite:performWithDelay(function()
        beHitSprite:removeSelf()
    end,0.2)
end

function Attacker:subHp(beHitType,decreaseHp,if_crit)
    if self.fCurLifeValue<=0 then return end
    self:addDecreaseHPLabel(decreaseHp,if_crit)
    if if_crit then self:addCritLabel() end
    local newhp = self.fCurLifeValue - decreaseHp
    if newhp <= 0 then
        newhp = 0
    end
    if newhp < self.fCurLifeValue then
        self.fCurLifeValue = newhp
        if newhp == 0 then
            self:runDeadAnimation()
            return
        end
    end
    self:runBeHitEffect(beHitType)
    local percent = self.fCurLifeValue / self.fullHP
    self.hp:setScaleX(percent)
    --self.hp:setPercentage(percent)
end

function  Attacker:runDeadAnimation()
    self.dead = true
    local goldRandomNum = math.random(1,3)
    --if GOLD_ON and gBattle.data.home_scene_id~=701  and gBattle.data.home_scene_id~=702 and goldRandomNum<=1 and self.gameScene.golds:count()<30 then
        local centerX,centerY = self:getTextureCenterPointInWorld(self:getPositionX(),self:getPositionY())
        local dida = display.newSprite("ui/home/home_icon_copper.png",centerX,centerY):addTo(self.gameScene,0)
        dida:setScale(0.7)
        local jumpAction = CCJumpBy:create(2, ccp(math.random(-5,10),math.random(-5,5)),math.random(10,20),3)
        dida:runAction(jumpAction)

        --gold:runAction(CCRepeatForever:create(CCBlink:create(2,5)))
        --gold:runAction(CCBlink:create(2, 10))
        --gold.cantouch=true
        --self.gameScene.golds:addObject(gold)
    --end

    self:stopAllActions()
    if self.gameScene.focusMonster==self then
        self.gameScene.focusMonster = nil
    end
    self.gameScene.quadTree:removeMonsterFromGrid(self,self.gridIndex)
    local gameScene = self.gameScene
    local monsterDida = self.attackerProperties.dida

    local doaction = function()

    local uiCopperPoint = gameScene.gameHud:getUICooperPosition()
    if uiCopperPoint~=nil then
        local x,y = self:getTextureCenterPointInWorld(self:getPositionX(),self:getPositionY())
        
       -- local dida = display.newSprite("ui/home/home_icon_copper.png",x,y):addTo(gameScene,FIRST_MENU_Z_ORDER)
       -- dida:setScale(0.7)
        local distance = ccpDistance(uiCopperPoint,ccp(x,y))
        local velocity = 800

        transition.moveTo(dida, {x = uiCopperPoint.x+10, y = uiCopperPoint.y-5, time = distance/velocity,onComplete=function()
            CommUtil.playSound("music/sfx_collect.mp3")
            dida:removeSelf()
            gPlayer:addCopper(monsterDida*3)
            gameScene.gameHud:updateCooper()
            local frames = display.newFrames("guide_%02d.png", 1,8)
                        local animation = display.newAnimation(frames,1/8)
                        local effect = display.newSprite(frames[1],uiCopperPoint.x,uiCopperPoint.y):addTo(self.gameScene,60001)
                        effect:setScale(0.5)
                        transition.execute(effect,CCAnimate:create(animation),{
                        onComplete = function()
                            effect:removeSelf()
                        end,
                        })
        end})
    end
end

   

    transition.scaleTo(dida, {
                scaleX     = 0.5,
                scaleY     = 0.5,
                time       = 1,
                onComplete = function()
                 doaction()
            
        end})
               
    local frames,animation,offset = AnimationFactory.getDeadAnimation(self.attackerProperties.death_effect)
    local deadSprite = display.newSprite(frames[1])
    deadSprite:setPosition(0,offset)
    self:addChild(deadSprite)
    
    self:addKillLabel()
    self.mainSprite:stopAllActions()
    local randomNum = math.random(1,5)
    if randomNum == 1 then
        CommUtil.playSound(string.format("music/die%d.mp3",self.dieMusicIndex))
    end
    --if randomNum == 1 and self.attackerProperties.death_sound~=0 then CommUtil.playSound(string.format("music/die%d.mp3",self.attackerProperties.death_sound)) end


    transition.execute(deadSprite,CCAnimate:create(animation), {
    onComplete = function()
        local array = self:getChildren()
        for i = 0, array:count() - 1 do
            local childNode = tolua.cast(array:objectAtIndex(i), "CCSprite")
            childNode:stopAllActions()
        end
        deadSprite:removeSelf()
        self.dead = true
        self.gameScene.quadTree:removeMonsterFromGrid(self,self.gridIndex)
        GameInstance:sharedInstance().monsters:removeObject(self)
        self:setVisible(false)
        self:performWithDelay(function()
            self.buffs:release()
            self._waypoints:release()
            self._enemies:release()
            self:stopAllActions()
            self:removeFromParentAndCleanup(true)
        end,2)
    end,
    })
end

function Attacker:getNextWayPoint()
    local lastWaypoint = self._waypoints:count()
    if self.curWaypoint == (lastWaypoint-1) then
        self.curWaypoint = 1
    else
        attacker.curWaypoint = attacker.curWaypoint + 1
    end
    local waypoint = tolua.cast(self._waypoints:objectAtIndex(self.curWaypoint),"WayPoint")
    return waypoint
end

--设置精灵显示控制
function Attacker:setSelfVisible(visible)
    self:setVisible(visible)
    self.bVisible = visible
end

--设置角色新的运动路径
function Attacker:setAttackerPath(path)
    if path ~= nil then
        self._waypoints:removeAllObjects()
        for i=0,path:count()-1 do
            local curWayPoint = tolua.cast(path:objectAtIndex(i), "WayPoint")
            local position = tolua.cast(curWayPoint.position,"CCPoint")
            local wp = WayPoint:new(position,curWayPoint.isFlipX,curWayPoint.direction,curWayPoint.underSea)
            self._waypoints:addObject(wp)
        end
    end
end

function Attacker:reverseAttackerPath(path)
    if path ~= nil then
        self._waypoints:removeAllObjects()
        for i=0,path:count()-1 do
            local curWayPoint = tolua.cast(path:objectAtIndex(path:count()-1-i), "WayPoint")
            local wp = WayPoint:new(curWayPoint.position,curWayPoint.isFlipX,curWayPoint.direction,curWayPoint.underSea)
            self._waypoints:addObject(wp)
        end
    end
end

function Attacker:stopCurrentAction()
    self.mainSprite:stopActionByTag(100)
end

--当前怪物受buff影响减速,不能叠加
function Attacker:slowDown(time,speedSlowDown)
    if self.ifSlowDown == true then
        return
    end
    local spriteTag = self:getChildByTag(100)
    if spriteTag~=nil then
        self.gameScene:performWithDelay(function()
            if tolua.isnull(self) then return end
            self:slowDown(time,speedSlowDown)
        end,1)
        return
    end
    self.ifSlowDown = true
    self.mainSprite:stopAllActions()
    self:stopWalk()
    self.move_speed = self.move_speed - self.attackerProperties.move_speed*math.abs(speedSlowDown)
    if self.move_speed<=0 then self.move_speed=0.1 end
    self.walk_animation_time = 1
    local waypoint = tolua.cast(self._waypoints:objectAtIndex(self.curWaypoint),"WayPoint")
    self:doWalkAnimation(waypoint.direction)
    self:startWalk()


    local frames,animation,offset = AnimationFactory.getBuffAnimation("slow")
    self.slowSprite = display.newSprite(frames[1])
    self.slowSprite:setPosition(0,offset)
    self.slowSprite:setScale(self.attackerProperties.scale)
    self.slowSprite:setTag(186)
    self:addChild(self.slowSprite)
    self.slowSprite:playAnimationForever(animation)

    local action = transition.sequence({
        CCDelayTime:create(time),
        CCCallFunc:create(function()
            self.mainSprite:stopAllActions()
            self:stopWalk()
            self.move_speed = self.move_speed + self.attackerProperties.move_speed*math.abs(speedSlowDown)
            if self.move_speed>self.attackerProperties.move_speed then self.move_speed=self.attackerProperties.move_speed end
            self.walk_animation_time = self.attackerProperties.walk_animation_time
            local waypoint = tolua.cast(self._waypoints:objectAtIndex(self.curWaypoint),"WayPoint")
            self:doWalkAnimation(waypoint.direction)
            self:startWalk()
            if self.ifPause then
                self:pause(false)
            end
            if pauseStatus then
                self:pause(true)
            end
            if pauseStatus then
                self:pause(true)
            end
            self.ifSlowDown = false
            local slowSprite = self:getChildByTag(186)
            self.slowSprite:removeSelf()
           
        end),
    })
    self.slowSprite:runAction(action)

end


function Attacker:addJumpEffect(effectName)
    local frames,animation,offset = AnimationFactory.getBuffAnimation(effectName)
    local sprite = display.newSprite(frames[1])
    sprite:setPosition(0,offset)
    sprite:setScale(self.attackerProperties.scale)
    self:addChild(sprite)
    transition.execute(sprite,CCAnimate:create(animation),{
    onComplete = function()
        if pauseStatus then
            self:pause(true)
        end
        sprite:removeSelf()
    end})
end

--添加眩晕效果
function Attacker:addDizzyEffect(delayTime,effectName)
    local spriteTag = self:getChildByTag(100)
    if spriteTag~=nil then
        spriteTag:removeFromParentAndCleanup(true)
        self:resume()
    end

    local frames,animation,offset = AnimationFactory.getBuffAnimation(effectName)
    local dizzySprite = display.newSprite(frames[1])
    dizzySprite:setPosition(0,offset)
    self:addChild(dizzySprite,-1)
    dizzySprite:setScale(self.attackerProperties.scale)
    dizzySprite:playAnimationForever(animation)
    dizzySprite:setTag(100)

    self:pause(false)
    self.gameScene:performWithDelay(function()
        --print("我睡醒了啊")
        if tolua.isnull(self) then return end
        local spriteTag = self:getChildByTag(100)
        if spriteTag~=nil then
            spriteTag:removeFromParentAndCleanup(true)
        end
        self:resume()
    end,delayTime)

    --[[
    self.ifSlowDown = true
    self.mainSprite:stopAllActions()
    self:stopWalk()
    self.move_speed = self.move_speed-self.attackerProperties.move_speed
    if self.move_speed<=0 then self.move_speed=0.1 end
    self.walk_animation_time = 2
    local waypoint = tolua.cast(self._waypoints:objectAtIndex(self.curWaypoint),"WayPoint")
    self:doWalkAnimation(waypoint.direction)
    self:startWalk()

    local frames,animation,offset = AnimationFactory.getBuffAnimation(effectName)
    local dizzySprite = display.newSprite(frames[1])
    dizzySprite:setPosition(0,offset)
    self:addChild(dizzySprite,-1)
    dizzySprite:setScale(self.attackerProperties.scale)
    dizzySprite:playAnimationForever(animation)
    dizzySprite:setTag(100)

    local action = transition.sequence({
        CCDelayTime:create(delayTime),
        CCCallFunc:create(function()
            self.mainSprite:stopAllActions()
            self:stopWalk()
            self.move_speed = self.move_speed + self.attackerProperties.move_speed
            if self.move_speed>self.attackerProperties.move_speed then self.move_speed=self.attackerProperties.move_speed end
            self.walk_animation_time = self.attackerProperties.walk_animation_time
            local waypoint = tolua.cast(self._waypoints:objectAtIndex(self.curWaypoint),"WayPoint")
            self:doWalkAnimation(waypoint.direction)
            self:startWalk()
            if self.ifPause then
                self:pause(false)
            end
            if pauseStatus then
                self:pause(true)
            end
            self.ifSlowDown = false
            dizzySprite:removeFromParentAndCleanup(true)
        end),
    })
    dizzySprite:runAction(action)]]
end


function Attacker:addArmorEffect(delayTime,effectName,buff)
    self.armor = self.attackerProperties.armor+self.attackerProperties.armor*buff:getValue()

    local frames,animation,offset = AnimationFactory.getBuffAnimation(effectName)
    local armorEffect = display.newSprite(frames[1])
    armorEffect:setPosition(0,offset)
    self:addChild(armorEffect,-1)
    armorEffect:setScale(self.attackerProperties.scale)
    armorEffect:playAnimationForever(animation)
    
    local action = transition.sequence({
        CCDelayTime:create(delayTime),
        CCCallFunc:create(function()
            self.armor = self.attackerProperties.armor
            self.buffs:removeObject(buff)
            if pauseStatus then
                self:pause(true)
            end
            armorEffect:removeSelf()
        end),
    })
    armorEffect:runAction(action) 
end

function Attacker:addResistEffect(delayTime,effectName,buff)
    self.resist = self.attackerProperties.resist+self.attackerProperties.resist*buff:getValue()
    
    local frames,animation,offset = AnimationFactory.getBuffAnimation(effectName)
    self.resistEffect = display.newSprite(frames[1])
    self.resistEffect:setPosition(0,offset)
    self:addChild(self.resistEffect,-1)
    self.resistEffect:setScale(self.attackerProperties.scale)
    self.resistEffect:playAnimationForever(animation)

    local action = transition.sequence({
        CCDelayTime:create(delayTime),
        CCCallFunc:create(function()
            self.resist = self.attackerProperties.resist
            self.buffs:removeObject(buff)
            self.resistEffect:removeFromParentAndCleanup(true)
            if pauseStatus then
                self:pause(true)
            end
        end),
    })
    self.resistEffect:runAction(action)
end

function Attacker:addGoBackEffect(effectName)
    local frames,animation,offset = AnimationFactory.getBuffAnimation(effectName)
    self.goBackEffect = display.newSprite(frames[1])
    self.goBackEffect:setPosition(0,offset)
    self:addChild(self.goBackEffect,-1)
    self.goBackEffect:setScale(self.attackerProperties.scale)
    self.goBackEffect:playAnimationForever(animation)
end

--提供给41009秀丽乾坤的接口，delayTime秒钟之后回到种buff的原地
function Attacker:goBackToEverPosition(delayTime)
    local posX,posY = self:getPosition()
    local wayPointIndex = self.curWaypoint

    local action = transition.sequence({
        CCDelayTime:create(delayTime),
        CCCallFunc:create(function()
            self.mainSprite:stopAllActions()
            self:stopWalk()
            local bezier = ccBezierConfig()
            bezier.controlPoint_1 = ccp(self:getPositionX(),self:getPositionY())
            bezier.controlPoint_2 = ccp((self:getPositionX()+posX)/2,posY+150)
            bezier.endPosition = ccp(posX,posY)
            local bezierTo = CCBezierTo:create(2,bezier)
            transition.execute(self,bezierTo,{
                onComplete = function()
                    local waypoint = tolua.cast(self._waypoints:objectAtIndex(self.curWaypoint),"WayPoint")
                    self:doWalkAnimation(waypoint.direction)
                    self.curWaypoint = wayPointIndex
                    self:startWalk()
                    if pauseStatus then
                        self:pause(true)
                    end
                end,
            })
            self.goBackEffect:removeFromParentAndCleanup(true)
        end),
    })
    self.goBackEffect:runAction(action)
end

--Dot是战斗力最高的英雄的攻击力*bufValue
--DOT_HP%是秒怪物满血状态的百分比
--1,DOT;2,护甲；3，移动速度；4，攻击速度；5.生命值上限；6，攻击力；7，暴击率；8，技能触发几率；9，眩晕；10，变形；11，抗性；12，牵引；13，石化；14，印记传送；15,DOT_HP%
--怪物的buff处理
function Attacker:addBuff(skillId,buff,decreasehp)
    self.buffs:addObject(buff)

    --buff.buffProperties.buff_time = 2
    --buff.buffProperties.buff_type = 3
    --buff.buffProperties.buff_value = -0.9
    
    local buffType = buff:getType()
    local buffEffect = buff:getBuffEffect()
    
    self.buffSkillIDs[#self.buffSkillIDs+1] = {skillId,buffType}
    scheduler.performWithDelayGlobal(function()
        if self~=nil and (not tolua.isnull(self) ) then
            self:removeBuffID(skillId,buffType)
            self.buffs:removeObject(buff)
        end
    end,buff:getTime())
    if buff:getType()==1 then
        if buff:getJumpTime()==nil then
            --print("没有jump次数")
        else
            local decreaseHP = decreasehp==nil and CommUtil.getLeadHeroPower()*math.abs(buff:getValue()) or decreasehp
            local jumpTimeCounter = 0
            local jumpMaxTimes = buff:getTime()/buff:getJumpTime()+ 1
            
            local doJump = function()
                jumpTimeCounter = jumpTimeCounter + 1
                if not tolua.isnull(self) then
                    self:addJumpEffect(buffEffect)
                    self:decreaseHpBySkill(decreaseHP,false)
                    --print("jump第",jumpTimeCounter,"次","掉血",decreaseHP)
                end
            end
            doJump()
            local onJump = function()
                if jumpTimeCounter < jumpMaxTimes then
                    doJump()
                end
            end
            local handle = scheduler.scheduleGlobal(onJump,buff:getJumpTime())
            scheduler.performWithDelayGlobal(function()
                scheduler.unscheduleGlobal(handle)
            end,buff:getTime()+3)
        end
    elseif buff:getType()==2 then--缺特效
        buff:setMonsterFlag(true)
        self:addArmorEffect(buff:getTime(),buffEffect,buff)
    elseif buff:getType()==3 then
        self:slowDown(buff:getTime(),buff:getValue())
    elseif buff:getType()==4 then
        
    elseif buff:getType()==5 then

    elseif buff:getType()==6 then

    elseif buff:getType()==7 then

    elseif buff:getType()==8 then
        
    elseif buff:getType()==9 then
        self:addDizzyEffect(buff:getTime(),buffEffect)
    elseif buff:getType()==11 then--缺特效
        buff:setMonsterFlag(true)
        self:addResistEffect(buff:getTime(),buffEffect,buff)
    elseif buff:getType()==12 then--缺特效

    elseif buff:getType()==13 then--缺特效
        self:addDizzyEffect(buff:getTime(),buffEffect)
    elseif buff:getType()==15 then
        if buff:getJumpTime()==nil then
            --print("没有jump次数")
        else
            local decreaseHP = math.abs(self.fullHP*buff:getValue())
            local jumpTimeCounter = 0
            local jumpMaxTimes = buff:getTime()/buff:getJumpTime()+ 1
            
            local doJump = function()
                jumpTimeCounter = jumpTimeCounter + 1
                if not tolua.isnull(self) then
                    self:addJumpEffect(buffEffect)
                    self:decreaseHpBySkill(decreaseHP,false)
                    --print("jump第",jumpTimeCounter,"次","掉血",decreaseHP)
                end
            end

            doJump()

            local onJump = function()
                if jumpTimeCounter < jumpMaxTimes then
                    doJump()
                end
            end
            
            local handle = scheduler.scheduleGlobal(onJump,buff:getJumpTime())
            scheduler.performWithDelayGlobal(function()
                scheduler.unscheduleGlobal(handle)
            end,buff:getTime()+3)
        end
    end
     print("add buff type " .. buff:getType())
    if buff:getType()==16 then
        self:addGoBackEffect(buffEffect)
        self:goBackToEverPosition(buff:getTime())
    end
end

function Attacker:doHeroJianFangBuff(delayTime)
    for i=0,GameInstance:sharedInstance().towers:count()-1 do
        local tower = GameInstance:sharedInstance().towers:objectAtIndex(i)
        tower:addJianFangEffect(delayTime)
    end
end


function Attacker:getBuffs()
    return self.buffs
end

function Attacker:removeBuffID(skillId,bType)
    for i=1,#self.buffSkillIDs do
        if (self.buffSkillIDs[i][1].."")==(skillId.."") and self.buffSkillIDs[i][2]==bType then
            table.remove(self.buffSkillIDs, i)
            return
        end
    end
end

function Attacker:ifAddBuff(skillId,bType)
    for i=1,#self.buffSkillIDs do
        if (self.buffSkillIDs[i][1].."")==(skillId.."") and self.buffSkillIDs[i][2]==bType then
            return true
        end
    end
    return false
end


--提供给通天塔杀技能的接口，判断怪物是否被杀死
function Attacker:ifCanKill()
    if self.attackerProperties.elite<=7 then
        return true
    else
        return false
    end
end

function Attacker:pause(ifStopWalkAnimation)
    self.ifPause = true
    transition.pauseTarget(self)
    if ifStopWalkAnimation then
        transition.pauseTarget(self.mainSprite)
        --[[
        if self.underground then
            transition.pauseTarget(self.undergroundSprite)
        end]]
    end
end

function Attacker:resume()
    local spriteTag = self:getChildByTag(100)
    if spriteTag~=nil then
        return
    end
    self.ifPause = false
    transition.resumeTarget(self)
    transition.resumeTarget(self.mainSprite)
    --[[
    if self.underground then
        transition.resumeTarget(self.undergroundSprite)
    end]]
end

function Attacker:setRect(rect)
    self.rect = rect
end

function Attacker:getOriginPointAndRange(posX,posY)
    local size = self.mainSprite:getCascadeBoundingBox().size
    local anchorPoint = self.mainSprite:getAnchorPoint()
    local textureCenterX = posX + (0.5-anchorPoint.x)*size.width
    local textureCenterY = posY + (0.5-anchorPoint.y)*size.height
    local range = 100
    return textureCenterX-range,textureCenterY-range,range*2
end

function Attacker:getRoleName()
    return "monster"
end

function Attacker:getRectAndOffset()
    local monsterRect = self.mainSprite:getBoundingBox()
    local anchorPoint = self.mainSprite:getAnchorPoint()
    local size = monsterRect.size
    local posX = self:getPositionX() + (0.5-anchorPoint.x)*size.width
    local posY = self:getPositionY() + (0.5-anchorPoint.y)*size.height
    local offset = 0
    local newRect = CCRect(posX-size.width/2+offset,posY-size.height/2+offset,size.width-offset*2,size.height-offset*2)
    return newRect,offset,ccp(posX,posY),size
end

function Attacker:getChildCatPathAndCurWayPoint()
    local paths = GameInstance:sharedInstance().paths
    for i=0,paths:count()-1 do
        local path = tolua.cast(GameInstance:sharedInstance().paths:objectAtIndex(i),"CCArray")
        for j=0,path:count()-1 do
            local curWayPoint = tolua.cast(path:objectAtIndex(j), "WayPoint")
            if ccpDistance(ccp(curWayPoint.position.x,curWayPoint.position.y),ccp(self:getPositionX(),self:getPositionY()))<20 then
                return path,j
            end
        end
    end
end

function Attacker:stopWalk()
    self:stopActionByTag(99)
    self.walkStatus = false
end

function Attacker:startWalk()
    self:walk()
    self.walkStatus = true
end

function Attacker:isBigBoss()
   return false
end

function Attacker:cleanBubble()
    if self.bubble==nil then return end
    self.bubble:removeFromParentAndCleanup(true)
    self.bubble = nil 
end

function Attacker:addBubbleAndWords(word)
    self:cleanBubble()
    self.bubble = display.newSprite("ui/public/public_dialog_box.png",60,100)
    local scale = 1
    if self.attackerProperties.scale<0.3 then
        scale = 1.3
    end
    self.bubble:setScale(scale)
    self:addChild(self.bubble,1000)
    self.words = ui.newTTFLabel({
                text = ( word==nil and self.npcProperties.words[math.random(1,#self.npcProperties.words)] or word ),
                font = "Arial",
                color = ccc3(0, 0, 0),
                x = -40,
                y = 10,
                size = 14,
                dimensions = CCSize(135,120),
                align = ui.TEXT_ALIGN_LEFT
    })
    self.words:setAnchorPoint(ccp(0,0))
    self.bubble:addChild(self.words)
    self:performWithDelay(function()
        self:cleanBubble()
    end,2)
end

function Attacker:addFocusFlag()
    local frames,animation = AnimationFactory.getFocusAnimation()
    self.focusFlag = display.newSprite(frames[1],-15,-20):addTo(self.mainSprite,-1)
    self.focusFlag:setAnchorPoint(ccp(0,0))
    self.focusFlag:setScale(1)
    self.focusFlag:playAnimationForever(animation)
end

function Attacker:cleanFocusFlag()
    if self.focusFlag then
        self.focusFlag:removeSelf()
    end
end

function  Attacker:isWhite()
    return false
end

return Attacker