local Text = require("app.ui.Text")
local MathUtil = require("app.common.MathUtil")
local AnimationFactory = require("app.factory.AnimationFactory")

local NPC = class("NPC",function(gameScene,npcType,path)
	local node = display.newNode()
	return node
end)

function NPC:ctor(gameScene,npcProperties,path,ifWalk,frontorBack,ifGirl,statyAtEnd)
	--self:setAnchorPoint(CCPoint(0.5,1))
	self.gameScene = gameScene

	self.npcProperties = totable(npcProperties)

	self.bubble = nil
	
	self:setPosition(self.npcProperties.position)
	self:setScale(self.npcProperties.scale)
	
	self.state="walk"
	
	--定义主精灵类
    self.mainSprite = display.newSprite("#"..self.npcProperties.body_id.."_walk_front_1.png"):addTo(self)
    self.mainSprite:setAnchorPoint(CCPoint(0.5,0))
    
    self._waypoints = CCArray:create()--怪物需要行走的路径
    self._waypoints:retain()
    self.wayOffset = ccp(0,0)--角色的运动路径偏移量
    
	self.curWaypoint = 0
	self.fVelocityRatio = 1.0
    
    self:setNPCPath(path)
    
    self.coolDown = false
    self.addDingMoney = false
    self.addDingGift = false

    --self:schedule(handler(self, self.updateNPCZorder),1)

    self.ifWalk = ifWalk
    self.ifGirl = ifGirl
    self.statyAtEnd = statyAtEnd
    local waypoint = tolua.cast(self._waypoints:objectAtIndex(self.curWaypoint),"WayPoint")
    if ifWalk then
    	self:doWalkAnimation(waypoint.direction)
    else
    	self:doStandAnimation(frontorBack)
    end

    self.geting = false

end

function NPC:updateNPCZorder()
    local x,y,size = self:getTextureCenterPointInWorld(self:getPositionX(),self:getPositionY())
    local zorder = GameInstance:sharedInstance():zorderForPosition(ccp(x,y))
    self.gameScene:reorderChild(self,zorder)
end

function NPC:getOffset()
	return self.npcProperties.offset
end

function NPC:ifCanGiveMoney()
	return ( self.moneySprite~=nil and (not self.geting) ) and true or false
end

function NPC:addMoneySprite()
	addRes("NPCScene","glod.plist","glod.pvr.ccz")
	if self.moneyCircleSprite==nil then
		self.moneyCircleSprite = display.newSprite("ui/home/home_goods_hint.png",0,100):addTo(self)
		self.moneyCircleSprite:runAction(CCRepeatForever:create(CCRotateBy:create(3,360)))
	end

	if self.moneySprite==nil then
		self.moneySprite = display.newSprite("#gold_1.png",0,150):addTo(self)
	end
	self.state="giveMoney"
	self.addDingMoney = true
	self:performWithDelay(function()
		self.addDingMoney = false
	end,20)
	self.moneyCircleSprite:setScale(1)
	self.moneySprite:setScale(1.3)
end

function NPC:removeMoneySprite()
	if self.geting then return end
	self.geting = true
	addRes("NPCScene","glod.plist","glod.pvr.ccz")
	local frames,animation = AnimationFactory.getGoldAnimation()
	CommUtil.playSound("music/gold.mp3")
	self.moneyCircleSprite:removeFromParentAndCleanup(true)
	self.moneyCircleSprite = nil
    transition.execute(self.moneySprite,CCAnimate:create(animation),{
    	onComplete = function()
    		self.geting = false
        	self.moneySprite:removeFromParentAndCleanup(true)
			self.moneySprite = nil
			self.state="walk"
        end,
    })
end

function NPC:addGiftSprite()
	if self.giftCircleSprite==nil then
		self.giftCircleSprite = display.newSprite("ui/home/home_goods_hint.png",0,100):addTo(self)
		self.giftCircleSprite:runAction(CCRepeatForever:create(CCRotateBy:create(3,360)))
	end
	if self.giftSprite==nil then
		self.giftSprite = display.newSprite("ui/home/home_cat_yykx.png",0,110):addTo(self)
	end
	self.addDingGift = true
	self:performWithDelay(function()
		self.addDingGift = false
	end,10)
	self.giftCircleSprite:setScale(1)
	self.giftSprite:setScale(1.3)
end

function NPC:removeGiftSprite()
	self.giftSprite:removeFromParentAndCleanup(true)
	self.giftCircleSprite:removeFromParentAndCleanup(true)
	self.giftSprite = nil
	self.giftCircleSprite = nil
	self.state="walk"
end

function NPC:ifCanGiveGift()
	return self.giftSprite~=nil and true or false
end

function NPC:getCoolDownTime()
	return self.npcProperties.coolDownTime
end

--获取贴图中心点在世界坐标的位置
function NPC:getTextureCenterPointInWorld(posX,posY)
    local size = self.mainSprite:getCascadeBoundingBox().size
    local anchorPoint = self.mainSprite:getAnchorPoint()
    local textureCenterX = posX + (0.5-anchorPoint.x)*size.width
    local textureCenterY = posY + (0.5-anchorPoint.y)*size.height
    return textureCenterX,textureCenterY,size
end

function NPC:doWalkAnimation(direction)
	self.mainSprite:stopAllActions()
	transition.execute(self.mainSprite, CCAnimate:create(AnimationFactory.getNpcAnimation(direction,self.npcProperties.body_id,self.npcProperties.walk_animation_time)), {
        onComplete = function()
            local waypoint = tolua.cast(self._waypoints:objectAtIndex(self.curWaypoint),"WayPoint")
            self:doWalkAnimation(waypoint.direction)
        end,
    })
end

function NPC:doRunAnimation(direction)
	self.mainSprite:stopAllActions()
	transition.execute(self.mainSprite, CCAnimate:create(AnimationFactory.getNpcRunAnimation(direction,self.npcProperties.body_id,self.npcProperties.walk_animation_time)), {
        onComplete = function()
            local waypoint = tolua.cast(self._waypoints:objectAtIndex(self.curWaypoint),"WayPoint")
            self:doRunAnimation(waypoint.direction)
        end,
    })
end

function NPC:changeWalkAnimation(direction)
	self.mainSprite:stopAllActions()
	transition.execute(self.mainSprite, CCAnimate:create(AnimationFactory.getNpcAnimation(direction,self.npcProperties.body_id,self.npcProperties.walk_animation_time)), {
        onComplete = function()
            self:changeWalkAnimation(direction)
        end,
    })
end

function NPC:doStandAnimation(direction)
	self.mainSprite:stopAllActions()
	transition.execute(self.mainSprite, CCAnimate:create(AnimationFactory.getNpcStandAnimation(direction,self.npcProperties.body_id,self.npcProperties.walk_animation_time)), {
        onComplete = function()
            self:doStandAnimation(direction)
        end,
    })
end

function NPC:getBodyId()
	return self.npcProperties.body_id
end

function NPC:clean()
	if self.statyAtEnd then
		self:stopAllActions()
		transition.execute(self,CCMoveTo:create(2,ccp(self.gameScene.beauty:getPositionX(),self.gameScene.beauty:getPositionY())),{
        onComplete = function()
        	print("猫叔躲在妖姬背后")
            self.mainSprite:setFlipX(true)
            self.mainSprite:stopAllActions()
			self:doWalkAnimation(true)
        end,
    	})
		return 
	end
	self:stopAllActions()
	if not self.ifWalk then
		if self.ifGirl then
			local action = transition.sequence({
		        CCMoveBy:create(1, ccp(30, 30)),
		        CCCallFunc:create(function()
		          	local sprite = display.newSprite("ui/guide/guide_words_11.png",self:getPositionX()-20,self:getPositionY()+80):addTo(display.getRunningScene(),60001)
					sprite:setOpacity(0)
					local action = transition.sequence({
					        CCFadeIn:create(0.3),
					        CCDelayTime:create(0.5),
					        CCCallFunc:create(function()
					        	self.gameScene:pauseGmaeWhenGirlReachEnd()
					          	sprite:removeSelf()
					        end),
					})
					sprite:runAction(action)
					self:doStandAnimation(true)
					self.mainSprite:setFlipX(true)
		        end),
		  	})
		  	self:runAction(action)
		end
		return
	end
	for k, v in ipairs(self.gameScene.npcs) do
		if v==self then
			table.remove(self.gameScene.npcs, k)
			self._waypoints:release()
			self:removeFromParentAndCleanup(true)
		end
	end
end

function NPC:walk()

	local lastWaypoint = self._waypoints:count()
    if self.curWaypoint == (lastWaypoint-1) then
		self:clean()
		return
	end

	--取得下一个要到达地点的位置
	local lastWaypoint = self._waypoints:count()
	if self.curWaypoint == (lastWaypoint-1) then
		self.curWaypoint = 1
	end
    
    local currentPosition = ccp(self:getPositionX(),self:getPositionY())
    local waypoint = tolua.cast(self._waypoints:objectAtIndex(self.curWaypoint+1),"WayPoint")
    
    local destPosition = ccp(CommUtil.getBlankWidth() + waypoint.position.x,waypoint.position.y-self:getOffset())
    
    --计算时间采用设计分辨率800x480
    local dist = ccpDistance(destPosition,ccp(self:getPositionX(),self:getPositionY()))
    local runTime = dist/self.npcProperties.move_speed
    
    --[[
    if dist > 0 then
        runTime = dist/self.npcProperties.move_speed *(1 + math.abs(destPosition.y-self:getPositionY())/dist)
    end]]

    self.mainSprite:setFlipX(waypoint.isFlipX)

    transition.execute(self,CCMoveTo:create(runTime,destPosition),{
    	onComplete = function()
        	self.curWaypoint = self.curWaypoint + 1
            self:walk()
        end,
    })
end

function NPC:startWalk()
	self.mainSprite:stopAllActions()
	self:doWalkAnimation(true)
	self:walk()
end


--设置角色新的运动路径
function NPC:setNPCPath(newWayPoints)
	if newWayPoints ~= nil then
		self._waypoints:removeAllObjects()
		for i=0,newWayPoints:count()-1 do
			local curWayPoint = tolua.cast(newWayPoints:objectAtIndex(i), "WayPoint")
			tolua.cast(curWayPoint.position,"CCPoint")
			local wp = WayPoint:new(curWayPoint.position,curWayPoint.isFlipX,curWayPoint.direction,curWayPoint.underSea)
			self._waypoints:addObject(wp)
		end
	end
end

function NPC:stopCurrentAction()
	self.mainSprite:stopActionByTag(100)
end

function NPC:stopCurrentMoveAction()
	if self.bIsMove == true then
        self.bIsMove = false
	end
    self.mainSprite:stopActionByTag(401)
end

function NPC:cleanBubble()
	if self.bubble==nil then return end
	self.bubble:removeFromParentAndCleanup(true)
	self.bubble = nil 
end

function NPC:addBubbleAndWords(word)
	self:cleanBubble()
	self.state = "talk"
	self.bubble = display.newSprite("ui/public/public_dialog_box.png",110,140)
	self.bubble:setScale(1.7)
	self:addChild(self.bubble,1000)
	self.words = ui.newTTFLabel({
		    	text = ( word==nil and self.npcProperties.words[math.random(1,#self.npcProperties.words)] or word ),
		    	font = "Arial",
		    	color = ccc3(0, 0, 0),
		    	x = -40,
		   		y = 10,
		    	size = 14,
		    	dimensions = CCSize(135,120),
		    	align = ui.TEXT_ALIGN_LEFT
	})
	self.words:setAnchorPoint(ccp(0,0))
	self.bubble:addChild(self.words)
	self:performWithDelay(function()
		self:cleanBubble()
	end,2)
end

function NPC:aGoodWeather()
	transition.moveBy(self,{x=-50,y=2,time=2.5,onComplete=function()
		self.mainSprite:setFlipX(false)
		transition.moveBy(self,{x=50,y=-2,time=2.5,onComplete=function()
			self.mainSprite:setFlipX(true)
			self:aGoodWeather()
		end})
	end})
end

function NPC:leftToRightCycle()
	self.gameScene:reorderChild(self,60000)
	transition.moveBy(self,{x=50,y=-2,time=4,onComplete=function()
		self.mainSprite:setFlipX(true)
		transition.moveBy(self,{x=-50,y=2,time=4,onComplete=function()
			self.mainSprite:setFlipX(false)
			self:leftToRightCycle()
		end})
	end})
end

function NPC:runToDestionation()
	self.npcProperties.move_speed = 60
	self.mainSprite:stopAllActions()
	self:stopAllActions()
	local path = tolua.cast(GameInstance:sharedInstance().paths:objectAtIndex((GameInstance:sharedInstance().paths:count()-1)),"CCArray")
	local spawnPoint = tolua.cast(path:objectAtIndex(1),"WayPoint")
	local xPos = spawnPoint.position.x + CommUtil.getBlankWidth()
    local yPos = spawnPoint.position.y
    self.mainSprite:setFlipX(false)
    self.mainSprite:playAnimationForever(AnimationFactory.getNpcRunAnimation(true,self.npcProperties.body_id,self.npcProperties.walk_animation_time))

    local action = transition.sequence({
    	CCMoveTo:create(3,ccp(xPos,yPos)),
        CCDelayTime:create(0.5),
        CCCallFunc:create(function()
            self:cleanBubble()
            self.mainSprite:stopAllActions()
            local imageName = "npc_1026_jump_front_%d.png"
	        local frames = display.newFrames(imageName, 1,8)
	        local animation = display.newAnimation(frames, 1 / 8)
            self.mainSprite:playAnimationForever(animation)
            self.mainSprite:setFlipX(true)
            self:leftToRightCycle()
        end),
    })

    self:runAction(action)

	--self:doRunAnimation(true)
	--self:walk()
	--print("npc的事件是",self.npcProperties.move_speed)
end

function NPC:pause()
    transition.pauseTarget(self)
    transition.pauseTarget(self.mainSprite)
end

function NPC:resume()
    transition.resumeTarget(self)
    transition.resumeTarget(self.mainSprite)
end

return NPC