local MathUtil = require("app.common.MathUtil")
local Attacker = require("app.actor.Attacker")
local AnimationFactory = require("app.factory.AnimationFactory")

local SwimMonster = class("SwimMonster",Attacker)

function SwimMonster:ctor(attackerProperties,parent,path)
    SwimMonster.super.ctor(self,attackerProperties,parent,path)
    local waypoint = tolua.cast(self._waypoints:objectAtIndex(self.curWaypoint),"WayPoint")
    if waypoint.underSea then
        self:doWalkAnimation(true,waypoint.direction)
    else
        self:doWalkAnimation(false,waypoint.direction)
    end
end

function SwimMonster:doWalkAnimation(underSea,direction)
    self.mainSprite:stopAllActions()
    local animation = AnimationFactory.getSwimMonsterAnimation(underSea,direction,self.attackerProperties.body_id,self.attackerProperties.walk_animation_time)
    if animation~=nil then 
        transition.execute(self.mainSprite, CCAnimate:create(animation), {
            onComplete = function()
                local wayPointCount = self._waypoints:count()
                local waypoint = tolua.cast(self._waypoints:objectAtIndex((self.curWaypoint+1>wayPointCount-1) and (wayPointCount-1) or (self.curWaypoint+1) ),"WayPoint")
                if waypoint.underSea then
                    self:doWalkAnimation(true,waypoint.direction)
                else
                    self:doWalkAnimation(false,waypoint.direction)
                end
            end,
        })
    end
end

--添加眩晕效果
function SwimMonster:addDizzyEffect(delayTime,effectName)
    local spriteTag = self:getChildByTag(100)
    if spriteTag~=nil then
        spriteTag:removeFromParentAndCleanup(true)
    end
    self.ifSlowDown = true

    self.mainSprite:stopAllActions()
    self:stopWalk()
    
    self.move_speed = 1
    self.walk_animation_time = 2
    local waypoint = tolua.cast(self._waypoints:objectAtIndex(self.curWaypoint),"WayPoint")
    if waypoint.underSea then
        self:doWalkAnimation(true,waypoint.direction)
    else
        self:doWalkAnimation(false,waypoint.direction)
    end
    self:startWalk()

    local frames,animation,offset = AnimationFactory.getBuffAnimation(effectName)
    self.dizzySprite = display.newSprite(frames[1])
    self.dizzySprite:setPosition(0,offset)
    self:addChild(self.dizzySprite,-1)
    self.dizzySprite:setScale(self.attackerProperties.scale)
    self.dizzySprite:playAnimationForever(animation)
    self.dizzySprite:setTag(100)

    local action = transition.sequence({
        CCDelayTime:create(delayTime),
        CCCallFunc:create(function()
            self.ifSlowDown = false
            self.dizzySprite:removeFromParentAndCleanup(true)
            self.mainSprite:stopAllActions()
            self:stopWalk()
            self.move_speed = self.attackerProperties.move_speed
            self.walk_animation_time = self.attackerProperties.walk_animation_time
            local waypoint = tolua.cast(self._waypoints:objectAtIndex(self.curWaypoint),"WayPoint")
            if waypoint.underSea then
                self:doWalkAnimation(true,waypoint.direction)
            else
                self:doWalkAnimation(false,waypoint.direction)
            end
            self:startWalk()
            if self.ifPause then
                self:pause(false)
            end
            if pauseStatus then
                self:pause(true)
            end
        end),
    })
    self.dizzySprite:runAction(action) 
end

--当前怪物受buff影响减速,不能叠加
function SwimMonster:slowDown(time,speedSlowDown)
    if self.ifSlowDown == true then
        return
    end
    self.ifSlowDown = true
    self.mainSprite:stopAllActions()
    self:stopWalk()
    self.move_speed = self.attackerProperties.move_speed + self.attackerProperties.move_speed*speedSlowDown
    self.walk_animation_time = 1
    local waypoint = tolua.cast(self._waypoints:objectAtIndex(self.curWaypoint),"WayPoint")
    if waypoint.underSea then
        self:doWalkAnimation(true,waypoint.direction)
    else
        self:doWalkAnimation(false,waypoint.direction)
    end
    self:startWalk()


    local frames,animation,offset = AnimationFactory.getBuffAnimation("slow")
    self.slowSprite = display.newSprite(frames[1])
    self.slowSprite:setPosition(0,offset)
    self.slowSprite:setScale(self.attackerProperties.scale)
    self.slowSprite:setTag(186)
    self:addChild(self.slowSprite)
    self.slowSprite:playAnimationForever(animation)

    local action = transition.sequence({
        CCDelayTime:create(time),
        CCCallFunc:create(function()
            self.mainSprite:stopAllActions()
            self:stopWalk()
            self.move_speed = self.attackerProperties.move_speed
            self.walk_animation_time = self.attackerProperties.walk_animation_time
            local waypoint = tolua.cast(self._waypoints:objectAtIndex(self.curWaypoint),"WayPoint")
            if waypoint.underSea then
                self:doWalkAnimation(true,waypoint.direction)
            else
                self:doWalkAnimation(false,waypoint.direction)
            end
            self:startWalk()
            if self.ifPause then
                self:pause(false)
            end
            if pauseStatus then
                self:pause(true)
            end
            local spriteTag = self:getChildByTag(100)
            if spriteTag~=nil then
                transition.pauseTarget(self)
            end
            if pauseStatus then
                self:pause(true)
            end
            self.ifSlowDown = false
            local slowSprite = self:getChildByTag(186)
            self.slowSprite:removeSelf()
        end),
    })
    self.slowSprite:runAction(action)
end

--提供给41009秀丽乾坤的接口，delayTime秒钟之后回到种buff的原地
function SwimMonster:goBackToEverPosition(delayTime)
    local posX,posY = self:getPosition()
    local wayPointIndex = self.curWaypoint

    local action = transition.sequence({
        CCDelayTime:create(delayTime),
        CCCallFunc:create(function()
            self.mainSprite:stopAllActions()
            self:stopWalk()
            local bezier = ccBezierConfig()
            bezier.controlPoint_1 = ccp(self:getPositionX(),self:getPositionY())
            bezier.controlPoint_2 = ccp((self:getPositionX()+posX)/2,posY+150)
            bezier.endPosition = ccp(posX,posY)
            local bezierTo = CCBezierTo:create(1, bezier)

            transition.execute(self,bezierTo,{
                onComplete = function()
                    local waypoint = tolua.cast(self._waypoints:objectAtIndex(self.curWaypoint),"WayPoint")
                    if waypoint.underSea then
                        self:doWalkAnimation(true,waypoint.direction)
                    else
                        self:doWalkAnimation(false,waypoint.direction)
                    end
                    self.curWaypoint = wayPointIndex
                    self:startWalk()
                    if pauseStatus then
                        self:pause(true)
                    end
                end,
            })
            self.goBackEffect:removeFromParentAndCleanup(true)
        end),
    })
    self.goBackEffect:runAction(action)
end

function SwimMonster:isBigBoss()
   return false
end

return SwimMonster