local MathUtil = require("app.common.MathUtil")
local Attacker = require("app.actor.Attacker")
local AnimationFactory = require("app.factory.AnimationFactory")

local Thief = class("Thief",Attacker)

function Thief:ctor(attackerProperties,parent,path)
    Thief.super.ctor(self,attackerProperties,parent,path)
    
    local thiefFlag = display.newSprite("ui/guide/guide_hint_1.png",0,self.attackerProperties.blood_offset):addTo(self)
    local sequence = transition.sequence({
            CCMoveBy:create(0.3, ccp(0, 10)),
            CCMoveBy:create(0.3, ccp(0, -10)),
    })
    thiefFlag:runAction(CCRepeatForever:create(sequence))

    self.attackerProperties.hit_proc = self.attackerProperties.hit_proc + 0.4

end

function Thief:get_hit_proc()
    return self.attackerProperties.hit_proc + 0.4
end

function Thief:walk()
    if self.dead then
        return
    end
    if self.fCurLifeValue <= 0 then
        self:clean()
        return
    end
    
    local waypoint1 = tolua.cast(self._waypoints:objectAtIndex(self.curWaypoint),"WayPoint")
    local waypoint2 = tolua.cast(GameInstance:sharedInstance().beautyPoint,"WayPoint")

    if ccpDistance(waypoint1.position,waypoint2.position)<100 then
        self.gameScene:helpBeauty(2)
    end

    local wayPointCount = self._waypoints:count()

    if self.curWaypoint == (wayPointCount-1) then
        --print("最后一个点了")
        self:setSelfVisible(false)
        if self.fCurLifeValue>0 then
            --播放怪物逃跑的音效
            killThiefNum = -1000
            self:getParent().gameHud:updateBeautyBlood()
        end
        self:clean()
        return
    end

    --取得下一个要到达地点的位置
    local currentPosition = ccp(self:getPositionX(),self:getPositionY())
    local waypoint = tolua.cast(self._waypoints:objectAtIndex(self.curWaypoint+1),"WayPoint")
    
    local destPosition = ccp(CommUtil.getBlankWidth() + waypoint.position.x,waypoint.position.y-self.offset)
    
    --计算时间采用设计分辨率800x480
    local dist = ccpDistance(destPosition,ccp(self:getPositionX(),self:getPositionY()))
    local runTime = dist/self.move_speed

    if dist > 0 then
        runTime = dist/self.move_speed *(1 + math.abs(destPosition.y-self:getPositionY())/dist)
    end
    if self.reverse then
        self.mainSprite:setFlipX(not waypoint.isFlipX)
    else
        self.mainSprite:setFlipX(waypoint.isFlipX)
    end
    
    local action = CCSequence:createWithTwoActions(
            CCMoveTo:create(runTime,destPosition), 
            CCCallFunc:create(function()
                self.curWaypoint = self.curWaypoint + 1
                self:walk()
    end))
    action:setTag(99)
    self:runAction(action)
end

function  Thief:runDeadAnimation()
    killThiefNum = killThiefNum + 1
    Thief.super.runDeadAnimation(self)
end

function Thief:doWalkAnimation(direction)
    self.mainSprite:stopAllActions()
    local animation = AnimationFactory.getMonsterAnimation(direction,self.attackerProperties.body_id,self.attackerProperties.walk_animation_time)
    if animation~=nil then 
        transition.execute(self.mainSprite, CCAnimate:create(animation), {
            onComplete = function()
                local wayPointCount = self._waypoints:count()
                local waypoint = tolua.cast(self._waypoints:objectAtIndex((self.curWaypoint+1>wayPointCount-1) and (wayPointCount-1) or (self.curWaypoint+1) ),"WayPoint")
                if waypoint.underSea then
                    self:doWalkAnimation(waypoint.direction)
                else
                    self:doWalkAnimation(waypoint.direction)
                end
            end,
        })
    end
end


return Thief