require("config")
local MathUtil = require("app.common.MathUtil")
local UserDefault = require("app.common.UserDefault")
local HeroSkill = require("app.bullet.HeroSkill")
local AnimationFactory = require("app.factory.AnimationFactory")
local DebugUtil = require("app.common.DebugUtil")
local BulletFactory = require("app.factory.BulletFactory")
local CommUtil = require("app.common.CommUtil")
local scheduler = require(cc.PACKAGE_NAME .. ".scheduler")
local HeroGroup = require("app.bullet.HeroGroup")

local Tower= class("Tower",function(properties,args,gameScene)
    local node = display.newNode()
    --node:setNodeEventEnabled(true)
    --require("framework.api.EventProtocol").extend(node)
    return node
end)

function Tower:ctor(properties,args,gameScene)
    --if true then return end
    self.buffSkillIDs = {}
    self.buffs = CCArray:create()--给英雄加的buff
    self.buffs:retain()
    self.towerEffect = {}

    self.properties = totable(properties)
    self.args = totable(args)
    self.towerInfo = self.properties.towerInfo

    self:initBattleProperties()

    self.designX = self.properties.designX
    self.designY = self.properties.designY
    
    self.cooldown = false--第一次默认是可以攻击的，之后根据各种塔和英雄的攻击时间设定攻击频率
    self.gameScene = gameScene

    --print("塔的位置和朝向，决定使用那张地图",self.properties.direction)
    self:addTowerBase()
    self.towerView = display.newSprite():addTo(self)
    self:addTowerFlag()
    
    --buff效果标志位
    self.jianFangFlag = false
    self.skillOverFlag = true--技能是否释放完毕的标志

    self.target = nil
    self.gridIndexs = {}
    self.damaged = false
    self.ifHelper = false
    self.frozened = false
    self.enhanceDamageDegree = 1

    self.forceSkill = false

    self.heroGroups = CCArray:create()
    self.heroGroups:retain()

    self.angerNum = 0
    self.upgradeTimes = 0
    self:initAnger()

end

function Tower:initAnger()
    self.angerBg = display.newProgressTimer("ui/fighting/anger_bg.png",display.PROGRESS_TIMER_BAR):addTo(self.towerBase)
    self.angerBg:setMidpoint(CCPointMake(0, 0))
    self.angerBg:setBarChangeRate(CCPointMake(1, 0))
    self.angerBg:setPosition(CCPointMake(40,13))
    self.angerBg:setPercentage(100)

    self.angerPer = display.newProgressTimer("ui/fighting/anger_per.png",display.PROGRESS_TIMER_BAR):addTo(self.towerBase)
    self.angerPer:setMidpoint(CCPointMake(0, 0))
    self.angerPer:setBarChangeRate(CCPointMake(1, 0))
    self.angerPer:setPosition(CCPointMake(40,13))

    self.angerPer:setPercentage(0)

    self.angerBg:setVisible(false)
    self.angerPer:setVisible(false)

end

function Tower:ifCanUpgrade()
    if self.heroId~=10001 and self.heroId~=10002 then
        return true
    else
        return false
    end
end

--升级塔
function Tower:upgradeTower()
    self.upgradeTimes = self.upgradeTimes + 1
    if self.upgradeTimes>3 then self.upgradeTimes = 3 end
end

function Tower:addRelease()
end


function Tower:addAnger(angerNum)
    self.angerNum = self.angerNum + angerNum
    if self.angerNum>=100 then
        self.angerNum=100
        if tolua.isnull(self.releaseSprite) then
             local frames = display.newFrames("fighting_light_%d.png",1,4)
            local animation = display.newAnimation(frames,1/4)
            self.releaseSprite = display.newSprite(frames[1],-25,-15):addTo(self.hero)
            self.releaseSprite:setAnchorPoint(ccp(0,0))
            self.releaseSprite:playAnimationForever(animation)
        end
    end
    --print("anger",self.angerNum)
    self.angerPer:setPercentage(self.angerNum)
end

function Tower:resetAnger()
    self.releaseSprite:removeFromParentAndCleanup(true)
    self.angerNum = 0
    self.angerPer:setPercentage(self.angerNum)
    --print("anger",self.angerNum)
end

function Tower:releaseSkill()
    local targetMonster = self.gameScene.gameLogicLayer:getTargetForTower(self)
    if targetMonster~=nil then
        local frontOrBack = nil
        local leftOrRight = nil
        if targetMonster:getPositionX()<self:getPositionX() then
            if targetMonster:getPositionY()<self:getPositionY() then
                frontOrBack = "front"
                leftOrRight = "l"
            else
                frontOrBack = "back"
                leftOrRight = "l"
            end
        else
            if targetMonster:getPositionY()<self:getPositionY() then
                frontOrBack = "front"
                leftOrRight = "r"
            else
                frontOrBack = "back"
                leftOrRight = "r"
            end
        end
        local imageName = "hero_"..self:getHeroId().."_".."attack".."_"..frontOrBack.."_1.png"
        local spriteFrame = display.newSpriteFrame(imageName)
        self.hero:setDisplayFrame(spriteFrame)
        if leftOrRight=="r" then
            self.hero:setFlipX(false)
        else
            self.hero:setFlipX(true)
        end
        self.hero:stopAllActions()
        local attackFrames,attackAnimation = AnimationFactory.getHeroAnimation(self:getHeroId(),"attack",frontOrBack)
        transition.execute(self.hero,CCAnimate:create(attackAnimation),{
        onComplete = function()
        end
        })
        self:doHeroStandAnimation(frontOrBack)
        self:resetAnger()
        display.getRunningScene():addChild(HeroSkill.new(self,self:getSkillId(),ccp(targetMonster:getPositionX(),targetMonster:getPositionY()),self),50001)
    else
        local attackFrames,attackAnimation = AnimationFactory.getHeroAnimation(self:getHeroId(),"attack","front")
        transition.execute(self.hero,CCAnimate:create(attackAnimation),{
        onComplete = function()
        end
        })
        self:doHeroStandAnimation("front")
        self:resetAnger()
        display.getRunningScene():addChild(HeroSkill.new(self,self:getSkillId(),self:findReleaseSkillPosition(),self),50001)
    end
end


function Tower:findReleaseSkillPosition()
    if GameInstance:sharedInstance().paths:count()==0 then return ccp(self:getPositionX(),self:getPositionY()) end
    for i=0,GameInstance:sharedInstance().paths:count()-1 do
        local path = tolua.cast(GameInstance:sharedInstance().paths:objectAtIndex((i)),"CCArray")
        for j=0,path:count()-1 do
            local sPoint = tolua.cast(path:objectAtIndex(j),"WayPoint")--怪物的出生点第一个点
            if ccpDistance(ccp(sPoint.position.x,sPoint.position.y),ccp(self:getPositionX(),self:getPositionY())) < 5 then
                return ccp(sPoint.position.x,sPoint.position.y-100)
            end
        end 
    end
    return ccp(self:getPositionX(),self:getPositionY())
end

function Tower:addUpgradeTip()
    if not self.args.npc then return end

    if self.upgradeTip then
        self.upgradeTip:removeFromParentAndCleanup(true)
        self.upgradeTip = nil
    end

    if gHero:toLowerLevelUp(self:getHeroId()) then
        self.upgradeTip = display.newSprite("ui/public/public_arrow_a.png",25,0):addTo(self)
            local sequence = transition.sequence({
                CCMoveBy:create(0.3, ccp(0, 10)),
                CCMoveBy:create(0.3, ccp(0, -10)),
        })
        self.upgradeTip:setRotation(-90)
        self.upgradeTip:setScale(0.7)
        self.upgradeTip:runAction(CCRepeatForever:create(sequence))
    end
end

function Tower:addTowerFlag()
    self.towerFlag = display.newSprite("ui/fighting/fighting_tower_call.png",0,35):addTo(self.towerView)
    local sequence = transition.sequence({
            CCMoveBy:create(0.3, ccp(0, 5)),
            CCMoveBy:create(0.3, ccp(0, -5)),
    })
    self.towerFlag:setScale(0.7)
    self.towerFlag:runAction(CCRepeatForever:create(sequence))
end

--控制tower cd coolDown时间--------------------------------------
function Tower:coolDown(delayTime)
    self.cooldown = true
    local action = transition.sequence({
        CCDelayTime:create(delayTime),
        CCCallFunc:create(function()
            self.cooldown = false
        end),
    })
    action:setTag(99)
    self:runAction(action)
end
function Tower:resume()
    self:stopActionByTag(99)
    self.cooldown = false
end
--控制tower cd coolDown时间--------------------------------------
function Tower:refresh()
    self:deleteTower()
    self:updateAppearence()
end

function Tower:deleteByBoss()
    self.damaged = true
    self:deleteTower()
end

function Tower:disappear()
    self.towerInfo=nil
    self:deleteTower()
end

function Tower:resumeTower()
    self.damaged = false
    self:addHeroView(false)
end

function Tower:getDamageDegree()
    return self.damagedegree
end

function Tower:getRangeBox()
    local size = self.towerView:getCascadeBoundingBox().size
    --print("怪物的宽高",size.width, size.height)
    local anchorPoint = self.towerView:getAnchorPoint()
    local textureCenterX = self:getPositionX() + (0.5-anchorPoint.x)*size.width
    local textureCenterY = self:getPositionY() + (0.5-anchorPoint.y)*size.height
    return CCRect(textureCenterX-self:getRange(),textureCenterY-self:getRange(),size.width,size.height)
end

--判断英雄是否设置的标志
function Tower:setSkillOverFlag(skillOverFlag)
    self.skillOverFlag = skillOverFlag
end

--获取塔上
function Tower:getBulletReleasePointInWorld()
    local size = self.towerView:getBoundingBox().size
    local anchorPoint = self.towerView:getAnchorPoint()
    local releaseX = self:getPositionX() + (0.5-anchorPoint.x)*size.width
    local releaseY = self:getPositionY() + (0.5-anchorPoint.y)*size.height + size.height/2
    return releaseX,releaseY
end

function Tower:ifBuild()
    if self.towerInfo~=nil and self:getHeroId()~=nil then
        return true
    end
    return false
end

function Tower:getSkillId()
    --print("技能id是",self.skillId)
    return self.skillid
    --return 41007
end

function Tower:getRange()
    return self.range
end

function Tower:getPower()
    return self.attackpower
end

function Tower:bianShen()
    if self.bianShenSprite then
        CommUtil.toast(display.getRunningScene(),"已变身",0)
        return 
    end
    self.hero:setPositionY(65)
    local array = CCArray:create()
    CommUtil.playSound("music/enlarge.mp3")
    array:addObject(CCCallFunc:create(function() self.hero:setScale(0.9) end))
    array:addObject(CCDelayTime:create(0.1))
    array:addObject(CCCallFunc:create(function() self.hero:setScale(1) end))
    array:addObject(CCDelayTime:create(0.1))
    array:addObject(CCCallFunc:create(function() self.hero:setScale(1.1) end))
    array:addObject(CCDelayTime:create(0.1))
    array:addObject(CCCallFunc:create(function() self.hero:setScale(1.2) end))
    array:addObject(CCDelayTime:create(0.1))
    array:addObject(CCCallFunc:create(function()
        local frames,animation = AnimationFactory.getAddBloodAnimation()
        self.bianShenSprite = display.newSprite(frames[1],0,60)
        self:addChild(self.bianShenSprite,50003)
        self.bianShenSprite:playAnimationForever(animation)
        self.attackpower = self.attackpower * 1.5
        self:performWithDelay(function()
            self.attackpower = self.attackpower / 1.5
            if self.bianShenSprite then
                self.bianShenSprite:removeSelf()
                self.bianShenSprite = nil
                self:removeBianshen()
            end
        end,10)
    end))
    self:runAction(CCSequence:create(array))
end

function Tower:removeBianshen()
    if self.hero==nil then return end
    self.hero:setPositionY(48)
    local array = CCArray:create()
    array:addObject(CCCallFunc:create(function() self.hero:setScale(1.1) end))
    array:addObject(CCDelayTime:create(0.1))
    array:addObject(CCCallFunc:create(function() self.hero:setScale(1) end))
    array:addObject(CCDelayTime:create(0.1))
    array:addObject(CCCallFunc:create(function() self.hero:setScale(0.9) end))
    array:addObject(CCDelayTime:create(0.1))
    array:addObject(CCCallFunc:create(function() self.hero:setScale(0.8) end))
    self:runAction(CCSequence:create(array))

end
function Tower:updateAppearencePower(have)
    if (not have and self.towerBaseSun) then
        self.towerBaseSun:removeSelf()
        self.towerBaseSun = nil
    end
    --
    if self:getHeroId()~=nil then
        if not self.towerBaseSun then
            if (have) then
            self.towerBaseSun = display.newSprite("#release_01.png",x, y):addTo(self, -2)
            local frames = display.newFrames("release_%02d.png", 1, 4)
            local animation = display.newAnimation(frames, 1 / 4)
            self.towerBaseSun:playAnimationForever(animation)
            self.towerBaseSun:setScale(0.9)
             end
        end
    else
        if self.towerBaseSun then
            self.towerBaseSun:removeSelf()
            self.towerBaseSun = nil
        end
    end
end


function Tower:addTowerBase()
    --
    self.towerBase = display.newSprite("ui/fighting/fighting_tower.png",0,8):addTo(self,-1)
    self.towerBase:setScale(0.8)
end

function Tower:deleteTower()
    self.towerView:removeSelf()
    self.towerView = display.newSprite():addTo(self)
    self:addTowerFlag()
    self.hero = nil
    if self.upgradeTip then
        self.upgradeTip:removeFromParentAndCleanup(true)
        self.upgradeTip = nil
    end
    self.upgradeTimes = 0
    self.angerBg:setVisible(false)
    self.angerPer:setVisible(false)
    self.angerNum = 0
end

--更新塔的外观
function Tower:updateAppearence()
    if not self:ifBuild() then
        return
    end
    self:addHeroView(self:ifFriendTower())
    self:addUpgradeTip()
end

function Tower:getTowerLevel()
    return self.towerLevel
end

--点击塔的时候需要显示的属性
function Tower:getTowerName()
    return self.towerName
end

function Tower:getHeroName()
    return self.name
end

function Tower:getHeroLevel()
    return self.heroLevel
end
-----------------------------------------------------------------
function Tower:getDamageType()
    return self.damagetype
end

function Tower:setTowerProperties(properties)
    self.towerInfo = properties
    self:initBattleProperties()
end
--是否只是空塔
function Tower:ifOnlyTower()
    if self.towerInfo==nil then
        return false
    end
    if self:getHeroId()==nil then
        return true
    end
    return false
end
function Tower:ifFriendTower()
    if self.towerInfo==nil then
        return false
    end
    if self.towerInfo.friend_player_id~=nil and self.towerInfo.friend_player_id~=0 then
        return true
    end
    return false
end

function Tower:getFriendLeaveTimes()
    return self.towerInfo.hero_data_point.today_fight_times
    --return 8
end

function Tower:getFriendName()
    return self.towerInfo.hero_data_point.name
end

function Tower:ifRecommendFriendTower()
    if self.towerInfo==nil then
        return false
    end
    if self.towerInfo.is_recommend_friend==1 then
        --print("推荐好友塔")
        return true
    else
        --print("非推荐好友塔")
        return false
    end
end

function Tower:ifSendHero()
    if self.towerInfo==nil then
        return false
    end
    if self:getHeroId()~=nil and self:getHeroId()~=0 then
        return true
    else
        return false
    end
end

function Tower:getTowerType()
    local class = self:getHeroClass()
    if class==1 then
        return "warrior"
    elseif class==2 then
        return "hunter"
    elseif class==3 then
        return "wizard"
    elseif class==4 then
        return "mage"
    end
    --'职业: 1.狂战 2.神射 3.方十 4.法师',
    --{"warrior","hunter","wizard","mage"}
end

function Tower:getHeroClass()
    return self.heroClass
end

function Tower:setTowerView()
    local imageName = string.format("tower_%s_lv%d.png",self:getTowerType(),self.towerLevel)
    local spriteFrame = display.newSpriteFrame(imageName)
    self.towerView:setDisplayFrame(spriteFrame)
end


function Tower:addFriendEffect()
    local num = nil 
    if self.towerInfo.hero_data_point.is_recommend_friend == 1 then
        num = 5
    else
        num = 5-(self.towerInfo.hero_data_point.today_fight_times==nil and 0 or self.towerInfo.hero_data_point.today_fight_times)
    end
    --print("今天好友塔战斗次数",self.towerInfo.hero_data_point.today_fight_times)
    for i=1,5 do
        if i<=num then
            local y = 15
            if i==2 or i==4 then
                y = y-5
            end
            if i==3 then y =y-8 end
            local timesIcon = display.newSprite("ui/fighting/fighting_door_yellow.png",i*15-5,y):addTo(self.towerView)
            timesIcon:setScale(0.4)
        else
            local y = 15
            if i==2 or i==4 then
                y = y-5
            end
            if i==3 then y =y-8 end
            local timesIcon = display.newSprite("ui/fighting/fighting_door_yellow.png",i*15-5,y):addTo(self.towerView)
            timesIcon:setOpacity(150)
            timesIcon:setScale(0.4)
        end
    end
end

function Tower:addHeroDropFromSkyEffect(callback)
    local frames = display.newFrames("intower_%02d.png", 1, 10)
    local animation = display.newAnimation(frames, 0.5 / 10)
    local towerEffect = display.newSprite(frames[1],-10,80)
    towerEffect:setScale(1)
    self.towerEffect[#self.towerEffect+1] = towerEffect
    self:addChild(towerEffect,20)
    transition.execute(towerEffect,CCAnimate:create(animation), {
        onComplete = function()
            self.towerEffect[#self.towerEffect]:removeSelf()
            table.remove(self.towerEffect,#self.towerEffect)
            callback()
        end,
    })
end

function Tower:removeHeroDropFromSkyEffect()
    for k, v in ipairs(self.towerEffect) do
        v:removeSelf()
        table.remove(self.towerEffect,k)
    end
end

function Tower:getHeroId()
    --if true then  return 10003 end
    return self.heroId
    --return 10006
end

function Tower:addHeroView(ifFriend)
    --self.towerBase:setVisible(true)
    self.towerView:stopAllActions()
    self.towerView:removeAllChildren()

    --self:setTowerView()
    local frontOrBack = nil
    local leftOrRight = nil

    if self.properties.direction=="1" then
        frontOrBack = "front"
        leftOrRight = "l"
    elseif self.properties.direction=="2" then
        frontOrBack = "front"
        leftOrRight = "r"
    elseif self.properties.direction=="3" then
        frontOrBack = "back"
        leftOrRight = "l"
    elseif self.properties.direction=="4" then
        frontOrBack = "back"
        leftOrRight = "l"
    else
        frontOrBack = "front"
        leftOrRight = "r"
    end

    local heroImageName = "#hero_"..self:getHeroId().."_stand".."_"..frontOrBack.."_1.png"
    local x = 0
    local y = 48
    self.hero = display.newSprite(heroImageName,x,y):addTo(self.towerView)
    --self.hero:setScale(0.8)
    if leftOrRight=="l" then
        self.hero:setFlipX(true)
    end
    self:doHeroStandAnimation("front")
    --self:addUpgradeTip()
    self:checkAddGroup()
    self.angerBg:setVisible(true)
    self.angerPer:setVisible(true)
    self.angerPer:setPercentage(0)


    --[[
    self.hero:setTouchEnabled(true) -- enable sprite touch
    self.hero:addTouchEventListener(function(event, x, y)
        if event == "began" and self.angerNum>=100 then
            self:releaseSkill()
            return false
        end
    end)]]
end

function Tower:checkAddGroup()

    local groups = require("app.client.desc.Group")

    for i=1,#groups do
        local group = groups[i]

        local heroId1 = group[2]
        local heroId2 = group[3]
        local heroId3 = group[4]

        local tower1,flag1 = self:ifHeroFighting(heroId1)
        local tower2,flag2 = self:ifHeroFighting(heroId2)
        local tower3,flag3 = self:ifHeroFighting(heroId3)

        if flag1 and flag2 and flag3 then
            local groupId = group[1]
            local groupType = group[5]
            local groupValue = group[6]

            if tower1~=nil then tower1:addGroup(groupId,groupType,groupValue) end
            if tower2~=nil then tower2:addGroup(groupId,groupType,groupValue) end
            if tower3~=nil then tower3:addGroup(groupId,groupType,groupValue) end

        end
    end

    for i=0,GameInstance:sharedInstance().towers:count()-1 do
        local tower = GameInstance:sharedInstance().towers:objectAtIndex(i)
        if tower:getHeroId()~=nil then
            --print(tower:getHeroId().."的组合的大小是",tower.heroGroups:count())
        end
    end
end

function Tower:playGroupAnimation()
    if self.groupSprite then return end
    local frames,animation = AnimationFactory.getAddBloodAnimation()
    self.groupSprite = display.newSprite(frames[1],0,30)
    self.groupSprite:setShaderProgram(CCShaderCache:sharedShaderCache():programForKey("blue"))
    self:addChild(self.groupSprite,50003)
    self.groupSprite:playAnimationForever(animation)
end

function Tower:addGroup(groupId,groupType,groupValue)
    local heroGroup = HeroGroup.new(groupId,groupType,groupValue)
    self.heroGroups:addObject(heroGroup)
    --self:playGroupAnimation()
end

function Tower:removeHero()
    self.angerBg:setVisible(false)
    self.angerPer:setVisible(false)
    self.angerNum = 0
    self.heroId = nil
    self.towerInfo = nil
    self.upgradeTimes = 0
    self.towerView:stopAllActions()
    self.towerView:removeAllChildren()
    self:addTowerFlag()
    self:clearRelatedGroups()

    if self.groupSprite then
        self.groupSprite:removeSelf()
        self.groupSprite = nil
    end

    if self.bianShenSprite then
        self.bianShenSprite:removeSelf()
        self.bianShenSprite = nil
    end

    for i=0,GameInstance:sharedInstance().towers:count()-1 do
        local tower = GameInstance:sharedInstance().towers:objectAtIndex(i)
        if tower:getHeroId()~=nil then
            if tower.heroGroups:count()<=0 and tower.groupSprite then
                tower.groupSprite:removeSelf()
                tower.groupSprite = nil
            end
            --print(tower:getHeroId().."的组合的大小是",tower.heroGroups:count())
        end
    end

end

function Tower:clearRelatedGroups()
    --print("清空相关塔的groups")
    for i=0,self.heroGroups:count()-1 do
        local eachGroup = self.heroGroups:objectAtIndex(i)
        local groupId = eachGroup.groupId
        for i=0,GameInstance:sharedInstance().towers:count()-1 do
            local tower = GameInstance:sharedInstance().towers:objectAtIndex(i)
            if tower:getHeroId()~=nil and tower:ifAtTheSameGroup(groupId) then end
        end
    end
    self.heroGroups:removeAllObjects()
end
------------------------------------------------如果在同一个组，则删除此组--------------------------------------------------------
function Tower:ifAtTheSameGroup(groupId)
    for i=0,self.heroGroups:count()-1 do
        local eachGroup = self.heroGroups:objectAtIndex(i)
        if eachGroup.groupId==groupId then
            self.heroGroups:removeObject(eachGroup)
            return
        end
    end
end

function Tower:ifHeroFighting(heroId)
    if heroId==0 or heroId==nil or heroId=="" then return nil,true end
    for i=0,GameInstance:sharedInstance().towers:count()-1 do
        local tower = GameInstance:sharedInstance().towers:objectAtIndex(i)
        if tower:getHeroId()==heroId then
            return tower,true
        end
    end
    return nil,false
end

function Tower:doHeroStandAnimation(frontOrBack)
    self.heroState="stand"
    self.hero:stopAllActions()
    self.frontOrBack = frontOrBack
    local frames,animation = AnimationFactory.getHeroAnimation(self:getHeroId(),"stand",frontOrBack)
    transition.execute(self.hero,CCAnimate:create(animation),{
    onComplete = function()
        self:doHeroStandAnimation(frontOrBack)
    end
    })
end

function Tower:doHeroAttackAnimation(frontOrBack)
    self.heroState="attack"
    self.hero:stopAllActions()
    self.frontOrBack = frontOrBack
    local frames,animation = AnimationFactory.getHeroAnimation(self:getHeroId(),"attack",frontOrBack)
    transition.execute(self.hero,CCAnimate:create(animation),{
    onComplete = function()
        self:doHeroStandAnimation(frontOrBack)
    end
    })
end

function Tower:addTipView()
    self.towerView:removeAllChildren()
    local tip = display.newSprite("ui/home/home_tower_bang.png",43,80):addTo(self.towerView)
    local sequence = transition.sequence({
        CCMoveBy:create(0.3, ccp(0, 10)),
        CCMoveBy:create(0.3, ccp(0, -10)),
    })
    tip:runAction(CCRepeatForever:create(sequence))
end

function Tower:runTowerAnimation(towerType)
    if towerType=="wizard" then
        transition.playAnimationForever(self.towerView,AnimationFactory.getTowerAnimation(towerType))
    end
end

function Tower:getBox()
    local posX,posY = self:getPosition()
    local rect = CCRect(posX-80,posY-80,160,160)
    return rect
end

function Tower:getToolBarBox()
    if self.buildToolBarView~=nil then
        return self.buildToolBarView.bg:getCascadeBoundingBox()
    else
        return nil
    end
end

function Tower:ifClickItem(x,y)
    if self.buildToolBarView~=nil then
        local posX,posY = self:getPosition()
        local rect = CCRect(posX-80,posY-80,160,160)
        if rect:containsPoint(ccp(x,y)) then
            return true
        end
    else
        return false
    end
end

function Tower:initBattleProperties()
    if self.towerInfo==nil then return end
    self.heroId = self.towerInfo.id
    self.damagetype = self.towerInfo.id
    self.stealth = self.towerInfo.stealth==1 and true or false
    self.sunder = self.towerInfo.sunder:get()
    self.range = self.towerInfo.range:get()
    self.ground = self.towerInfo.ground==1 and true or false
    self.crit = self.towerInfo.crit:get()
    self.star = self.towerInfo.star:get()
    self.attackpower = self.towerInfo.attackpower:get()
    self.skillid = self.towerInfo.skillid
    self.bornid = self.towerInfo.bornid
    self.attackspeed = self.towerInfo.attackspeed:get()
    self.air = self.towerInfo.air==1 and true or false
    self.property = self.towerInfo.property
    self.name = self.towerInfo.name
    self.hit = self.towerInfo.hit
    self.dida = self.towerInfo.energy:get()

    self.anti_wood = self.towerInfo.anti_wood:get()
    self.anti_water = self.towerInfo.anti_water:get()
    self.anti_fire = self.towerInfo.anti_fire:get()
    self.anti_soil = self.towerInfo.anti_soil:get()


    if self.heroId==10001 then
        self.bulletIndex = 1
        self.beHitType = 2
        self.sountdIndex = 1
    elseif self.heroId==10002 then
        self.bulletIndex = 2
        self.beHitType = 2
        self.sountdIndex = 1
    elseif self.heroId==10003 then
        self.bulletIndex = 3
        self.beHitType = 1
        self.sountdIndex = 1
    elseif self.heroId==10004 then
        self.bulletIndex = 4
        self.beHitType = 1
        self.sountdIndex = 1
    elseif self.heroId==10005 then
        self.bulletIndex = 5
        self.beHitType = 1
        self.sountdIndex = 2
    elseif self.heroId==10006 then
        self.bulletIndex = 6
        self.beHitType = 2
        self.sountdIndex = 3
    else
        self.bulletIndex = 1
        self.beHitType = 2
    end


end

function Tower:getCanClickBox()
    local boudingBox = self.towerView:getCascadeBoundingBox()
    local size = boudingBox.size
    local anchorPoint = self.towerView:getAnchorPoint()
    local textureCenterX = self:getPositionX() + (0.5-anchorPoint.x)*size.width
    local textureCenterY = self:getPositionY() + (0.5-anchorPoint.y)*size.height

    local rect  = CCRect(textureCenterX-size.width/2,textureCenterY-size.height/2,size.width,size.height)

    return boudingBox
end

function Tower:doClick()

    --[[
    local size = self.towerView:getCascadeBoundingBox().size
    local anchorPoint = self.towerView:getAnchorPoint()
    local textureCenterX = self:getPositionX() + (0.5-anchorPoint.x)*size.width
    local textureCenterY = self:getPositionY() + (0.5-anchorPoint.y)*size.height]]
    --print("tower坐标信息是",textureCenterX-size.width/2,textureCenterY-size.height/2,2*self:getRange())
    --print("尼玛啊",self.air,self.stealth,self.ground)
    --print(nil.."1231")
    --print("塔的等级",self.towerLevel)
    --print("塔是否可以攻击",self.skillId)
    --print("塔得攻击",self.air,self.underground,self.stealth)
    --[[
    if true then return end
    local towerData = gTowerTemplate:getTowerData(self.towerInfo.hero_data_point.class,self.towerLevel)
    print("塔的隐身对空对地属性",towerData.air,towerData.underground,towerData.stealth)
    print("特殊属性",self.air,self.underground,self.stealth)]]
end

function Tower:get_hit_proc()
    return self.hit_proc
end
function Tower:get_crit_proc()
    return self.crit
end
function Tower:get_sunder()
    return self.sunder
end
function Tower:get_crit_mul()
    return self.crit_mul
end
function Tower:get_attack_power()
    if (self.attackpower > SKILL_ATTACKPOWER) then
        SKILL_ATTACKPOWER = self.attackpower
    end
    return self.attackpower
end
function Tower:get_damage_lower_proc()
    return self.damage_lower_proc
end
function Tower:get_skill_proc()
    return self.skill_proc
end
function Tower:getAttackSpeed()
    return self.attackspeed
end

function Tower:addSkillWord(callback)
    --CCScaleTo:create(2.0, 0.5)
    self.skillTip = display.newProgressTimer("ui/skill_word/skill_wodr_"..self.skillId..".png",kCCProgressTimerTypeBar)
    self.skillTip:setPosition(self:getPositionX(),self:getPositionY()+60)
    self.skillTip:setMidpoint(CCPointMake(0, 0))
    self.skillTip:setBarChangeRate(CCPointMake(1, 0))
    self.skillTip:setPercentage(30)
    self.gameScene:addChild(self.skillTip,50007)
    local action = transition.sequence({
        CCProgressTo:create(0.2, 100),
        CCDelayTime:create(0.2),
        CCCallFunc:create(function()
            self.skillTip:setMidpoint(CCPointMake(1, 0))
            self.skillTip:setBarChangeRate(CCPointMake(1, 0))
        end),
        CCDelayTime:create(0.5),
        CCCallFunc:create(function()
            self.skillTip:setPercentage(90)
        end),
        CCDelayTime:create(0.04),
        CCCallFunc:create(function()
            self.skillTip:setPercentage(70)
        end),
        CCDelayTime:create(0.04),
        CCCallFunc:create(function()
            self.skillTip:setPercentage(50)
        end),
        CCDelayTime:create(0.04),
        CCCallFunc:create(function()
            self.skillTip:setPercentage(30)
        end),
        CCDelayTime:create(0.04),
        CCCallFunc:create(function()
            self.skillTip:setPercentage(20)
        end),
        CCDelayTime:create(0.04),
        CCCallFunc:create(function()
            self.skillTip:setPercentage(0)
        end),
        CCCallFunc:create(function()
            self.skillTip:removeFromParentAndCleanup(true)
        end),
    })
    callback()
    self.skillTip:runAction(action)
end

--Dot是战斗力最高的英雄的攻击力*bufValue
--DOT_HP%是秒怪物满血状态的百分比
--1,DOT;2,免伤；3，移动速度；4，攻击速度；5.生命值上限；6，攻击力；7，暴击率；8，技能触发几率；9，眩晕；10，变形；11，击退；12，牵引；13，石化；14，印记传送；15,DOT_HP%
--怪物的buff处理
function Tower:addBuff(buff)
    
    if buff:getType()==1 then
        
    elseif buff:getType()==2 then
       
    elseif buff:getType()==3 then
        
    elseif buff:getType()==4 then

    elseif buff:getType()==5 then

    elseif buff:getType()==6 then

    elseif buff:getType()==7 then

    elseif buff:getType()==8 then

    end
end

function Tower:getBuffs()
    return self.buffs
end

function Tower:addBuff(skillId,buff)
    self.buffSkillIDs[#self.buffSkillIDs+1] = skillId
    if buff:getType()==1 then
    elseif buff:getType()==2 then
    elseif buff:getType()==3 then
        self:slowDown(buff:getTime(),buff:getValue())
    elseif buff:getType()==4 then
        
    elseif buff:getType()==5 then

    elseif buff:getType()==6 then
        self:addPowerEffect(buff:getTime())
        buff:setMonsterFlag(false)
        self.buffs:addObject(buff)
        self:performWithDelay(function()
            self.buffs:removeObject(buff)
        end,buff:getTime())
    elseif buff:getType()==7 then

    elseif buff:getType()==8 then
    end
end

function Tower:removeBuffID(skillId)
    for i=1,#self.buffSkillIDs do
        if (self.buffSkillIDs[i].."")==(skillId.."") then
            table.remove(self.buffSkillIDs, i)
            return
        end
    end
end

function Tower:ifAddBuff(skillId)
    for i=1,#self.buffSkillIDs do
        if (self.buffSkillIDs[i].."")==(skillId.."") then
            return true
        end
    end
    return false
end

--是否可以攻击天上的怪物
function Tower:canAttackAir()
    --return true
    return self.air
end
--是否可以攻击砖地的怪物
function Tower:canAttackUnderground()
    --return true
    return self.ground
end
--是否可以攻击隐身的怪物
function Tower:canAttackStealth()
    --return true
    --print("是否可以攻击隐身怪",self.stealth)
   return self.stealth
end

function Tower:getIdInTileMap()
    return self.properties.id
end

function Tower:setRect(rect)
    self.rect = rect
end

function Tower:getRoleName()
    return "tower"
end

function Tower:getOriginPointAndRange()
    local size = self.towerView:getCascadeBoundingBox().size
    local anchorPoint = self.towerView:getAnchorPoint()
    local textureCenterX = self:getPositionX() + (0.5-anchorPoint.x)*size.width
    local textureCenterY = self:getPositionY() + (0.5-anchorPoint.y)*size.height
    --print("tower坐标信息是",textureCenterX-size.width/2,textureCenterY-size.height/2,2*self:getRange())
    local range = self:getRange()+30
    return textureCenterX-range,textureCenterY-range,self:getRange()*2
end


function Tower:getRectAndOffset()
    local towerRect = self.towerView:getBoundingBox()
    local anchorPoint = self.towerView:getAnchorPoint()
    local size = towerRect.size
    local posX = self:getPositionX() + (0.5-anchorPoint.x)*size.width
    local posY = self:getPositionY() + (0.5-anchorPoint.y)*size.height
    local offset = 0
    local range = self:getRange()*2
    --local newRect = CCRect(posX-size.width/2+offset,posY-size.height/2+offset,size.width-offset*2,size.height-offset*2)
    --return newRect,offset,ccp(posX,posY),size
    local newRect = CCRect(posX-range/2+offset,posY-range/2+offset,range-offset*2,range-offset*2)
    return newRect,offset,ccp(posX,posY),CCSize(range,range)
end

function Tower:getCenterPoint()
    local size = self.towerView:getCascadeBoundingBox().size
    local anchorPoint = self.towerView:getAnchorPoint()
    local textureCenterX = self:getPositionX() + (0.5-anchorPoint.x)*size.width
    local textureCenterY = self:getPositionY() + (0.5-anchorPoint.y)*size.height
    return ccp(textureCenterX,textureCenterY)
end

function Tower:getTopPoint()
    local size = self.towerView:getCascadeBoundingBox().size
    local anchorPoint = self.towerView:getAnchorPoint()
    local textureCenterX = self:getPositionX() + (0.5-anchorPoint.x)*size.width
    local textureCenterY = self:getPositionY() + (0.5-anchorPoint.y)*size.height
    return ccp(textureCenterX,textureCenterY)
end

function Tower:getBottomPoint()
    local size = self.towerView:getCascadeBoundingBox().size
    local anchorPoint = self.towerView:getAnchorPoint()
    local textureCenterX = self:getPositionX() + (0.5-anchorPoint.x)*size.width
    local textureCenterY = self:getPositionY() + (0.5-anchorPoint.y)*size.height
    return ccp(textureCenterX,textureCenterY-size.height/2-20)
end

-----------------冻住冰封住塔
function Tower:frozen()
    self.frozened = true
    self:unEnhance()
    --播放一个被冻住的特效---------------------
    --[[]]
    local frames,animation = AnimationFactory.getFrozenTowerAnimation()
    self.freezSprite = display.newSprite(frames[1],5,35)
    self.freezSprite:setScale(0.7)
    self:addChild(self.freezSprite,50003)
    self.freezSprite:playAnimationOnce(animation)

    local action = transition.sequence({
        CCDelayTime:create(30),
        CCCallFunc:create(function()
            self.frozened = false
            self.freezSprite:removeFromParentAndCleanup(true)
        end),
    })
    action:setTag(5566)
    self:runAction(action)
end

function Tower:freeFrozen()
    --播放一个解封的特效---------------------
    self:stopActionByTag(5566)
    self.frozened = false
    self.freezSprite:removeFromParentAndCleanup(true)
end

-----------------奋功能,增强塔50%的攻击力s
function Tower:enhance(degree)
    if self.enhanceDamageDegree~=1 then return end
    self.enhanceDamageDegree = 1.5
    if degree~=nil then self.enhanceDamageDegree=1.15 end
    --播放一个增加攻击力的特效---------------------
    local frames,animation = AnimationFactory.getAddBloodAnimation()
    self.enhanceSprite = display.newSprite(frames[1],0,30)
    self:addChild(self.enhanceSprite,50003)
    self.enhanceSprite:playAnimationForever(animation)
end

function Tower:unEnhance()
    self.enhanceDamageDegree = 1
    if self.enhanceSprite then
        self.enhanceSprite:removeFromParentAndCleanup(true)
        self.enhanceSprite = nil
    end 
end


function Tower:add41014Buff(delayTime)
    if self.buff41014Sprite then
        self.buff41014Sprite:removeFromParentAndCleanup(true)
        self.buff41014Sprite = nil
        self.forceSkill = false
        self:stopActionByTag(5577)
    end
    self.forceSkill = true
    local frames,animation = AnimationFactory.getAddBloodAnimation()
    self.buff41014Sprite = display.newSprite(frames[1],0,30)
    self.buff41014Sprite:setShaderProgram(CCShaderCache:sharedShaderCache():programForKey("yellow"))--红色怪物
    self:addChild(self.buff41014Sprite,50003)
    self.buff41014Sprite:playAnimationForever(animation)

    local action = transition.sequence({
        CCDelayTime:create(delayTime),
        CCCallFunc:create(function()
            self.forceSkill = false
            self.buff41014Sprite:removeFromParentAndCleanup(true)
            self.buff41014Sprite = nil
        end),
    })
    action:setTag(5577)
    self:runAction(action)
end

function Tower:addBubbleAndWords(word)
    self:removeBubble()
    self.bubble = display.newSprite("ui/public/public_dialog_box.png",self:getPositionX()+60,self:getPositionY()+80)
    self.bubble:setTag(777)
    display.getRunningScene():addChild(self.bubble,1000)
    self.words = ui.newTTFLabel({
        text = word,
        font = "Arial",
        color = ccc3(0, 0, 0),
        x = -40,
        y = 10,
        size = 14,
        dimensions = CCSize(135,120),
        align = ui.TEXT_ALIGN_LEFT
    })
    self.words:setAnchorPoint(ccp(0,0))
    self.bubble:addChild(self.words)

    self:performWithDelay(function()
        self:removeBubble()
    end,2)
end

function Tower:removeBubble()
    local bubble = display.getRunningScene():getChildByTag(777)
    if bubble then
        bubble:removeSelf()
    end
end

function Tower:heroRideBirdToScene(heroId,text)

    --require("app.common.NpcUtil").geneHelpHeroData(gBattle.data.home_pass_id,heroId,self:getIdInTileMap(),true)--产生一个塔位数据

    local frames,animation = AnimationFactory.getBirdFlyAnimation()
    local bird = display.newSprite(frames[1],display.right,self:getPositionY()+50)
    bird:playAnimationForever(animation)
    bird:setFlipX(false)

    local flyToDestinationTime = (display.right-self:getPositionX())/200

    local frames,animation = AnimationFactory.getHeroAnimation(heroId,"stand","front")
    local hero = display.newSprite(frames[1],display.right,self:getPositionY()+75)
    hero:setFlipX(true)
    hero:playAnimationForever(animation)

    self.gameScene:addChild(bird,5000)
    self.gameScene:addChild(hero,5001)


    transition.execute(bird,CCMoveBy:create(flyToDestinationTime,ccp(self:getPositionX()-display.right,0)),{
        onComplete=function()
            bird:setFlipX(true)
            transition.execute(bird,CCMoveBy:create(4,ccp(800,0)),{
                onComplete=function()
                    bird:removeSelf()
                end
            })
        end
    })

    transition.execute(hero,CCMoveBy:create(flyToDestinationTime,ccp(self:getPositionX()-display.right,0)),{
        onComplete=function()
            local towerInfo = nil
            require("app.common.NpcUtil").geneHelpHeroData(lastHelpSceneId,heroId,self:getIdInTileMap(),true)--产生一个塔位数据
            local sceneInfo = gNTower:getSceneInfo(lastHelpSceneId)["scene_info"]

            local tempHeroId = nil
            for i=1,#sceneInfo do
                local eachInfo = sceneInfo[i]
                if tonumber(eachInfo.hero_data_point.quality:get())~=1 then
                    tempHeroId = eachInfo.hero_data_point.bornid
                else
                    tempHeroId = 10001
                end
                if tempHeroId==heroId then
                    towerInfo = eachInfo
                end
            end


            self:setTowerProperties(towerInfo)
            self:refresh()

            local x = 40
            local y = 50
            if self:getTowerType()=="hunter" then
                x = x
                y = y + 30
            elseif self:getTowerType()=="warrior" then
                y = y + 20      
            elseif self:getTowerType()=="wizard" then
                x = x + 2
                y = y + 29
            elseif self:getTowerType()=="mage" then
                x = x
                y = y + 25
            end

           self.hero:setPosition(ccp(x,y+30))

           transition.execute(self.hero,CCMoveBy:create(0.2,ccp(0,10)),{
            onComplete=function()
                transition.execute(self.hero,CCMoveBy:create(0.5,ccp(0,-40)),{
                onComplete=function()
                    self:addBubbleAndWords(text)
                end
                })
            end
            })

            self:performWithDelay(function()
                self.gameScene.quadTree:updateTowerRect(self)
            end,1)

            hero:removeSelf()

        end
    })


    
end

return Tower 