local MonsterRunCycleWay = require("app.actor.MonsterRunCycleWay")
local AnimationFactory = require("app.factory.AnimationFactory")

local WorldBoss = class("WorldBoss",MonsterRunCycleWay)

function WorldBoss:ctor(attackerProperties,parent,path)
    WorldBoss.super.ctor(self,attackerProperties,parent,path)

    --[[
    self.mainSprite:setTouchEnabled(true)
    self.mainSprite:addTouchEventListener(function(event, x, y)
        if event == "began" then
            print("boss走的哪一条路",self.way)
            return false
        end
    end)]]

    self.forceWalkStatus = false
    self.countOrStandCounter = 0

end

function WorldBoss:decreaseHpBySkill(decreaseHp,if_crit)
    bossDecreaseTotalHP = bossDecreaseTotalHP + decreaseHp
    self:addDecreaseHPLabel(decreaseHp,if_crit)
    if if_crit then self:addCritLabel() end

    --[[
    if bossDecreaseTotalHP>200000 then
        self.hp:setPercentage(80)
    elseif bossDecreaseTotalHP>100000 then
        self.hp:setPercentage(85)
    elseif bossDecreaseTotalHP>50000 then
        self.hp:setPercentage(90)
    elseif bossDecreaseTotalHP>10000 then
        self.hp:setPercentage(95)
    else
        self.hp:setPercentage(100)
    end]]

    local newhp = self.fCurLifeValue - decreaseHp
    if newhp <= 0 then
        newhp = self.fullHP
        local frames,animation = AnimationFactory.getAddBloodAnimation()
        local addBloodSprite = display.newSprite(frames[1],0,80)
        self:addChild(addBloodSprite,50003)
        transition.execute(addBloodSprite,CCAnimate:create(animation),{
            onComplete = function()
                addBloodSprite:removeFromParentAndCleanup(true)
            end,
        })
    end
    self.fCurLifeValue = newhp
    local percent = self.fCurLifeValue / self.fullHP * 100
    self.hp:setPercentage(percent)
end

--英雄的技能造成的伤害
function WorldBoss:decreaseHp(hitType,decreaseHp,if_crit)
    bossDecreaseTotalHP = bossDecreaseTotalHP + decreaseHp
    self:addDecreaseHPLabel(decreaseHp,if_crit)
    if if_crit then self:addCritLabel() end
    self:runBeHitAnimation(hitType)

    local newhp = self.fCurLifeValue - decreaseHp
    if newhp <= 0 then
        newhp = self.fullHP
        local frames,animation = AnimationFactory.getAddBloodAnimation()
        local addBloodSprite = display.newSprite(frames[1],0,30)
        self:addChild(addBloodSprite,50003)
        transition.execute(addBloodSprite,CCAnimate:create(animation),{
            onComplete = function()
                addBloodSprite:removeFromParentAndCleanup(true)
            end,
        })
    end
    self.fCurLifeValue = newhp
    local percent = self.fCurLifeValue / self.fullHP * 100
    self.hp:setPercentage(percent)

    --[[
    if bossDecreaseTotalHP>200000 then
        self.hp:setPercentage(80)
    elseif bossDecreaseTotalHP>100000 then
        self.hp:setPercentage(85)
    elseif bossDecreaseTotalHP>50000 then
        self.hp:setPercentage(90)
    elseif bossDecreaseTotalHP>10000 then
        self.hp:setPercentage(95)
    else
        self.hp:setPercentage(100)
    end]]


end

function WorldBoss:doStandAnimation()
    self:stopBossAction()
    local animation = nil
    self.countOrStandCounter = self.countOrStandCounter + 1
    if self.countOrStandCounter<5 then
        animation = AnimationFactory.getBossStandAnimation(true,self.attackerProperties.body_id)
    else
        self.countOrStandCounter = 0
        animation = AnimationFactory.getBossCallAnimation(true,self.attackerProperties.body_id)
    end
    if animation~=nil then 
        transition.execute(self.mainSprite, CCAnimate:create(animation), {
            onComplete = function()
                self:doStandAnimation()
            end,
        })
    end
end


function WorldBoss:doWalkAnimation(direction)
    if self.forceWalkStatus then
        return
    end
    self:stopBossAction()
    local animation = AnimationFactory.getBossAnimation(direction,self.attackerProperties.body_id,self.attackerProperties.walk_animation_time)
    if animation~=nil then 
        transition.execute(self.mainSprite, CCAnimate:create(animation), {
            onComplete = function()
                local wayPointCount = self._waypoints:count()
                local waypoint = tolua.cast(self._waypoints:objectAtIndex((self.curWaypoint+1>wayPointCount-1) and (wayPointCount-1) or (self.curWaypoint+1) ),"WayPoint")
                self:doWalkAnimation(waypoint.direction)
            end,
        })
    end
end


function WorldBoss:playWalkToTowerAnimation(direction)
    self:stopBossAction()
    local animation = AnimationFactory.getBossAnimation(direction,self.attackerProperties.body_id,self.attackerProperties.walk_animation_time)
    if animation~=nil then 
        transition.execute(self.mainSprite, CCAnimate:create(animation), {
            onComplete = function()
                 self:playWalkToTowerAnimation(direction)
            end,
        })
    end
end

function WorldBoss:addGoBackEffect(effectName)
    local frames,animation,offset = AnimationFactory.getBuffAnimation(effectName)
    self.goBackEffect = display.newSprite(frames[1])
    self.goBackEffect:setPosition(0,offset)
    self:addChild(self.goBackEffect,-1)
    self.goBackEffect:setScale(self.attackerProperties.scale)
    self.goBackEffect:playAnimationForever(animation)
end

--提供给41009秀丽乾坤的接口，delayTime秒钟之后回到种buff的原地
function WorldBoss:goBackToEverPosition(delayTime)
    local posX,posY = self:getPosition()
    local wayPointIndex = self.curWaypoint

    local action = transition.sequence({
        CCDelayTime:create(delayTime),
        CCCallFunc:create(function()
            self:stopWalk()
            local bezier = ccBezierConfig()
            bezier.controlPoint_1 = ccp(self:getPositionX(),self:getPositionY())
            bezier.controlPoint_2 = ccp((self:getPositionX()+posX)/2,posY+150)
            bezier.endPosition = ccp(posX,posY)
            local bezierTo = CCBezierTo:create(1, bezier)

            transition.execute(self,bezierTo,{
                onComplete = function()
                    local waypoint = tolua.cast(self._waypoints:objectAtIndex(self.curWaypoint),"WayPoint")
                    self:doWalkAnimation(waypoint.direction)
                    self.curWaypoint = wayPointIndex
                    self:startWalk()
                    if pauseStatus then
                        self:pause(true)
                    end
                end,
            })
            self.goBackEffect:removeFromParentAndCleanup(true)
        end),
    })
    self.goBackEffect:runAction(action)
end


--当前怪物受buff影响减速,不能叠加
function WorldBoss:slowDown(time,speedSlowDown)
    if self.ifSlowDown == true then
        return
    end
    self.ifSlowDown = true
    self:stopWalk()
    self.move_speed = self.attackerProperties.move_speed + self.attackerProperties.move_speed*speedSlowDown
    self.walk_animation_time = 1
    local waypoint = tolua.cast(self._waypoints:objectAtIndex(self.curWaypoint),"WayPoint")
    self:doWalkAnimation(waypoint.direction)
    self:startWalk()


    local frames,animation,offset = AnimationFactory.getBuffAnimation("slow")
    self.slowSprite = display.newSprite(frames[1])
    self.slowSprite:setPosition(0,offset)
    self.slowSprite:setScale(self.attackerProperties.scale)
    self.slowSprite:setTag(186)
    self:addChild(self.slowSprite)
    self.slowSprite:playAnimationForever(animation)

    local action = transition.sequence({
        CCDelayTime:create(time),
        CCCallFunc:create(function()
            self:stopWalk()
            self.move_speed = self.attackerProperties.move_speed
            self.walk_animation_time = self.attackerProperties.walk_animation_time
            local waypoint = tolua.cast(self._waypoints:objectAtIndex(self.curWaypoint),"WayPoint")
            self:doWalkAnimation(waypoint.direction)
            self:startWalk()
            if self.ifPause then
                self:pause(false)
            end
            if pauseStatus then
                self:pause(true)
            end
            local spriteTag = self:getChildByTag(100)
            if spriteTag~=nil then
                transition.pauseTarget(self)
            end
            if pauseStatus then
                self:pause(true)
            end
            self.ifSlowDown = false
            self.slowSprite:removeSelf()
        end),
    })
    self.slowSprite:runAction(action)
end

function WorldBoss:continueWalk()
    local lastDistance = nil 
    for i=0,self._waypoints:count()-1 do
        local curWayPoint = tolua.cast(self._waypoints:objectAtIndex(i), "WayPoint")
        if lastDistance~=nil then
            local distance = ccpDistance(ccp(curWayPoint.position.x,curWayPoint.position.y),ccp(self:getPositionX(),self:getPositionY()))
            if distance<lastDistance then
                lastDistance = distance
                self.curWaypoint = i
            end
        else
            local distance = ccpDistance(ccp(curWayPoint.position.x,curWayPoint.position.y),ccp(self:getPositionX(),self:getPositionY()))
            lastDistance = distance
            self.curWaypoint = i
        end
    end
    self:startWalkForce()
    self:doWalkAnimation(true)
    --self.bloodAddSprite:removeFromParentAndCleanup(true)
end

--添加眩晕效果
function WorldBoss:addDizzyEffect(delayTime,effectName)
    local spriteTag = self:getChildByTag(100)
    if spriteTag~=nil then
        spriteTag:removeFromParentAndCleanup(true)
    end
    self.ifSlowDown = true

    self:stopWalk()
    self.move_speed = 1
    self.walk_animation_time = 2
    local waypoint = tolua.cast(self._waypoints:objectAtIndex(self.curWaypoint),"WayPoint")
    self:doWalkAnimation(waypoint.direction)
    self:startWalk()

    local frames,animation,offset = AnimationFactory.getBuffAnimation(effectName)
    local dizzySprite = display.newSprite(frames[1])
    dizzySprite:setPosition(0,offset)
    self:addChild(dizzySprite,-1)
    dizzySprite:setScale(self.attackerProperties.scale)
    dizzySprite:playAnimationForever(animation)
    dizzySprite:setTag(100)

    local action = transition.sequence({
        CCDelayTime:create(delayTime),
        CCCallFunc:create(function()
            self:stopWalk()
            self.move_speed = self.attackerProperties.move_speed
            self.walk_animation_time = self.attackerProperties.walk_animation_time
            local waypoint = tolua.cast(self._waypoints:objectAtIndex(self.curWaypoint),"WayPoint")
            self:doWalkAnimation(waypoint.direction)
            self:startWalk()
            if pauseStatus then
                self:pause(true)
            end
            if self.ifPause then
                self:pause(false)
            end
            dizzySprite:removeFromParentAndCleanup(true)
            self.ifSlowDown = false
        end),
    })
    dizzySprite:runAction(action) 
end

function WorldBoss:stopWalk()
    if self.forceWalkStatus then
        return
    end
    if self.walkStatus then
        self:stopActionByTag(99)
        self.walkStatus = false
        --print("停止走路了")
    else
        --print("不能重复停止")
    end
end

function WorldBoss:startWalk()
    if self.forceWalkStatus then
        return
    end
    if not self.walkStatus then
        self:walk()
        self.walkStatus = true
        --print("开始走路了")
    else
        --print("不能重复走路")
    end
end

function WorldBoss:stopWalkForce()
    self.forceWalkStatus = true
    self:stopActionByTag(99)
    self.walkStatus = false
    --[[
    local frames,animation = AnimationFactory.getAddBloodAnimation()
    self.bloodAddSprite = display.newSprite(frames[1],5,90)
    self:addChild(self.bloodAddSprite)
    self.bloodAddSprite:playAnimationForever(animation)]]
end

function WorldBoss:startWalkForce()
    self.forceWalkStatus = false
    self:walk()
    self.walkStatus = true
end

function WorldBoss:stopBossAction()
    if self.forceWalkStatus then
        return
    end
    self.mainSprite:stopAllActions()
end

function WorldBoss:resetPath()
    if self.way==1 then 
        self.way=5
        self._waypoints:removeAllObjects()
        local paths = GameInstance:sharedInstance().paths
        self.path = tolua.cast(GameInstance:sharedInstance().paths:objectAtIndex((self.way-1)),"CCArray")
        for i=0,self.path:count()-1 do
            local curWayPoint = tolua.cast(self.path:objectAtIndex(i), "WayPoint")
            local wp = WayPoint:new(curWayPoint.position,curWayPoint.isFlipX,curWayPoint.direction,curWayPoint.underSea)
            self._waypoints:addObject(wp)
        end
    elseif self.way==2 then
        self.way=6
        self._waypoints:removeAllObjects()
        local paths = GameInstance:sharedInstance().paths
        self.path = tolua.cast(GameInstance:sharedInstance().paths:objectAtIndex((self.way-1)),"CCArray")
        for i=0,self.path:count()-1 do
            local curWayPoint = tolua.cast(self.path:objectAtIndex(i), "WayPoint")
            local wp = WayPoint:new(curWayPoint.position,curWayPoint.isFlipX,curWayPoint.direction,curWayPoint.underSea)
            self._waypoints:addObject(wp)
        end
    elseif self.way==3 then
        self.way=5
        self._waypoints:removeAllObjects()
        local paths = GameInstance:sharedInstance().paths
        self.path = tolua.cast(GameInstance:sharedInstance().paths:objectAtIndex((self.way-1)),"CCArray")
        for i=0,self.path:count()-1 do
            local curWayPoint = tolua.cast(self.path:objectAtIndex(i), "WayPoint")
            local wp = WayPoint:new(curWayPoint.position,curWayPoint.isFlipX,curWayPoint.direction,curWayPoint.underSea)
            self._waypoints:addObject(wp)
        end
    elseif self.way==9 then
        self.way=13
        self._waypoints:removeAllObjects()
        local paths = GameInstance:sharedInstance().paths
        self.path = tolua.cast(GameInstance:sharedInstance().paths:objectAtIndex((self.way-1)),"CCArray")
        for i=0,self.path:count()-1 do
            local curWayPoint = tolua.cast(self.path:objectAtIndex(i), "WayPoint")
            local wp = WayPoint:new(curWayPoint.position,curWayPoint.isFlipX,curWayPoint.direction,curWayPoint.underSea)
            self._waypoints:addObject(wp)
        end
    elseif self.way==10 then
        self.way=14
        self._waypoints:removeAllObjects()
        local paths = GameInstance:sharedInstance().paths
        self.path = tolua.cast(GameInstance:sharedInstance().paths:objectAtIndex((self.way-1)),"CCArray")
        for i=0,self.path:count()-1 do
            local curWayPoint = tolua.cast(self.path:objectAtIndex(i), "WayPoint")
            local wp = WayPoint:new(curWayPoint.position,curWayPoint.isFlipX,curWayPoint.direction,curWayPoint.underSea)
            self._waypoints:addObject(wp)
        end
    elseif self.way==11 then
        self.way=13
        self._waypoints:removeAllObjects()
        local paths = GameInstance:sharedInstance().paths
        self.path = tolua.cast(GameInstance:sharedInstance().paths:objectAtIndex((self.way-1)),"CCArray")
        for i=0,self.path:count()-1 do
            local curWayPoint = tolua.cast(self.path:objectAtIndex(i), "WayPoint")
            local wp = WayPoint:new(curWayPoint.position,curWayPoint.isFlipX,curWayPoint.direction,curWayPoint.underSea)
            self._waypoints:addObject(wp)
        end
    end
    self.curWaypoint = 0
end

function WorldBoss:getTextureCenterPoint(posX,posY)
    local size = self.mainSprite:getCascadeBoundingBox().size
    --print("怪物的宽高",size.width, size.height)
    local anchorPoint = self.mainSprite:getAnchorPoint()
    local textureCenterX = posX + (0.5-anchorPoint.x)*size.width
    local textureCenterY = posY + (0.5-anchorPoint.y)*size.height
    return textureCenterX,textureCenterY,size
end

--获取贴图中心点在世界坐标的位置
function WorldBoss:getTextureCenterPointInWorld(posX,posY)
    local size = self.mainSprite:getCascadeBoundingBox().size
    --print("怪物的宽高",size.width, size.height)
    local anchorPoint = self.mainSprite:getAnchorPoint()
    local textureCenterX = posX + (0.5-anchorPoint.x)*size.width
    local textureCenterY = posY + (0.5-anchorPoint.y)*size.height
    if textureCenterY>display.cy then
        textureCenterY = textureCenterY -160
    end
    return textureCenterX,textureCenterY,size
end

function WorldBoss:isBigBoss()
   return true
end

function WorldBoss:addBubbleAndWords(word)
    local parent =self:getParent()
    if parent~=nil then
        self.bubble = display.newSprite("ui/public/public_dialog_box.png",self:getPositionX()+60,self:getPositionY()+150)
        parent:addChild(self.bubble,1000)
        self.words = ui.newTTFLabel({
                text = word,
                font = "Arial",
                color = ccc3(0, 0, 0),
                x = -40,
                y = 10,
                size = 14,
                dimensions = CCSize(135,120),
                align = ui.TEXT_ALIGN_LEFT
        })
        self.words:setAnchorPoint(ccp(0,0))
        self.bubble:addChild(self.words)
    end
end

function WorldBoss:removeBubble()
    if self.bubble then
        self.bubble:removeSelf()
    end
end

function WorldBoss:frozen()
    --local frames,animation = AnimationFactory.getFrozenTowerAnimation()
    --self.freezSprite = display.newSprite(frames[10],5,100)
    --self.freezSprite:setScale(0.9)
    --self:addChild(self.freezSprite,50003)

    self.freezSprite = display.newProgressTimer("seal_10.png",kCCProgressTimerTypeBar)
    self.freezSprite:setPosition(5,100)
    self.freezSprite:setMidpoint(CCPointMake(0, 0))
    self.freezSprite:setBarChangeRate(CCPointMake(0, 1))
    self.freezSprite:setPercentage(0)
    self:addChild(self.freezSprite,50003)

    self.freezSprite:runAction(CCProgressTo:create(7, 100))

    --self.freezSprite:playAnimationOnce(animation)
end


return WorldBoss