require ("config")
local scheduler = require(cc.PACKAGE_NAME .. ".scheduler")
local Text = require("app.ui.Text")
local MathUtil = require("app.common.MathUtil")
local AnimationFactory = require("app.factory.AnimationFactory")
local Buff = require("app.actor.Buff")
Attributes    = require("app.common.Attributes")
--local beauty_skill_list = require("app.client.desc.beauty_skill")
local DebugUtil = require("app.common.DebugUtil")
local CommUtil = require("app.common.CommUtil")
local BattleUtil = require("app.new.BattleUtil")

BEAUTY_SKILLS={
    {40001,22,"channel_"},
    {40002,17,"channel_"},
    {40003,1,"big"},
    {40004,7,"cast_"},
    {42007,21,"channel_"},
    --新妖姬技能---------------------------------------------------------------------
    {"40005",23,"channel_"},
    {"40007",26,"cast_"},
    {"40008",21,"cast_"},
    {"40009",16,"cast_"},
    {"40010",23,"cast_"},
    {"40011",24,"channel_"},
    {"40012",24,"cast_"},
    {"40015",25,"channel_"},
    {"40016",18,"channel_"},
    --------------------------------------------------------------------------------
}

local BeautySkill = class("BeautySkill",function(parent,skill_id,releasePoint)
	local node = display.newNode()
	return node
end)

function BeautySkill:ctor(parent,skill_id,releasePoint)
	
	self.damageMonsters = {}
	self.buffMonsters = {}

	self.parent = parent
	
	self.releasePoint = releasePoint
	self.skill_id = skill_id
	self.parent = parent

	if tostring(skill_id) ~= "40009" then
		self.beautySkillRigion = require("app.region.BeautySkill"..self.skill_id.."Rigion")
	end
	self.frameIndex = 0
	self.frameNum = self:getFrameNum(skill_id)
	--print("self.skillId",skill_id,self.frameNum)
	self.removeTag = false

    --self.skillProperties = beauty_skill_list[skill_id..""]
    self.skillProperties = Attributes.get_real_beauty_skill(skill_id.."")

    if self.skillProperties == nil then return end
    
    self.ifDamage = true
	self.damageType = self.skillProperties.damage.damage_type
	
	local ifMultipleDamage = function(sId)
		local ids = {"40001","42007", "40005","40007","40011","40015","40016","40010","40012"}
		for i=1,#ids do
			if ids[i]==tostring(sId) then
				return true
			end
		end
		return false
	end
	if ifMultipleDamage(self.skill_id) then
		self.mutipleDamage = true--可以重复伤害的技能
	else
		self.mutipleDamage = false
	end

	self:makeSkillAndRun(skill_id)--播放技能动画

	if self.skillProperties.sound~=nil then
		CommUtil.playSound("music/soundeffect/"..self.skillProperties.sound..".mp3")
	else
		CommUtil.playSound("music/soundeffect/".."default_skill_sound.mp3")
	end
	
end

function BeautySkill:getFrameNum(skill_id)
	for i=1,#BEAUTY_SKILLS do
		local skill = BEAUTY_SKILLS[i]
		if (skill[1].."")== (skill_id.."") then
			return skill[2]
		else
		end
	end
end

function BeautySkill:checkHit()
	if self.frameIndex==nil or self.frameStep==nil then return end
	self.frameIndex = self.frameIndex + self.frameStep
	if self.frameIndex>self.frameNum then
		self.frameIndex = self.frameNum
	end
	local curRigion = self.beautySkillRigion["skill"..self.skill_id.."_"..self.frameIndex.."_"]
	if curRigion==nil then
		return
	end
	for i = 0, GameInstance:sharedInstance().monsters:count()-1 do
		local monster = GameInstance:sharedInstance().monsters:objectAtIndex(i)
		
		self:doBuffEffect(monster,curRigion)

		if self.ifDamage then
			if not self:ifHasDamaged(monster) then
				self:doDamageForEachMonster(monster,curRigion)
			else
				--print("海，哥们，你要打我几次，打一次就够了三")
			end
		else
			--print("海，哥们，此技能没有伤害，只加buff哟")
		end
	end
end

function BeautySkill:ifAddBuff(monster,bType)
	for i=1,#self.buffMonsters do
		local curMonster = self.buffMonsters[i][1]
		local buffType = self.buffMonsters[i][2]
		if curMonster==monster and buffType== bType then
			return true
		end
	end
	return false
end

function BeautySkill:ifHasDamaged(monster)
	if self.mutipleDamage then return false end
	for i=1,#self.damageMonsters do
		local curMonster = self.damageMonsters[i]
		if curMonster==monster then
			return true
		end
	end
	return false
end

function BeautySkill:iteratorEachBuff(buff,buffType,monster,curRigion)
	if buffType==6 then
		for i=0,GameInstance:sharedInstance().towers:count()-1 do
			local tower = GameInstance:sharedInstance().towers:objectAtIndex(i)
			if not tower:ifAddBuff(self.skill_id) then
				tower:addBuff(self.skill_id,Buff.new(buff))
			end
		end
	else
		local centerX,centerY,textTureSize =  monster:getCenterPointInWorld(monster:getPositionX(),monster:getPositionY())
		for i=1,#curRigion do
			local circle = curRigion[i]
			local worldPoint = self.skillSprite:convertToWorldSpace(ccp(circle[1],circle[2]))
			local distance = ccpDistance(ccp(centerX,centerY),worldPoint)
			local edge = (10+circle[3]*self.skillSprite:getScale())
			if distance<(edge) then
				if not monster:ifAddBuff(self.skill_id,buffType) then
			    	self.buffMonsters[#self.buffMonsters+1] = {monster,buffType}
			       	monster:addBuff(self.skill_id,Buff.new(buff))
			        return
			    end
			end
		end
	end
end

--Dot是战斗力最高的英雄的攻击力*bufValue
--DOT_HP%是秒怪物满血状态的百分比
--1,DOT;2,免伤；3，移动速度；4，攻击速度；5.生命值上限；6，攻击力；7，暴击率；8，技能触发几率；9，眩晕；10，变形；11，击退；12，牵引；13，石化；14，印记传送；15,DOT_HP%
function BeautySkill:doBuffEffect(monster,curRigion)
	if self.skillProperties==nil then return end
	local buffs = self.skillProperties.buffs
	if buffs~=nil then
		for i=1,#buffs do
			local buff = buffs[i]
			local buffType = buff.buff_type
			if not self:ifAddBuff(monster,buffType) then
				self:iteratorEachBuff(buff,buffType,monster,curRigion)
			end
		end
	end
end

--获取主将所在塔
function BeautySkill:getLeadHeroTower(leadHeroId)
	for i=0,GameInstance:sharedInstance().towers:count()-1 do
		local tower = GameInstance:sharedInstance().towers:objectAtIndex(i)
		if tower:getHeroId()==leadHeroId then
			return tower
		end
	end
	return nil
end

function BeautySkill:doDamageForEveryMonster(monster)

	if monster.dead then return end

	self.damageMonsters[#self.damageMonsters+1] = monster

  
	         local damage = SKILL_ATTACKPOWER*self.skillProperties.damage.damage_coeff
	        monster:decreaseHpBySkill(damage,false)

end

function BeautySkill:doDamageForEachMonster(monster,curRigion)
	local distance = ccpDistance(self.releasePoint,ccp(monster:getPositionX(),monster:getPositionY()))
	if distance >150 then
		--print("距离太远了")
		return
	else
		--print("这个距离合适哟")
	end
	local centerX,centerY,textTureSize =  monster:getCenterPointInWorld(monster:getPositionX(),monster:getPositionY())
	for i=1,#curRigion do
        local circle = curRigion[i]
        local worldPoint = self.skillSprite:convertToWorldSpace(ccp(circle[1],circle[2]))
        local distance = ccpDistance(ccp(centerX,centerY),worldPoint)
        local offset = 0
        local edge = offset+(circle[3]*self.skillSprite:getScale())
        if self.skill_id~=40020 then offset=10 end
        if distance<(edge) then

        	self.damageMonsters[#self.damageMonsters+1] = monster

		    local damage = SKILL_ATTACKPOWER*self.skillProperties.damage.damage_coeff
	        

	        --
	        if (gPlayer.data.pass_id <= 60009) then
	        		damage =  damage * 2
	        end
	        if (self.skill_id==40008) then
	        	print("妖姬技能40008造成的伤害是1",SKILL_ATTACKPOWER)
	        	print("妖姬技能40008造成的伤害是2",damage)
	        else
	        	print("妖姬技能造成的伤害是",damage)
	        end
	        monster:decreaseHpBySkill(damage,false)
            return
        end
    end
end

function BeautySkill:resetCheckHitSchedule(cycleTime,totalTime)
	--[[]]
	if self.handle~=nil then
		scheduler.unscheduleGlobal(self.handle)
		self.handle = nil
	end
	if totalTime==nil then return end
	self.checkEachTime = cycleTime
	self.totalTime = totalTime
	self.frameStep = math.ceil(cycleTime/totalTime * self.frameNum)
	self.frameIndex = 0
	self:checkHit()
	self.handle = scheduler.scheduleGlobal(handler(self, self.checkHit), self.checkEachTime)
end

function BeautySkill:clean()
end


--清除自己
function BeautySkill:clean()
	self.removeTag = true
	if self.handle~=nil then
		scheduler.unscheduleGlobal(self.handle)
		self.handle = nil
	end
	for i = 0, GameInstance:sharedInstance().towers:count()-1 do
		local tower = GameInstance:sharedInstance().towers:objectAtIndex(i)
		tower:removeBuffID(self.skill_id)
	end
	for i = 0, GameInstance:sharedInstance().monsters:count()-1 do
		local monster = GameInstance:sharedInstance().monsters:objectAtIndex(i)
		monster:removeBuffID(self.skill_id)
	end
	self:removeFromParentAndCleanup(true)
end


function BeautySkill:shotPlace(orix, oriy, destx, desty)

 local shootVector = ccpSub(ccp(destx,desty),ccp(orix,oriy+60))
        local shootAngle = -1*ccpToAngle(shootVector)*57.29577951
        local v = ccpForAngle(shootAngle)
        --print("aaaasda",v.x,v.y)

        local skillSprite = display.newSprite("#bullet_42012_01.png",orix,oriy+60):addTo(self)
            skillSprite:setAnchorPoint(ccp(0.7,0.5))
            skillSprite:setRotation(shootAngle)
            
            local action = CCSpawn:createWithTwoActions(CCMoveTo:create(0.5,ccp(destx, desty)),CCAnimate:create(AnimationFactory.getSkillBulletAnimation(42012,10,0.5)))
            
            transition.execute(skillSprite,action,{
              easing = "exponentialIn",
          onComplete = function()
            local size = skillSprite:getContentSize()
            local endPoint = skillSprite:convertToWorldSpace(ccp(size.width-size.width/3,size.height/2))
              skillSprite:removeFromParentAndCleanup(true)
                skillSprite = display.newSprite("#bullet_42012_end_01.png",endPoint.x,endPoint.y):addTo(self)
                transition.execute(skillSprite,CCAnimate:create(AnimationFactory.getEndAnimation(42012,5,0.4)),{
              onComplete = function()
                skillSprite:removeFromParent()
                  end
          })
              end
      })
end


function BeautySkill:runChannelRepeatAnimation(sprite,skill_id,owner,target,totalFrames)
    transition.execute(sprite,CCAnimate:create(AnimationFactory.getSkillChannelAnimation(skill_id,totalFrames,self.single_time)),{
    	onComplete = function()
    		self.cycle_time =  self.cycle_time - 1
    		if self.cycle_time==0 then
    			self:clean()
	    	else
	    		self:runChannelRepeatAnimation(sprite,skill_id,owner,target,totalFrames)
	    		if skill_id=="41010" then
	    			self:resetCheckHitSchedule(0.4,self.single_time)
	    		elseif skill_id=="42007" then
	    			self:resetCheckHitSchedule(0.3,self.single_time)
	    		else
	    			self:resetCheckHitSchedule(self.single_time/totalFrames,self.single_time)
	    		end
	    	end
    	end
    })
end

function BeautySkill:makeSkillAndRun(skill_id)
	if skill_id==42007 then
		self.cycle_time = self.skillProperties.effect.cycle_time
			self.single_time = self.skillProperties.effect.single_time
    		self.skillSprite = display.newSprite("#channel_"..skill_id.."_01.png",self.targetPoint.x+30,self.targetPoint.y+50):addTo(self)
    		self.skillSprite:setScale(1.1)
			self:resetCheckHitSchedule(0.3,1)
            self:runChannelRepeatAnimation(self.skillSprite,skill_id,owner,target,21)
	elseif skill_id==40001 then
			--self.skillProperties = beauty_skill_list["40001"]
			self.skillProperties = Attributes.get_real_beauty_skill("40001")
    		self.skillSprite = display.newSprite("#channel_"..skill_id.."_01.png",self.releasePoint.x+10,self.releasePoint.y+60):addTo(self)
    		--self.skillSprite:setScale(0.4)
			self.cycle_time = self.skillProperties.effect.cycle_time
            self:runRepeatAnimation(self.skillSprite,skill_id,22,0.3)
    elseif skill_id == 40002 then
			--self.skillProperties = Attributes.get_real_beauty_skill("40002")
    		--self.skillSprite = display.newSprite("#channel_"..skill_id.."_01.png",self.releasePoint.x,self.releasePoint.y):addTo(self)
    		--self.cycle_time = self.skillProperties.effect.cycle_time
            --self:runRepeatAnimation(self.skillSprite,skill_id,17,0.2)
	elseif skill_id == 40004 then

		 --保留原来所有的逻辑处理
    		self.skillSprite = display.newSprite("#cast_".."40014".."_01.png",0,self.releasePoint.y+20):addTo(self)
			local action = CCSpawn:createWithTwoActions(
			CCAnimate:create(AnimationFactory.getSkillCastAnimation("40014",7,1)),
			CCMoveTo:create(1, ccp(display.right,self.releasePoint.y+20)))
			self:resetCheckHitSchedule(0.1,1)
			transition.execute(self.skillSprite,action,{
		    	onComplete = function()
		    		--self.skillSprite:removeSelf()
		    	end
		    })
		    self:performWithDelay(function()
		    	self.skillSprite = display.newSprite("#cast_".."40014".."_01.png",0,self.releasePoint.y+20):addTo(self)
				local action = CCSpawn:createWithTwoActions(
				CCAnimate:create(AnimationFactory.getSkillCastAnimation("40014",7,1)),
				CCMoveTo:create(1, ccp(display.right,self.releasePoint.y+20)))
				self:resetCheckHitSchedule(0.1,1)
				transition.execute(self.skillSprite,action,{
				    onComplete = function()
				    --	self:clean()
				    end
				})
		    end,0.6)
		    self:performWithDelay(function()
		    	self.skillSprite = display.newSprite("#cast_".."40014".."_01.png",0,self.releasePoint.y+20):addTo(self)
				local action = CCSpawn:createWithTwoActions(
				CCAnimate:create(AnimationFactory.getSkillCastAnimation("40014",7,1)),
				CCMoveTo:create(1, ccp(display.right,self.releasePoint.y+20)))
				self:resetCheckHitSchedule(0.1,1)
				transition.execute(self.skillSprite,action,{
				    onComplete = function()
				    	self:clean()
				    end
				})
		    end,1.2)
			--[[CommUtil.playSound("music/soundeffect/q2.mp3") 
			for i=0,GameInstance:sharedInstance().towers:count()-1 do
        		local tower = GameInstance:sharedInstance().towers:objectAtIndex(i)
        		self:shotPlace(display.right-100, display.bottom+50, tower:getPositionX(),tower:getPositionY())
	    	end]]
			
	elseif skill_id == 40005 then
    		self.skillSprite = display.newSprite("#channel_"..skill_id.."_01.png",self.releasePoint.x,self.releasePoint.y+20):addTo(self)
    		--self.skillSprite:setScale(0.7)
    		self:resetCheckHitSchedule(0.2,1)
			transition.execute(self.skillSprite,CCAnimate:create(AnimationFactory.getSkillChannelAnimation(skill_id,23,1)),{
		    	onComplete = function()
		    		self:clean()
		    	end
		    })
	elseif skill_id == 40007 then
    		self.skillSprite = display.newSprite("#cast_"..skill_id.."_01.png",self.releasePoint.x,self.releasePoint.y+20):addTo(self)
    		self.skillSprite:setScale(1.2)
    		self:resetCheckHitSchedule(0.05,1)
			transition.execute(self.skillSprite,CCAnimate:create(AnimationFactory.getSkillCastAnimation(skill_id,26,1)),{
		    	onComplete = function()
		    		self:clean()
		    	end
		    })
	elseif skill_id == 40008 then
    		self.skillSprite = display.newSprite("#cast_"..skill_id.."_01.png",self.releasePoint.x,self.releasePoint.y+90):addTo(self)
    		self.skillSprite:setScale(1.2)
    		self:resetCheckHitSchedule(0.1,1.5)
    		self:performWithDelay(function()
    			CommUtil.vibrator(self.parent)
    			end, 0.6)
			transition.execute(self.skillSprite,CCAnimate:create(AnimationFactory.getSkillCastAnimation(skill_id,21,1.5)),{
		    	onComplete = function()
		    		self:clean()
		    	end
		    })
	elseif skill_id == 40009 then
			self.maskSprite = CommUtil.addMaskLayer(self.parent)
			self.parent:setOpacity(0)
    		self.skillSprite = display.newSprite("#cast_"..skill_id.."_01.png",self.releasePoint.x,self.releasePoint.y+30):addTo(self)
    		--self.skillSprite:setScale(0.7)
			transition.execute(self.skillSprite,CCAnimate:create(AnimationFactory.getSkillCastAnimation(skill_id,16,0.8)),{
		    	onComplete = function()
		    		self.maskSprite:removeFromParentAndCleanup(true)
		    		self.parent:setOpacity(255)
		    		self:clean()
		    	end
		    })
		    for i=0,GameInstance:sharedInstance().monsters:count()-1 do
				local monster = GameInstance:sharedInstance().monsters:objectAtIndex(i)
				if self.skillProperties==nil then return end
				self:doDamageForEveryMonster(monster)
				local buffs = self.skillProperties.buffs
				if buffs~=nil then
					for i=1,#buffs do
						local buff = buffs[i]
						local buffType = buff.buff_type
						monster:addBuff(self.skill_id,Buff.new(buff))
					end
				end
			end
	elseif skill_id == 40010 then
    		self.skillSprite = display.newSprite("#cast_"..skill_id.."_01.png",self.releasePoint.x,self.releasePoint.y+20):addTo(self)
    		--self.skillSprite:setScale(0.5)
    		self:resetCheckHitSchedule(0.1,0.8)
			transition.execute(self.skillSprite,CCAnimate:create(AnimationFactory.getSkillCastAnimation(skill_id,23,0.8)),{
		    	onComplete = function()
		    		self:clean()
		    	end
		    })
	elseif skill_id == 40011 then--只是加buff
    		self.skillSprite = display.newSprite("#channel_"..skill_id.."_01.png",self.releasePoint.x,self.releasePoint.y+20):addTo(self)
    		--self.skillSprite:setScale(0.5)
    		self:resetCheckHitSchedule(0.3,3)
			transition.execute(self.skillSprite,CCAnimate:create(AnimationFactory.getSkillChannelAnimation(skill_id,24,3)),{
		    	onComplete = function()
		    		self:clean()
		    	end
		    })
	elseif skill_id == 40012 then
    		self.skillSprite = display.newSprite("#cast_"..skill_id.."_01.png",self.releasePoint.x,self.releasePoint.y+20):addTo(self)
    		--self.skillSprite:setScale(0.5)
    		self:resetCheckHitSchedule(0.1,2)
			transition.execute(self.skillSprite,CCAnimate:create(AnimationFactory.getSkillCastAnimation(skill_id,24,2)),{
		    	onComplete = function()
		    		self:clean()
		    	end
		    })
	elseif skill_id == 40014 then
    		self.skillSprite = display.newSprite("#channel_"..skill_id.."_01.png",self.releasePoint.x,self.releasePoint.y+20):addTo(self)
    		--self.skillSprite:setScale(0.5)
			transition.execute(self.skillSprite,CCAnimate:create(AnimationFactory.getSkillChannelAnimation(skill_id,24,3)),{
		    	onComplete = function()
		    		self:clean()
		    	end
		    })
	elseif skill_id == 40015 then
    		self.skillSprite = display.newSprite("#channel_"..skill_id.."_01.png",self.releasePoint.x,self.releasePoint.y+20):addTo(self)
    		--self.skillSprite:setScale(0.5)
    		self:resetCheckHitSchedule(0.3,2)
			transition.execute(self.skillSprite,CCAnimate:create(AnimationFactory.getSkillChannelAnimation(skill_id,25,2)),{
		    	onComplete = function()
		    		self:clean()
		    	end
		    })
	elseif skill_id == 40016 then
    		self.skillSprite = display.newSprite("#channel_"..skill_id.."_01.png",self.releasePoint.x,self.releasePoint.y+20):addTo(self)
    		self.skillSprite:setScale(1.3)
    		self.cycle_time = self.skillProperties.effect.cycle_time
    		self:runRepeatAnimation(self.skillSprite,skill_id,18,0.1)
    end
end

function BeautySkill:runRepeatAnimation(sprite,skill_id,totalFrames,frequence)
    self:resetCheckHitSchedule(frequence,self.skillProperties.effect.single_time)
    transition.execute(sprite,CCAnimate:create(AnimationFactory.getSkillChannelAnimation(skill_id,totalFrames,self.skillProperties.effect.single_time)),{
    	onComplete = function()
    		self.cycle_time =  self.cycle_time - 1
    		if self.cycle_time==0 then
    			self:clean()
	    	else
	    		self:resetCheckHitSchedule(frequence,self.skillProperties.effect.single_time)
	    		self:runRepeatAnimation(sprite,skill_id,totalFrames,frequence)
	    	end
    	end
    })
end


function BeautySkill:doFullScreenEffect()
	if self.skillProperties==nil then return end
	local buffs = self.skillProperties.buffs
	if buffs~=nil then
		for i=1,#buffs do
			local buff = buffs[i]
			local buffType = buff.buff_type
			
			if buffType==6 then
				for i=0,GameInstance:sharedInstance().towers:count()-1 do
			        local tower = GameInstance:sharedInstance().towers:objectAtIndex(i)
			        if not tower:ifAddBuff(self.skill_id) then
			        	tower:addBuff(self.skill_id,Buff.new(buff))
			        end
			    end
			end
		end
	end
end

return BeautySkill