local Buff = require("app.actor.Buff")
local CommUtil = require("app.common.CommUtil")

local FullScreenSkill = class("FullScreenSkill", function()
    return display.newNode()
end)

function FullScreenSkill:ctor(gameScene,index,delay)
    self.gameScene = gameScene
    self.buffMonsters = {}
    if index==1 then

        CommUtil.playSound("music/flash.mp3")

        self.colorLayer = display.newColorLayer(ccc4(0,0,20,50)):addTo(self)

        self.gameScene:addLightBlot()
        self:schedule(handler(self,self.damageInterval),1)
        self:performWithDelay(function()
            self.gameScene:removeLightBlot()
            self:stopAllActions()
            self.colorLayer:removeSelf()
        end,delay)


    elseif index==2 then

            CommUtil.playSound("music/moon.mp3")

            --self.maskSprite = CommUtil.addMaskLayer(self.gameScene)
            self.colorLayer = display.newColorLayer(ccc4(0,0,0,100)):addTo(self)
            self.gameScene:setOpacity(0)
            local frames = display.newFrames("cast_40009_%02d.png", 1, 16)
            local animation = display.newAnimation(frames, 1.2 / 16)
            self.skillSprite = display.newSprite(frames[1],display.cx,display.cy):addTo(self)
            self.skillSprite:setScale(1.1)
            transition.execute(self.skillSprite,CCAnimate:create(animation),{
                onComplete = function()
                    self.skillSprite:removeFromParentAndCleanup(true)
                    self.gameScene:setOpacity(255)
                end
            })
            self:dingShen()
            self:schedule(handler(self,self.dingShen),1)
            self:performWithDelay(function()
                --self.maskSprite:removeFromParentAndCleanup(true)
                self.colorLayer:removeSelf()
                self:stopAllActions()
            end,delay)

    elseif index==3 then
        CommUtil.playSound("music/rain.mp3")
        self.particle = require("app.controllers.ParticleManager").new(self.gameScene,nil,1):addTo(self)
        self:performWithDelay(function()
            self:stopAllActions()
            self.particle:removeFromParentAndCleanup(true)
        end,delay)
        self:slowFullScreenMonster()
        self:schedule(handler(self,self.slowFullScreenMonster),1)
        self:slowFullScreenMonster()

    end
end

function FullScreenSkill:ifDamage(monster)
    for i=1,#self.buffMonsters do
        local curMonster = self.buffMonsters[i]
        if curMonster==monster then
            return true
        end
    end
    return false
end

function FullScreenSkill:damageInterval()
    for i=0,GameInstance:sharedInstance().monsters:count()-1 do
        local monster = GameInstance:sharedInstance().monsters:objectAtIndex(i)
        if not self:ifDamage(monster) then
            monster:decreaseHp("hunter",monster.fullHP*0.2,false)
            self.buffMonsters[#self.buffMonsters+1] = monster
        end
    end
end


function FullScreenSkill:dingShen()
    for i=0,GameInstance:sharedInstance().monsters:count()-1 do
        local monster = GameInstance:sharedInstance().monsters:objectAtIndex(i)
        local buffs = require("app.client.desc.skill")[tostring(3)].buffs
        if buffs~=nil and (not self:ifDamage(monster)) then
            for i=1,#buffs do
                local buff = buffs[i]
                local buffType = buff.buff_type
                monster:addBuff(40009,Buff.new(buff))
            end
            self.buffMonsters[#self.buffMonsters+1] = monster
        end
    end
end


function FullScreenSkill:slowFullScreenMonster()
    for i=0,GameInstance:sharedInstance().monsters:count()-1 do
        local monster = GameInstance:sharedInstance().monsters:objectAtIndex(i)
        local buffs = require("app.client.desc.skill")[tostring(4)].buffs
                if buffs~=nil and (not self:ifDamage(monster)) then
                    for i=1,#buffs do
                        local buff = buffs[i]
                        local buffType = buff.buff_type
                        monster:addBuff(4,Buff.new(buff))
                    end
                    self.buffMonsters[#self.buffMonsters+1] = monster
                end
    end
end

return FullScreenSkill
