require ("config")
local scheduler = require(cc.PACKAGE_NAME .. ".scheduler")
local Text = require("app.ui.Text")
local MathUtil = require("app.common.MathUtil")
local AnimationFactory = require("app.factory.AnimationFactory")
local Buff = require("app.actor.Buff")
local hero_skill_list = require("app.client.desc.hero_skill")
local DebugUtil = require("app.common.DebugUtil")
local CommUtil = require("app.common.CommUtil")
local BattleUtil = require("app.new.BattleUtil")

HERO_SKILLS={
    {"40001",30,"cast_"},
    {40002,17,"channel_"},
    {40003,10,"channel_"},
    {"40004",23,"cast_"},
    {"40005",11,"ending_"},
    {"40006",35,"channel_"},
}

local HeroSkill = class("HeroSkill",function(parent,skill_id,releasePoint)
	local node = display.newNode()
	return node
end)

function HeroSkill:ctor(parent,skill_id,releasePoint,owner)

    self.owner = owner
	
	self.damageMonsters = {}
	self.buffMonsters = {}

	self.parent = parent
	
	self.releasePoint = releasePoint
	self.skill_id = skill_id
	
	
	self.reareal_skill_id = real_skill_id
	self.parent = parent

	if tostring(skill_id) ~= "40009" then
		self.beautySkillRigion = require("app.region.HeroSkill"..self.skill_id.."Rigion")
	end
	self.frameIndex = 0
	self.frameNum = self:getFrameNum(skill_id)
	self.removeTag = false

    self.skillProperties = hero_skill_list[skill_id..""]

    if self.skillProperties == nil then return end
    
    self.ifDamage = true
	self.damageType = self.skillProperties.damage.damage_type
	
	local ifMultipleDamage = function(sId)
		local ids = {"40001","40002","40003","40004","40005","40006"}
		for i=1,#ids do
			if ids[i]==tostring(sId) then
				return true
			end
		end
		return false
	end
	if ifMultipleDamage(self.skill_id) then
		self.mutipleDamage = true--可以重复伤害的技能
	else
		self.mutipleDamage = false
	end

	self:makeSkillAndRun(skill_id)--播放技能动画
	if self.skillProperties.sound~=nil then
		CommUtil.playSound("music/"..self.skillProperties.sound..".mp3")
	else
		CommUtil.playSound("music/soundeffect/".."default_skill_sound.mp3")
	end
end

function HeroSkill:getFrameNum(skill_id)
	for i=1,#HERO_SKILLS do
		local skill = HERO_SKILLS[i]
		if (skill[1].."")== (skill_id.."") then
			return skill[2]
		else
		end
	end
end

function HeroSkill:checkHit()
	if self.frameIndex==nil or self.frameStep==nil then return end
	self.frameIndex = self.frameIndex + self.frameStep
	if self.frameIndex>self.frameNum then
		self.frameIndex = self.frameNum
	end
	local curRigion = self.beautySkillRigion["skill"..self.skill_id.."_"..self.frameIndex.."_"]
	if curRigion==nil then
		return
	end
	for i = 0, GameInstance:sharedInstance().monsters:count()-1 do
		local monster = GameInstance:sharedInstance().monsters:objectAtIndex(i)
		--print("check buff")
		self:doBuffEffect(monster,curRigion)

		if self.ifDamage then
			if not self:ifHasDamaged(monster) then
				self:doDamageForEachMonster(monster,curRigion)
			else
				--print("海，哥们，你要打我几次，打一次就够了三")
			end
		else
			--print("海，哥们，此技能没有伤害，只加buff哟")
		end
	end
end

function HeroSkill:ifAddBuff(monster,bType)
	for i=1,#self.buffMonsters do
		local curMonster = self.buffMonsters[i][1]
		local buffType = self.buffMonsters[i][2]
		if curMonster==monster and buffType== bType then
			return true
		end
	end
	return false
end

function HeroSkill:ifHasDamaged(monster)
	if self.mutipleDamage then return false end
	for i=1,#self.damageMonsters do
		local curMonster = self.damageMonsters[i]
		if curMonster==monster then
			return true
		end
	end
	return false
end

function HeroSkill:iteratorEachBuff(buff,buffType,monster,curRigion)
	if buffType==6 then
		for i=0,GameInstance:sharedInstance().towers:count()-1 do
			local tower = GameInstance:sharedInstance().towers:objectAtIndex(i)
			if not tower:ifAddBuff(self.skill_id) then
				tower:addBuff(self.skill_id,Buff.new(buff))
			end
		end
	else
		local centerX,centerY,textTureSize =  monster:getCenterPointInWorld(monster:getPositionX(),monster:getPositionY())
		for i=1,#curRigion do
			local circle = curRigion[i]
			local worldPoint = self.skillSprite:convertToWorldSpace(ccp(circle[1],circle[2]))
			local distance = ccpDistance(ccp(centerX,centerY),worldPoint)
			local edge = (10+circle[3]*self.skillSprite:getScale())
			if distance<(edge) then
				if not monster:ifAddBuff(self.skill_id,buffType) then
			    	self.buffMonsters[#self.buffMonsters+1] = {monster,buffType}
			    	monster:addBuff(self.skill_id,Buff.new(buff))
			        return
			    end
			end
		end
	end
end

--Dot是战斗力最高的英雄的攻击力*bufValue
--DOT_HP%是秒怪物满血状态的百分比
--1,DOT;2,免伤；3，移动速度；4，攻击速度；5.生命值上限；6，攻击力；7，暴击率；8，技能触发几率；9，眩晕；10，变形；11，击退；12，牵引；13，石化；14，印记传送；15,DOT_HP%
function HeroSkill:doBuffEffect(monster,curRigion)
	if self.skillProperties==nil then return end
	local buffs = self.skillProperties.buffs
	if buffs~=nil then
		for i=1,#buffs do
			local buff = buffs[i]
			local buffType = buff.buff_type

			if self:towerCanAttackMonster(monster) and (not self:ifAddBuff(monster,buffType)) then
				--print("buffType == "  .. buffType)
				self:iteratorEachBuff(buff,buffType,monster,curRigion)
			end
		end
	end
end

--获取主将所在塔
function HeroSkill:getLeadHeroTower(leadHeroId)
	for i=0,GameInstance:sharedInstance().towers:count()-1 do
		local tower = GameInstance:sharedInstance().towers:objectAtIndex(i)
		if tower:getHeroId()==leadHeroId then
			return tower
		end
	end
	return nil
end

function HeroSkill:doDamageForEachMonster(monster,curRigion)
	local distance = ccpDistance(self.releasePoint,ccp(monster:getPositionX(),monster:getPositionY()))
	if distance >150 then
		--print("距离太远了")
		return
	else
		--print("这个距离合适哟")
	end
	local centerX,centerY,textTureSize =  monster:getCenterPointInWorld(monster:getPositionX(),monster:getPositionY())
	for i=1,#curRigion do
        local circle = curRigion[i]
        local worldPoint = self.skillSprite:convertToWorldSpace(ccp(circle[1],circle[2]))
        local distance = ccpDistance(ccp(centerX,centerY),worldPoint)
        local offset = 0
        local edge = offset+(circle[3]*self.skillSprite:getScale())
        if self.skill_id~=40020 then offset=10 end
        if self:towerCanAttackMonster(monster) and distance<(edge) then------------------------在这里要计算伤害值
        	self.damageMonsters[#self.damageMonsters+1] = monster
	        
	        local crit_proc = BattleUtil.getCrit(self.owner,monster)--爆击率设定
		    local if_crit = false--是否爆击状态标志
		    local crit_proc_random = CPlusMathUtil:getRandomNum(100)/100
		    if crit_proc_random <= crit_proc then
		        if_crit = true
		    else
		    end
    		local damage = BattleUtil.getDamage(self.owner,monster)*self.skillProperties.damage.damage_coeff
    		if if_crit then damage = damage*2 end
    		--print("妖姬造成的伤害是",damage)
	        monster:decreaseHpBySkill(damage,if_crit)
            return
        end
    end
end

function HeroSkill:resetCheckHitSchedule(cycleTime,totalTime)
	if self.handle~=nil then
		scheduler.unscheduleGlobal(self.handle)
		self.handle = nil
	end
	if totalTime==nil then return end
	self.checkEachTime = cycleTime
	self.totalTime = totalTime
	self.frameStep = math.ceil(cycleTime/totalTime * self.frameNum)
	self.frameIndex = 0
	self:checkHit()
	self.handle = scheduler.scheduleGlobal(handler(self, self.checkHit), self.checkEachTime)
end

--清除自己
function HeroSkill:clean()
	self.removeTag = true
	if self.handle~=nil then
		scheduler.unscheduleGlobal(self.handle)
		self.handle = nil
	end
	for i = 0, GameInstance:sharedInstance().towers:count()-1 do
		local tower = GameInstance:sharedInstance().towers:objectAtIndex(i)
		tower:removeBuffID(self.skill_id)
	end
	for i = 0, GameInstance:sharedInstance().monsters:count()-1 do
		local monster = GameInstance:sharedInstance().monsters:objectAtIndex(i)
		monster:removeBuffID(self.skill_id)
	end
	self:removeFromParentAndCleanup(true)
end



function HeroSkill:run40003RepeatAnimation(sprite,skill_id)
	local frames = display.newFrames("channel_"..skill_id.."_%02d.png", 1,10)
    local animation = display.newAnimation(frames, self.skillProperties.effect.single_time / 10)
    transition.execute(sprite,CCAnimate:create(animation),{
	    onComplete = function()
	    	self.cycle_time =  self.cycle_time - 1
	    	if self.cycle_time==0 then
	    		local frames = display.newFrames("ending_"..skill_id.."_%02d.png", 1,4)
    			local animation = display.newAnimation(frames, 0.5 / 4)
	    		transition.execute(sprite,CCAnimate:create(animation),{
	    		onComplete = function()
            		self:clean()
	    		end,
				})
	    	else
	    		CommUtil.vibrator(display.getRunningScene())
	    		self:resetCheckHitSchedule(0.2,self.skillProperties.effect.single_time)
	    		self:run40003RepeatAnimation(sprite,skill_id)
	    	end
	    end,
	})
end


function HeroSkill:run40002RepeatAnimation(sprite,skill_id,totalFrames)
	local frames = display.newFrames("channel_"..skill_id.."_%02d.png", 1, totalFrames)
    local animation = display.newAnimation(frames, self.single_time / totalFrames)

    transition.execute(sprite,CCAnimate:create(animation),{
    	onComplete = function()
    		self.cycle_time =  self.cycle_time - 1
    		if self.cycle_time==0 then
    			self:clean()
	    	else
	    		self:run40002RepeatAnimation(sprite,skill_id,totalFrames)
	    		self:resetCheckHitSchedule(0.5,self.single_time)
	    	end
    	end
    })
end

function HeroSkill:makeSkillAndRun(skill_id)
	if skill_id==40001 then
            local frames = display.newFrames("cast_".."41002".."_%02d.png", 1, 30)
    		local animation = display.newAnimation(frames, 1 / 20)
    		self.skillSprite = display.newSprite(frames[1],self.releasePoint.x,self.releasePoint.y+130):addTo(self)
    		self.skillSprite:setScale(1.3)
    		self:resetCheckHitSchedule(0.1,1)
			transition.execute(self.skillSprite,CCAnimate:create(animation),{
		    	onComplete = function()
		    		CommUtil.vibrator(display.getRunningScene())
		    		self:clean()
		    	end
		    })

    elseif skill_id == 40002 then
            self.cycle_time = self.skillProperties.effect.cycle_time
			self.single_time = self.skillProperties.effect.single_time
			local frames = display.newFrames("channel_"..skill_id.."_%02d.png", 1, 21)
    		local animation = display.newAnimation(frames, self.single_time / 21)
    		self.skillSprite = display.newSprite(frames[1],self.releasePoint.x+20,self.releasePoint.y+110):addTo(self)
    		self.skillSprite:setScale(2)
			self:resetCheckHitSchedule(0.5,1)
            self:run40002RepeatAnimation(self.skillSprite,skill_id,21)
	elseif skill_id == 40003 then

		 local walkaction = display.newFrames("cast_41015_%02d.png", 1, 10)
    local walk_action = display.newAnimation(walkaction, 1/10)
   
     local walk_action1 = CCRepeat:create(CCAnimate:create(walk_action), 1)
    local cast = display.newSprite("#cast_41015_01.png"):pos(self.owner:getPositionX()+100,self.owner:getPositionY()+30):addTo(self)
    transition.execute( cast, walk_action1, {
      onComplete = function()
        CommUtil.playSound("res/music/soundeffect/music_41015.mp3",false)
        --self:skillRelease()

   local walkaction1 = display.newFrames("ending_41015_%02d.png", 1, 15)
    local walk_action1 = display.newAnimation(walkaction1, 1/15)

    local walk_action10 = CCRepeat:create(CCAnimate:create(walk_action1), 1)
    local cast = display.newSprite("#ending_41015_01.png"):pos(self.releasePoint.x-120,self.releasePoint.y+80):addTo(self)
     transition.execute( cast, walk_action10, {
      onComplete = function()
        cast:removeFromParentAndCleanup()
      end
    })




        local frames = display.newFrames("channel_"..skill_id.."_%02d.png", 1,5)
        local animation = display.newAnimation(frames, 0.2 / 5)
        self.skillSprite = display.newSprite(frames[1],self.releasePoint.x,self.releasePoint.y+60):addTo(self)
        self.cycle_time = self.skillProperties.effect.cycle_time
        self.skillSprite:setScale(2)    
        self:resetCheckHitSchedule(0.2,self.skillProperties.effect.single_time)
        transition.execute(self.skillSprite,CCAnimate:create(animation),{
	   		onComplete = function()
        		self:run40003RepeatAnimation(self.skillSprite,skill_id)
	    	end,
		})

      end
    })


		
	elseif skill_id == 40004 then
			local frames = display.newFrames("cast_"..skill_id.."_%02d.png", 1, 23)
    		local animation = display.newAnimation(frames, 1.5 / 23)
    		self.skillSprite = display.newSprite(frames[1],self.releasePoint.x,self.releasePoint.y+30):addTo(self)
    		local action = CCSpawn:createWithTwoActions(CCMoveTo:create(1,self.releasePoint),CCAnimate:create(animation))
    		self.skillSprite:setScale(2)
    		self:resetCheckHitSchedule(0.1,1.5)
			transition.execute(self.skillSprite,CCAnimate:create(animation),{
		    	onComplete = function()
		    		self:clean()
		    	end
		    })
	elseif skill_id == 40005 then

			local frames = display.newFrames("bullet_"..skill_id.."_%02d.png", 1, 20)
    		local animation = display.newAnimation(frames, 2 / 20)
    		self.skillSprite = display.newSprite(frames[1],self.owner:getPositionX(),self.owner:getPositionY()+30):addTo(self)
    		self.skillSprite:setAnchorPoint(ccp(0.2,0.5))
    		self.skillSprite:setScale(3)
    		local shootVector = ccpSub(ccp(self.releasePoint.x,self.releasePoint.y),ccp(self.owner:getPositionX(),self.owner:getPositionY()))
    		local shootAngle = -1*ccpToAngle(shootVector)*57.29577951
    		self.skillSprite:setRotation(shootAngle)
    		
    		self:resetCheckHitSchedule(0.2,1)
			transition.execute(self.skillSprite,CCAnimate:create(animation),{
		    	onComplete = function()
		    		self:clean()
		    	end
		    })



			--[[
	    	local frames = display.newFrames("cast_"..skill_id.."_%02d.png", 1, 16)
            local animation = display.newAnimation(frames, 0.5 / 16)
            self.skillSprite = display.newSprite(frames[1],self.releasePoint.x,self.releasePoint.y+150):addTo(self)
		    local moveAction = CCMoveBy:create(1,ccp(0,-100))
		    --local action = CCSpawn:createWithTwoActions(moveAction,CCAnimate:create(animation))
		    transition.execute(self.skillSprite,CCAnimate:create(animation),{
	    		onComplete = function()
	    			self.skillSprite:removeSelf()
	    			CommUtil.vibrator(display.getRunningScene())
	    			local frames = display.newFrames("ending_"..skill_id.."_%02d.png", 1, 11)
		            local animation = display.newAnimation(frames, 1 / 11)
		            self.skillSprite = display.newSprite(frames[1],self.releasePoint.x,self.releasePoint.y+70):addTo(self)
		            self:resetCheckHitSchedule(0.2,1)
		            transition.execute(self.skillSprite,CCAnimate:create(animation),{
			    		onComplete = function()
			    			self:clean()
			    		end
			    	})
	    		end
	    	})]]
	elseif skill_id == 40006 then
		  --[[self.cycle_time = self.skillProperties.effect.cycle_time
		 self.skillSprite = display.newSprite("#channel_"..skill_id.."_01.png",self.targetPoint.x,self.targetPoint.y+30):addTo(self)
            self.cycle_time = self.skillProperties.effect.cycle_time
            self:resetCheckHitSchedule(0.2,self.skillProperties.effect.single_time)
            transition.execute(self.skillSprite,CCAnimate:create(AnimationFactory.getSkillChannelAnimation(skill_id,25,self.skillProperties.effect.single_time)),{
	    		onComplete = function()
	    			self.cycle_time =  self.cycle_time - 1
            		self:runGoBackRepeatAnimation(self.skillSprite,skill_id,25,owner,target)
	    		end,
			})]]
CommUtil.playSound("music/music_40006.mp3")
		self.cycle_time = self.skillProperties.effect.cycle_time
			local frames = display.newFrames("channel_"..skill_id.."_%02d.png", 1, 16)
    		local animation = display.newAnimation(frames, 3 / 16)
    		self.skillSprite = display.newSprite(frames[1],self.releasePoint.x,self.releasePoint.y+140):addTo(self)
    		self.skillSprite:setScale(1)
    		self:resetCheckHitSchedule(0.2,1.5)
			transition.execute(self.skillSprite,CCAnimate:create(animation),{
		    	onComplete = function()
		    		--CommUtil.vibrator(display.getRunningScene())
		    		--self:clean()
		    		--self.cycle_time =  self.cycle_time - 1
		    		--self:runGoBackRepeatAnimation(self.skillSprite,skill_id,25,owner,target)
		    		CommUtil.playSound("music/music_40006.mp3")
		    		self.skillSprite:removeFromParentAndCleanup(true)
		    		local frames = display.newFrames("channel_"..skill_id.."_%02d.png", 1, 16)
    		local animation = display.newAnimation(frames, 3 / 16)
    		self.skillSprite = display.newSprite(frames[1],self.releasePoint.x,self.releasePoint.y+140):addTo(self)
    		self.skillSprite:setScale(1)
    		self:resetCheckHitSchedule(0.2,1.5)
			transition.execute(self.skillSprite,CCAnimate:create(animation),{
		    	onComplete = function()
		    		--CommUtil.vibrator(display.getRunningScene())
		    		--self.cycle_time =  self.cycle_time - 1
		    		--self:runGoBackRepeatAnimation(self.skillSprite,skill_id,25,owner,target)
		    		self:clean()
		    	end
		    })
		    	end
		    })
		    
    end
end

function HeroSkill:runGoBackRepeatAnimation(sprite,skill_id,totalFrames,owner,target)
    transition.execute(sprite,CCAnimate:create(AnimationFactory.getSkillChannelAnimation(skill_id,totalFrames,self.skillProperties.effect.single_time)),{
    	onComplete = function()
    		self.cycle_time =  self.cycle_time - 1
    		if self.cycle_time==0 then
    			self:clean()
	    	else
	    		self:resetCheckHitSchedule(0.2,self.skillProperties.effect.single_time)
	    		self:runGoBackRepeatAnimation(sprite,skill_id,totalFrames,owner,target)
	    	end
    	end
    })
end

function HeroSkill:runRepeatAnimation(sprite,skill_id,totalFrames,frequence)
	local frames = display.newFrames("channel_"..skill_id.."_%02d.png", 1, totalFrames)
    local animation = display.newAnimation(frames, self.skillProperties.effect.single_time / totalFrames)
    self:resetCheckHitSchedule(frequence,self.skillProperties.effect.single_time)
    transition.execute(sprite,CCAnimate:create(animation),{
    	onComplete = function()
    		self.cycle_time =  self.cycle_time - 1
    		if self.cycle_time==0 then
    			self:clean()
	    	else
	    		self:resetCheckHitSchedule(frequence,self.skillProperties.effect.single_time)
	    		self:runRepeatAnimation(sprite,skill_id,totalFrames,frequence)
	    	end
    	end
    })
end

function HeroSkill:releaseBeautySkill()
    local mask = CommUtil.addMaskLayer(self)
    local blue = display.newSprite("ani/pve/skill/beauty_blue.png",display.cx,display.cy):addTo(self)
    local beauty = display.newSprite("ani/pve/skill/beauty_1_skill.png",-display.cx,display.cy):addTo(self)

    self:doFullScreenEffect()--全屏幕技能buff处理
    
    transition.execute(beauty, CCMoveTo:create(0.1, CCPoint(display.cx+50, display.cy)),{
    easing = "exponentialIn",
    onComplete = function()
    	local skillTip = display.newSprite("ui/skill_word/skill_wodr_42011.png",display.cx,70):addTo(self)
        beauty:performWithDelay(function()
            transition.execute(beauty, CCMoveTo:create(0.2, CCPoint(display.right+300, display.cy)),{
            easing = "exponentialIn",
            onComplete = function()
                mask:removeSelf()
                blue:removeSelf()
                beauty:removeSelf()
                skillTip:removeSelf()
                self:clean()
            end,
            })
        end,0.3)
    end,
    })
end

function HeroSkill:doFullScreenEffect()
	if self.skillProperties==nil then return end
	local buffs = self.skillProperties.buffs
	if buffs~=nil then
		for i=1,#buffs do
			local buff = buffs[i]
			local buffType = buff.buff_type
			
			if buffType==6 then
				for i=0,GameInstance:sharedInstance().towers:count()-1 do
			        local tower = GameInstance:sharedInstance().towers:objectAtIndex(i)
			        if not tower:ifAddBuff(self.skill_id) then
			        	tower:addBuff(self.skill_id,Buff.new(buff))
			        end
			    end
			end
		end
	end
end

function HeroSkill:towerCanAttackMonster(monster)
    local canAttack = true
    if monster.air then
        if self.owner:canAttackAir() then
            canAttack = true
        else
            canAttack = false
            return canAttack
        end
    end
    if monster.underground then
        if self.owner:canAttackUnderground() then
            canAttack = true
        else
            canAttack = false
            return canAttack
        end
    end
    if monster.stealth then
        if self.owner:canAttackStealth() then
            canAttack = true
        else
            canAttack = false
            return canAttack
        end
    end
    return canAttack
end

return HeroSkill