local AnimationFactory = require("app.factory.AnimationFactory")
local MathUtil = require("app.common.MathUtil")
local Buff = require("app.actor.Buff")

local MageBullet = class("MageBullet", function(sourcePoint,target,if_crit)
    local sprite = display.newSprite("#mage_attack_1.png",sourcePoint.x,sourcePoint.y)
    return sprite
end)

function MageBullet:ctor(sourcePoint,target,if_crit)
    
    self.target = target

    self.if_crit = if_crit
    
    local centerX,centerY,textTureSize = target:getCenterPointInWorld(target:getPredictPosition().x,target:getPredictPosition().y)
    self.endX = centerX
    self.endY = centerY
    self.targetTextureSize = textTureSize

    CommUtil.playSound("music/attack_mage.mp3")

    local destPos = ccp(self.endX,self.endY)


    local diffPos = ccpSub(destPos,sourcePoint)
    local dist = ccpDistance(destPos,sourcePoint)
    local bulletVelocity = 300

    self.needRemove = false

    local shootVector = ccpSub(destPos,sourcePoint)
    local shootAngle = -1*ccpToAngle(shootVector)*57.29577951
    self:setRotation(shootAngle)

    local frames = display.newFrames("mage_attack_%d.png", 1, 8)
    local animation = display.newAnimation(frames, 0.2 / 8)
    self:playAnimationForever(animation)
    

    local flyTime = dist/bulletVelocity

    local actionMove = CCMoveBy:create(flyTime,diffPos)
    transition.execute(self,actionMove,{
        easing = "elastic",
        onComplete = function()
            self:decreaseTargetHp()
            self:removeFromParentAndCleanup(true)
        end,
    })
    --self:schedule(handler(self, self.updateZorder),1/30)
end

function MageBullet:updateZorder()

    local monsters = display.getRunningScene().baseScene.quadTree.grids[self.target.gridIndex]
    local posX = self.endX
    local posY = self.endY
    local damage = self.damage
    --print("怪物的个数是多少个",#monsters)
    if monsters~=nil then
        for i=1,#monsters do
            local monster = monsters[i]
            local distance = ccpDistance(ccp(monster:getPositionX(),monster:getPositionY()),ccp(posX,posY))
            if monster~=nil and (not tolua.isnull(monster)) and distance<40 then
                local buffProperties = {
                    buff_time = 2,
                    buff_type = 1,
                    jump_time = 1,
                    buff_value = -0.35,
                    buff_effect = "flame",
                }
                local buff = Buff.new(buffProperties)
                monster:addBuff("10000",buff,damage*0.03)
            end
        end
    end
    
    if tolua.isnull(self) then
        return
    end
    if self.needRemove then
        return
    end
    -------------------------------------------------------------------------------------------------碰撞检测
    local BulletRigion = require("app.region.BulletRigion")
        local curRigion = BulletRigion["mage_attack_1_"]
        for i=1,#curRigion do
            local circle = curRigion[i]
            local worldPoint = self:convertToWorldSpace(ccp(circle[1],circle[2]))
            local distance = ccpDistance(ccp(self.endX,self.endY),worldPoint)
            local edge = 10+circle[3]
            if distance<(edge) then
                --print("你被击中了")
                self:decreaseTargetHp()
                self:removeFromParentAndCleanup(true)
            return
        end
    end
    ------------------------------------------------------------------------------------------------------
    local zorder = GameInstance:sharedInstance():zorderForPosition(ccp(self:getPositionX(),self:getPositionY()))
    self:getParent():reorderChild(self,zorder)
end

function MageBullet:decreaseTargetHp()
    self.needRemove = true
    --[[
    local frames,animation = AnimationFactory.getMageExplodeAniamtion()
    local explodeSprite = display.newSprite(frames[1],self:getPositionX()+15,self:getPositionY()+20):addTo(display.getRunningScene(),self.target:getZOrder())
    explodeSprite:setScale(0.8)
    transition.execute(explodeSprite,CCAnimate:create(animation),{
        onComplete = function()
            explodeSprite:removeSelf()
        end,
    })]]
    if self.ifEnHance then
        self:getParent():addChild(BulletBomb.new(self.target,2,self.damage),5999)
    end
    if tolua.isnull(self.target) then
        transition.fadeTo(self, {opacity = 0, time = 0.3,onComplete = function()
            self:removeFromParentAndCleanup(true)
        end
        })
    else
        self.target:decreaseHp("mage",self.damage,self.if_crit)
        --CommUtil.playSound("music/lose_beauty.mp3")
    end
end

return MageBullet
