local MathUtil = require("app.common.MathUtil")

local WizardBullet = class("WizardBullet", function(sourcePoint,target,if_crit)
    --local sprite = display.newSprite("warrior_attack_1.png",sourcePoint.x,sourcePoint.y)
    local sprite = display.newSprite("#wizard_attack_1.png",sourcePoint.x,sourcePoint.y)
    sprite:setScale(0.5)
    return sprite
end)

function WizardBullet:ctor(sourcePoint,target,if_crit)
    
    self.target = target

    self.if_crit = if_crit

    local centerX,centerY,textTureSize = target:getCenterPointInWorld(target:getPredictPosition().x,target:getPredictPosition().y)
    self.endX = centerX
    self.endY = centerY
    self.targetTextureSize = textTureSize

    CommUtil.playSound("music/attack_wizard.mp3")

    local destPos = ccp(self.endX,self.endY)

    local diffPos = ccpSub(destPos,sourcePoint)
    local dist = ccpDistance(destPos,sourcePoint)
    local bulletVelocity = 350

    self.needRemove = false

    local shootVector = ccpSub(destPos,sourcePoint)
    local shootAngle = -1*ccpToAngle(shootVector)*57.29577951
    self:setRotation(shootAngle)

    
    
    local flyTime = dist/bulletVelocity


    local actionMove = CCMoveBy:create(flyTime,diffPos)
    transition.execute(self,actionMove,{
        easing = "elastic",
        onComplete = function()
            self:decreaseTargetHp()
            self:removeFromParentAndCleanup(true)
        end,
    })

    transition.fadeIn(self,{opacity = 255,time = 0.1})
    
    --self:schedule(handler(self, self.updateZorder),1/30)

end

function WizardBullet:updateZorder()
    if tolua.isnull(self) then
        return
    end
    if self.needRemove then
        return
    end
    -------------------------------------------------------------------------------------------------碰撞检测
    local BulletRigion = require("app.region.BulletRigion")
    local curRigion = BulletRigion["mage_attack_1_"]
    for i=1,#curRigion do
        local circle = curRigion[i]
        local worldPoint = self:convertToWorldSpace(ccp(circle[1],circle[2]))
        local distance = ccpDistance(ccp(self.endX,self.endY),worldPoint)
        local edge = 10+circle[3]
        if distance<(edge) then
            --print("你被击中了")
            self:decreaseTargetHp()
            self:removeFromParentAndCleanup(true)
            return
        end
    end
    ------------------------------------------------------------------------------------------------------
    local zorder = GameInstance:sharedInstance():zorderForPosition(ccp(self:getPositionX(),self:getPositionY()))
    self:getParent():reorderChild(self,zorder)
end

function WizardBullet:decreaseTargetHp()
    self.needRemove = true
    if self.ifEnHance then
        self:getParent():addChild(BulletBomb.new(self.target,3,self.damage),5999)
    end
    if tolua.isnull(self.target) then
        transition.fadeTo(self, {opacity = 0, time = 0.3,onComplete = function()
            self:removeFromParentAndCleanup(true)
        end
        })
    else
       self.target:decreaseHp("warrior",self.damage,self.if_crit)
       --CommUtil.playSound("music/lose_beauty.mp3")
    end
end

return WizardBullet
