local CountDown = class("CountDown")
local scheduler = require("framework.scheduler")

local STATUS = {
	STOP = 0,
	RUNNING = 1,
	PAUSE = 2,
}

function CountDown:ctor()
--[[ when it create, the uptime will be changed
]]
	self.cds = {}		--reference of all cds
	self.uptime = 0 	--total online time of player
	self.handle = scheduler.scheduleGlobal(handler(self,self.onInterval), 1)

	self.now = os.time()
end

function CountDown:debug(v)
--[[ just for debug
]]
	print("--------------------------")
	print("cur_time",v.cur_time)
	print("remain_sec",v.remain_sec)
	print("now",os.time())
end

function CountDown:onInterval()
--[[ every second to do
]]
	-- if now time isnot change
	if self.now < os.time() then
		self.now = os.time()

		self.uptime = self.uptime + 1
		for i,v in ipairs(self.cds) do
			-- if i == 1 then self:debug(v) end
			if next(v) ~= nil then
				if self.now - v.cur_time ~= 1 then
					local remain = v.cur_time + 1 + v.remain_sec - self.now
					if remain <= v.remain_sec then
						v.cur_time = self.now - 1
						v.remain_sec = (remain < 0) and 0 or remain
					elseif (v.cur_time - self.now) > 0 then
						v.cur_time = self.now - 1
					end
				end
				v.cur_time = v.cur_time + 1

				if v.status == STATUS.RUNNING and v.remain_sec >= 0 then
					if v.remain_sec == 0 then
						self:stop(i)
						v.done_to_do(i)
					end
					if v.remain_sec > 0 then
						v.remain_sec = v.remain_sec - 1
					end
				end
			end
		end
	end
end

--[[
cd = {
	cycle_sec  = 60,				-- cycle seconds of this cd (use for restart function)
	remain_sec = 10,				-- remain seconds of this cd
	done_to_do = callback, 			-- when count down handler has done, call the callback function
	status     = STATUS.RUNNING, 	-- the running status of this cd
	cur_time   = os.time(),			-- cd start time
}
]]
function CountDown:make( args )
--[[ factory function, to make new cd
args = {
	cycle_sec = 60,
	remain_sec = 10,
	done_to_do = callback,
}
]]
	assert(type(args) == "table", "param #1 should be a table")
	assert(type(args.cycle_sec) == "number" and type(args.remain_sec) == "number" ,"args type error")
	assert(type(args.done_to_do) == "function" ,"args type error")
	assert(args.cycle_sec >= 0 and args.remain_sec >= 0,"remain_sec and cycle_sec must great then or equal 0")

	local cd = clone(args)
	cd.status = STATUS.STOP  -- initial status
	cd.remain_sec = cd.remain_sec - 1
	return cd
end

function CountDown:insert( cd )
--[[ insert cd into CountDown handler
return the index of this cd
]]
	local size = #self.cds
	local index = size + 1
	self.cds[index] = cd
	return index
end

function CountDown:delete(cd_id)
	if cd_id <= #self.cds then
		self.cds[cd_id] = {}
	end
end

function CountDown:free()
--[[ free handle
]]
	scheduler.unscheduleGlobal(self.handle)
end

function CountDown:startAll()
--[[ to start all of these cds
]]
	for i,_ in ipairs(self.cds) do
		self:start(i)
	end
end

function CountDown:continueAll()
--[[ to continue all of these cds
]]
	for i,_ in ipairs(self.cds) do
		self:continue(i)
	end
end

function CountDown:pauseAll()
--[[ to pause all of these cds
]]
	for i,_ in ipairs(self.cds) do
		self:pause(i)
	end
end

function CountDown:stopAll()
--[[ to stop all of these cds
]]
	for i,_ in ipairs(self.cds) do
		self:stop(i)
	end
end

function CountDown:start( index )
--[[ to start which cd
]]
	assert(index <= #self.cds,"Array Index Overflow")
	if self:isEmpty(index) and not self:isRunning(index) then
		self.cds[index].cur_time   = os.time()
		self.cds[index].status     = STATUS.RUNNING
	end
end

function CountDown:continue( index )
--[[ to continue which cd
]]
	assert(index <= #self.cds, "Array Index Overflow")
	if self:isEmpty(index) then
		self.cds[index].status = STATUS.RUNNING
	end
end

function CountDown:pause( index )
--[[ to pause which cd
]]
	assert(index <= #self.cds, "Array Index Overflow")
	if self:isEmpty(index) then
		self.cds[index].status = STATUS.PAUSE
	end
end

function CountDown:stop( index )
--[[ to stop which cd
]]
	assert(index <= #self.cds, "Array Index Overflow")
	if self:isEmpty(index) then
		self.cds[index].status = STATUS.STOP
		self.cds[index].remain_sec = 0
	end
end

function CountDown:restart( index )
--[[ to restart which cd
]]
	assert(index <= #self.cds, "Array Index Overflow")
	if self:isEmpty(index) then
		self.cds[index].remain_sec = self.cds[index].cycle_sec
		self:start(index)
	end
end


function CountDown:isRunning( index )
--[[ return true if index is running
]]
	assert(index <= #self.cds, "Array Index Overflow")
	return self.cds[index].status == STATUS.RUNNING
end

function CountDown:formatSec( sec )
--[[ format second number as string
]]
	assert(type(sec) == "number", "param #1 should be a number")
	local hour   = math.floor(sec/3600)
	local minute = math.floor((sec-hour*3600)/60)
	local second = sec - hour*3600 - minute*60
	--TODO: when hour >= 100 or < 0, this format string is buggy
	if hour < 0 then
		return "00:00:00"
	end
	return string.format("%02d:%02d:%02d", hour, minute, second)
end

function CountDown:formatSec_( sec )
--[[ format second number as string
]]
	assert(type(sec) == "number", "param #1 should be a number")
	local hour   = math.floor(sec/3600)
	local minute = math.floor((sec-hour*3600)/60)
	local second = sec - hour*3600 - minute*60
	--TODO: when hour >= 100 or < 0, this format string is buggy
	if hour >= 24 then
		return math.floor(hour / 24) .. "天"
	elseif hour < 0 then
		return "00:00:00"
	end

	return string.format("%02d:%02d:%02d", hour, minute, second)
end

function CountDown:formatStr( str )
--[[ format second string as number
]]
	assert(type(str) == "string", "param #1 should be a string")
	local hour, minute, second = string.match(str, "(%d+):(%d+):(%d+)")
	--TODO: when each value < 0, it is buggy
	return hour*3600 + minute*60 + second
end

function CountDown:getStr( index )
--[[ return countdown string. eg: 10:55:21
]]
	assert(index <= #self.cds, "Array Index Overflow")
	return self:formatSec(self.cds[index].remain_sec)
end

--当剩余时间>=1天时，以‘天数’返回；其他以‘小时：分钟：秒’
function CountDown:getStr_( index )
--[[ return countdown string. eg: 10:55:21
]]
	assert(index <= #self.cds, "Array Index Overflow")
	return self:formatSec_(self.cds[index].remain_sec)
end

function CountDown:setRemain( index, remain_sec )
--[[ set remain seconds
]]
	assert(index <= #self.cds, "Array Index Overflow")
	if self:isEmpty(index) then
		self.cds[index].remain_sec = remain_sec
	end
end


function CountDown:getRemain( index )
--[[ set remain seconds
]]
	assert(index <= #self.cds, "Array Index Overflow")

	return self.cds[index].remain_sec
end

function CountDown:size()
--[[ return size of countdown cds
]]
	return #self.cds
end

function CountDown:isEmpty(index)
	return next(self.cds[index]) ~= nil
end

return CountDown