--[[ 
@ Todo  : 本地加密数据存储
@ Author: Nemo
@ Date  : 2014-5-23
]]
local LocalEncryptDataUtil = {}

function LocalEncryptDataUtil.get_str( signature )
--[[ 通过签名读取加密文件  读取存储的string
]]
	assert(type(signature) == "string", "invalid parameter [signature]")

	local file_name = crypto.md5(signature..gPlayer.data.account_id.."|"..gPlayer.data.player_id) -- 文件名 将签名md5

	local base64_str = FileUtil.readFile(gStorePath.."/"..file_name) -- 读文件
	if not base64_str then return end -- 文件不存在

	local bz2_str = crypto.decodeBase64(base64_str)

	local decompressor = bz2.initDecompress()
	local un_bz2_str, err = decompressor:update(bz2_str) -- 解压缩得 前32位校验码后为加密数据
	decompressor:close()
	if not un_bz2_str then return end -- 无法解压

	-- 校验内容是否被修改
	local check_str = string.sub(un_bz2_str, 0, 32)
	local xxtea_str = string.sub(un_bz2_str, 33, string.len(un_bz2_str))
	if check_str ~= crypto.md5(xxtea_str) then return end

	local data_str = crypto.decryptXXTEA(xxtea_str, "m0bI2us") --解密
	return data_str
end

function LocalEncryptDataUtil.set_str( signature, data_str )
--[[ 字符串加密
]]
	assert(type(signature) == "string", "invalid parameter [signature]")
	assert(type(data_str) == "string", "invalid parameter [data_str]")

	local file_name = crypto.md5(signature..gPlayer.data.account_id.."|"..gPlayer.data.player_id) -- 文件名 将签名和当前account_id和player_id串起来 再md5

	local xxtea_str = crypto.encryptXXTEA(data_str, "m0bI2us")
	local check_str = crypto.md5(xxtea_str) -- 加密数据校验MD5值

	local compressor = bz2.initCompress()
	local bz2_str = compressor:update(check_str..xxtea_str) -- 前32位校验码后为加密数据 再压缩
	bz2_str = compressor:update(nil)
	compressor:close()

	local base64_str = crypto.encodeBase64(bz2_str) -- Base64转码
	FileUtil.writeFile(gStorePath.."/"..file_name, base64_str)
end

function LocalEncryptDataUtil.get( signature )
--[[ 读取存储的table
]]
	assert(type(signature) == "string", "invalid parameter [signature]")

	local data_str = LocalEncryptDataUtil.get_str(signature) --数据写入
	if not data_str then return end
	local data = json.decode(data_str)
	return data
end

function LocalEncryptDataUtil.set( signature, data_table )
--[[ 通过签名写入加密文件
]]
	assert(type(signature) == "string", "invalid parameter [signature]")
	assert(type(data_table) == "table", "invalid parameter [data_table]")

	local data_str = json.encode(data_table) -- 数据table转为string
	LocalEncryptDataUtil.set_str(signature, data_str) --数据写入
end

function LocalEncryptDataUtil.getRechargeFileName(signature)
    local file_name = crypto.md5(signature .. "|" .. gPlayer.data.realm_id .. "|" .. gPlayer.data.account_id)
    return string.upper(file_name)
end

return LocalEncryptDataUtil