
local MathUtil = {}


function  MathUtil.getBaseAngle1(p1,p2)
     local x = 0
     local ix,iy = 0,0
     ix = p1.x - p2.x
     iy = p1.y - p2.y
     if ix >= 0 and iy < 0 then
         if ix == 0 then
            x= 270
         else
            iy = iy*-1
            x = 180 + math.atan(iy / ix) * 57.29577951
        end
     elseif ix > 0 and iy >= 0 then
         if iy == 0 then
             x = 180
         else
             x = 90+math.atan(ix / iy) * 57.29577951
         end
     elseif ix <= 0 and iy > 0 then
        if ix == 0 then
             x = 90
        else
             ix = ix*-1
             x = math.atan(iy / ix) * 57.29577951
         end
    else
         if iy == 0 then
             x = 0
         else
             ix = ix*-1
             iy = iy*-1
             x = 270 + math.atan(ix / iy) * 57.29577951
         end
     end
     return x
end


function MathUtil.getBaseAngle(point)
    local angle = math.atan(point.y/point.x)* 57.29577951
    return angle
end

function MathUtil.getAngle2(point1,point2)
    local angle = math.atan((point2.y-point1.y)/(point1.x-point1.x))* 57.29577951
    return angle
end

function MathUtil.getAngle(pSrc,p1,p2)
    local angle = 0.0 
      
    --向量Vector a的(x, y)坐标  
    local va_x = p1.x - pSrc.x 
    local va_y = p1.y - pSrc.y  
      
    --向量b的(x, y)坐标  
    local vb_x = p2.x - pSrc.x
    local vb_y = p2.y - pSrc.y  
      
    local productValue = (va_x * vb_x) + (va_y * vb_y)--向量的乘积  
    local va_val = math.sqrt(va_x * va_x + va_y * va_y)--向量a的模  
    local vb_val = math.sqrt(vb_x * vb_x + vb_y * vb_y)--向量b的模  
    local cosValue = productValue / (va_val * vb_val)--余弦公式  
      
    --acos的输入参数范围必须在[-1, 1]之间，否则会"domain error"  
    --对输入参数作校验和处理  
    if cosValue < -1 and cosValue > -2  then
        cosValue = -1  
    elseif cosValue > 1 and cosValue < 2 then
        cosValue = 1
    end
    --acos返回的是弧度值，转换为角度值  
    angle = math.acos(cosValue) * 57.29577951
    
    return angle
end

--PI * 180
function MathUtil.radiansToDegrees(angle) 
     return angle * 57.29577951
end

function MathUtil.degreesToRadians(degree) 
     return degree / 57.29577951   
end

-- 求两点间距离
function MathUtil.distance(ax, ay, bx, by)
    local dx, dy = bx - ax, by - ay
    return math.sqrt(dx * dx + dy * dy)
end

--根据弓箭等的当前位置和攻击目标的位置计算弓箭的攻击方向
function MathUtil.getRotateDegrees(v1,v2,baseAngle)
    local diff = ccpSub(v1,v2)
    local angle = ccpToAngle(diff)
    
    --v1 is on v2's rightTop
    if diff.x>0 and diff.y>0 then
        local degrees = MathUtil.radiansToDegrees(-1* angle)-baseAngle
        --print("攻击角度"..degrees)
        return degrees
    --v1 is on v2's rightBottom
    elseif diff.x>0 and diff.y<0 then
        local degrees = MathUtil.radiansToDegrees(-1* angle)-baseAngle
        --print("攻击角度"..degrees)
        return degrees
    --v1 is on v2's leftTop
    elseif diff.x<0 and diff.y>0 then
        local degrees = MathUtil.radiansToDegrees(-1* angle)-baseAngle
        --print("攻击角度"..degrees)
        return degrees
    --v1 is on v2's leftBottom
    elseif diff.x<0 and diff.y<0 then
        local degrees = MathUtil.radiansToDegrees(-1* angle)-baseAngle
        --print("攻击角度"..degrees)
        return degrees
    else
    end
end

--计算数据集中数据值最长的长度
function MathUtil.calMaxLengthFromData(data)
    local maxLength = 0
    for k, v in ipairs(data) do
        local length = string.len(v)
        if length > maxLength then
            maxLength = length
        end
    end
    return maxLength
end

--[[
功能：对数值进行处理，数值分为整数、小数、百分数(仍为小数，待后续处理为百分数)
输入：
    type = 1,   --整数
    type = 2,   --小数，保留两位小数
    type = 3,   --百分数，保留三位小数(百分数处理后，仅存在一位小数)
    type = 4,   --百分数，保留两位小数
    ]]
function MathUtil.processNumerical(numerical, type)
    local value
    if type == 1 then
        value = numerical - numerical % 1
    elseif type == 2 then
        value = numerical - numerical % 0.01
    elseif type == 3 then
        value = numerical * 100 - numerical * 100 % 0.1
    elseif type == 4 then
        value = numerical * 100 - numerical * 100 % 1
    end
    return value
end


function MathUtil.ccpDistance(p1,p2)
    local x = p2.x-p1.x
    local y = p2.y-p1.y
    return x*x+y*y
end

return MathUtil
