local ProtectedNumber = class("ProtectedNumber")

--[[
使用方法：
local ingot = ProtectedNumber.new(100)    -- 不传值默认初始化为0
ingot:get()                               -- 读取真实值
ingot:set((ingot:get() + 100) * 2 - 50)   -- 运算之后设置值  即ingot = (ingot + 100) * 2 - 50
]]

function ProtectedNumber:ctor( num )
	num = num or 0
	self:init()
	self.left = 0
	self.right = 0
	self.isFloat = false
	self:set(num)
end

function ProtectedNumber:setSecret( val, secret, decimal  )
--[[ 还原该Number值
]]
	local num = BitUtil:_xor(val, secret)
	self:init()
	self.left = 0
	self.right = 0
	self:set(num)
	self.decimal = decimal or 0
end

function ProtectedNumber:getSecret()
--[[ 获取加密值和密码
]]
	return self.left, self.leftSecret, self.decimal or 0
end

function ProtectedNumber:init()
--[[ 为每个数值创建两个不同的密码
]]
	local random1 = math.random(5)
	local random2 = math.random(6)
	local random3 = math.random(8)
	local random4 = math.random(9)

	self.leftSecret = BitUtil:_or(BitUtil:_lshift(random1, 16), random2)
	self.rightSecret = BitUtil:_or(BitUtil:_lshift(random3, 16),random4)
end

function ProtectedNumber:get()
--[[ 存储的加密值与密码异或为原值
]]
	local leftV = BitUtil:_xor(self.left, self.leftSecret)
	local rightV = BitUtil:_xor(self.right, self.rightSecret)
	assert(leftV==rightV, '内存数据已经被更改！')
	if leftV ~= rightV then
		self.left = 0
		self.right = 0
		return 0
	end

	return self.isFloat and leftV/1000 or leftV
end

function ProtectedNumber:set( num )
--[[ 将值的整数部分加密存储
]]
	if type(num) ~= 'number' then return end
	local integer = 0
	if num ~= math.floor(num) then --是浮点数
		-- integer = math.floor(num)
		-- self.decimal = num - integer   --保存小数 加密整数部分
		self.isFloat = true
		integer = num*1000
	else
		self.isFloat = false
		integer = num
		-- self.decimal = 0
	end

	self.left = BitUtil:_xor(integer, self.leftSecret)
	self.right = BitUtil:_xor(integer, self.rightSecret)
end

return ProtectedNumber