local TimeUtil = {}

function TimeUtil.now()
--[[ return current time string
use os.date("*t") to get current time table
format current time as string
]]
	return os.date("%Y-%m-%d %H:%M:%S")
end

function TimeUtil.toTime(_time)
--[[ convert time string to time table
use default format string: "%Y-%m-%d %H:%M:%S"
]]
	assert(type(_time) == "string", "assert param #1 type error")
	local t = {}
	string.gsub(_time, "[^ ]+", function(sub) table.insert(t, sub) end)
	assert(t ~= nil, "assert param #1 format error")
	local tab = {}
	local pattern
	for i, v in pairs(t) do
		if i == 1 then
			pattern = "[^-]+"
		else
			pattern = "[^:]+"
		end
		string.gsub(v, pattern, function(sub) table.insert(tab, sub) end)
	end
	assert((tab ~= nil) and (#tab == 6), "assert param #1 format error")
	local time_table = {}
	time_table.year  = tab[1]
	time_table.month = tab[2]
	time_table.day   = tab[3]
	time_table.hour  = tab[4]
	time_table.min   = tab[5]
	time_table.sec   = tab[6]
	return time_table
end

function TimeUtil.toStr(_time)
--[[ convert time table or timestamp to string
use default format string: "%Y-%m-%d %H:%M:%S"
]]
	local tp = type(_time)
	assert(tp == "number" or tp == "table", "assert param #1 type error")

	if tp == "table" then
		return os.date("%Y-%m-%d %H:%M:%S", TimeUtil.toTimestamp(_time))
	else
		return os.date("%Y-%m-%d %H:%M:%S", _time)
	end
end

function TimeUtil.toTimestamp(_time)
--[[ convert time table to timestamp
]]
	local tp = type(_time)
	assert(tp == "table" or tp == "string", "assert param #1 type error")

	if tp == "table" then
		return os.time(_time)
	else 
		return os.time(TimeUtil.toTime(_time))
	end
end

function TimeUtil.diffSec(_time1, _time2)
--[[ caculate diff time between two times
return diff seconds (_time2 - _time1)
]]
	local t1, t2 = _time1, _time2
	if type(_time1) ~= "number" then
		t1 = TimeUtil.toTimestamp(_time1)
	end
	if type(_time2) ~= "number" then
		t2 = TimeUtil.toTimestamp(_time2)
	end
	return t2 - t1
end

function TimeUtil.addDiff(_time, _diff)
--[[ add _diff seconds to _time
]]
	assert(type(_diff) == "number", "assert param #2 type error")
	local tp = type(_time)
	local t = _time
	if tp ~= "number" then
		t = TimeUtil.toTimestamp(_time)
	end
	t = t + _diff
	if tp == "string" then
		return TimeUtil.toStr(t)
	elseif tp == "table" then
		return TimeUtil.toTime(TimeUtil.toStr(t))
	else
		return t
	end
end

function TimeUtil.toDate(_time)
--[[ _time : "2013-12-31"
]]
	local year, month, day = string.match(_time, "(%d+)-(%d+)-(%d+)")
	if year == nil or month == nil or day == nil then
		return
	end
	local date_table = {}
	date_table.year  = year
	date_table.month = month
	date_table.day   = day
	return date_table
end

function TimeUtil.diffDay(_date1, _date2)
--[[ caculate diff days between two date
]]
	local date1, date2 = {}, {}
	if type(_date1) ~= "table" then
		date1 = TimeUtil.toDate(_date1)
	end	
	if type(_date2) ~= "table" then
		date2 = TimeUtil.toDate(_date2)
	end
	--先分别计算与1970-01-01的日期差，再相减
	local z = math.floor((14 - date1.month)/12)
	local y = math.floor(date1.year+4800-z)
	local m = math.floor(date1.month+12*z-3)
	local j1 = math.floor(153*m+2)
	j1 = math.floor(j1/5+date1.day+y*365+y/4-y/100+y/400-2472633)

	z = math.floor((14 - date2.month)/12)
	y = math.floor(date2.year+4800-z)
	m = math.floor(date2.month+12*z-3)
	local j2 = math.floor(153*m+2)
	j2 = math.floor(j2/5+date2.day+y*365+y/4-y/100+y/400-2472633)

	return j2 - j1
end

function TimeUtil.todayRemainSec(_time)
--[[ 计算当天剩余的秒数
]]
	local today = string.match(_time, "([^ ]+)") .. " 00:00:00"
	local pass_sec = TimeUtil.diffSec(today, _time)
	return 3600*24 - pass_sec
end

function TimeUtil.remainSec(_time, _start_time, refresh_days)
--[[ 计算距下次刷新的秒数
]]
	local run_days = TimeUtil.diffDay(_start_time, _time)
	local remain_day = refresh_days - run_days%refresh_days - 1
	local sec = TimeUtil.todayRemainSec(_time)
	return sec+remain_day*3600*24
end

return TimeUtil