
local DebugUtil = require("app.common.DebugUtil")
local CommUtil = require("app.common.CommUtil")
local HeroSkill = require("app.bullet.HeroSkill")
local AnimationFactory = require("app.factory.AnimationFactory")
local BulletFactory = require("app.factory.BulletFactory")
local scheduler = require(cc.PACKAGE_NAME .. ".scheduler")
local MathUtil = require("app.common.MathUtil")
local Rect = require("app.quadTree.Rect")
local GameLogic = require("app.controllers.GameLogic")


local BattleStoryGameLogic = class("BattleStoryGameLogic",GameLogic)


function BattleStoryGameLogic:checkWinOrLose()
    if totalHP<=0 then     --失败
        self.gameOver = true
        self.gameScene.parent:displayLosePanel()
        --------------------------------清除助战的塔位信息---------------------------------------------
        for i=1,#helpTowerPositions do
          gNTower:DeleteTowerPosition(lastHelpSceneId,helpTowerPositions[i])
        end
        helpTowerPositions = {}
--------------------------------清除助战的塔位信息---------------------------------------------
        return
    end
    if self.gameScene.battleStoryPlay and ( self.gameScene.storyOver==nil or (self.gameScene.storyOver==false) ) then
        return
    end
    if theLastMonster and GameInstance:sharedInstance().monsters:count()<=0 then--胜利
        self.gameOver = true
        self.gameScene.parent:displayWinPanel()
        --------------------------------清除助战的塔位信息---------------------------------------------
        for i=1,#helpTowerPositions do
          gNTower:DeleteTowerPosition(lastHelpSceneId,helpTowerPositions[i])
        end
        helpTowerPositions = {}
--------------------------------清除助战的塔位信息---------------------------------------------
        --[[if gBattle.data.home_pass_id==60002 and gPlayer.data.pve_pass_id==gBattle.data.home_pass_id then
            self.gameScene.skillPanel:removeSelf()
            self.gameScene.skillPanel = nil
            if self.gameScene.hand then self.gameScene.hand:removeSelf() self.gameScene.hand = nil end
            self.dialog = CommUtil.addDialog({
              node = self, 
              x = -20, 
              y =0, 
              direct = 0,      --方向 0 放在左边, 1放在右边
              head_type = 3,        --头像类似  0引导仙子 1主将 2仙将
              head_id = head_id,      --仙将ID
              name = "   妲己",                       --"显示名称"
              content ="#7".."纣王那死相又叫我回去陪酒，留下".."#9".."贴身技#1【画地为牢】#7给你防身！",--"内容"
              hideTip = false,
            })

            local touchLayer = display.newLayer():addTo(display.getRunningScene(),60000)
            touchLayer:setTouchEnabled(true)
            touchLayer:addTouchEventListener(function(event, x, y)
                if event == "began" then
                    self.dialog:removeSelf()
                    touchLayer:removeSelf()
                    require("app.common.CommUtil").rewardNewSkill(40020, function()
                        self.gameScene.parent:displayWinPanel()
                        --------------------------------清除助战的塔位信息---------------------------------------------
        for i=1,#helpTowerPositions do
          gNTower:DeleteTowerPosition(lastHelpSceneId,helpTowerPositions[i])
        end
        helpTowerPositions = {}
--------------------------------清除助战的塔位信息---------------------------------------------
                    end)
                    return true
                end
                return false
            end)

        else
            self.gameScene.parent:displayWinPanel()
            --------------------------------清除助战的塔位信息---------------------------------------------
        for i=1,#helpTowerPositions do
          gNTower:DeleteTowerPosition(lastHelpSceneId,helpTowerPositions[i])
        end
        helpTowerPositions = {}
--------------------------------清除助战的塔位信息---------------------------------------------
        end]]
        return
    end
end

function BattleStoryGameLogic:getTargetForTower(tower)
    --print("tower的rangeIndex是",#tower.gridIndexs)
    if not tolua.isnull(self.gameScene.focusMonster) and self.gameScene.focusMonster~=nil and not self.gameScene.focusMonster.dead then
        if self:towerCanAttackMonster(tower,self.gameScene.focusMonster) then--必须能够攻击此怪物才可以,地怪，飞行怪
            local curx,cury,size = self.gameScene.focusMonster:getCenterPointInWorld(self.gameScene.focusMonster:getPositionX(),self.gameScene.focusMonster:getPositionY())
            local distance = ccpDistance(ccp(curx,cury),ccp(tower:getPositionX(),tower:getPositionY()))
            local range = tower:getRange()
            if distance<range then
                return self.gameScene.focusMonster
            end
        end
    end
    local everTarget = tower.target
    if not tolua.isnull(everTarget) and everTarget~=nil and not everTarget.dead then
        local curx,cury,size = everTarget:getMonsterPositionAfterTowerAnimation(0.4)
        local distance = ccpDistance(ccp(curx,cury),ccp(tower:getPositionX(),tower:getPositionY()))
        local range = tower:getRange()
        if distance<range then
            return everTarget
        end
    end
    local targetMonster = nil
    local lastDistance = 0--此前的最短距离
    local num = 0
    for i=1,#tower.gridIndexs do
        local gridIndex = tower.gridIndexs[i]
        local monsters = self.gameScene.quadTree.grids[gridIndex]
        for k, v in ipairs(monsters) do
            num = num + 1 
            if not tolua.isnull(v) and v~=nil and not v.dead and self:towerCanAttackMonster(tower,v) then
                local curx,cury,size = v:getMonsterPositionAfterTowerAnimation(0.4)
                if targetMonster~=nil then
                    local distance = ccpDistance(ccp(curx,cury),ccp(tower:getPositionX(),tower:getPositionY()))
                    local range = tower:getRange()
                    if distance<range then
                        --local dist1 = v.curWaypoint
                        --local dist2 = targetMonster.curWaypoint
                        local waypoint = tolua.cast(v._waypoints:objectAtIndex(v._waypoints:count()-1),"WayPoint")
                        tolua.cast(waypoint.position,"CCPoint")
                        local dist = ccpDistance(ccp(curx,cury),waypoint.position)
                        if dist < lastDistance then
                            targetMonster = v
                            lastDistance = dist
                        end 
                    end
                else
                    local distance = ccpDistance(ccp(curx,cury),ccp(tower:getPositionX(),tower:getPositionY()))
                    local range = tower:getRange()
                    if distance<range then
                        targetMonster = v
                        local waypoint = tolua.cast(targetMonster._waypoints:objectAtIndex(targetMonster._waypoints:count()-1),"WayPoint")
                        tolua.cast(waypoint.position,"CCPoint")
                        lastDistance = ccpDistance(ccp(curx,cury),waypoint.position)
                    end
                end
            end
        end
    end
    --print("检测了多少个怪物",num,GameInstance:sharedInstance().monsters:count())
    return targetMonster
end


function BattleStoryGameLogic:fire(tower,monster,attack_speed,frontOrBack,leftOrRight)
    tower:coolDown(attack_speed)

    local hit_proc = Attributes.get_hit_proc(tower:get_hit_proc(),monster.hit_proc,tower:getBuffs(),monster:getBuffs())
    --print("hit_proc is"..hit_proc)
    local hit_proc_random = CPlusMathUtil:getRandomNum(100)/100
    --local hit_proc_random = math.random(100)/100
    --print("hit_proc_random is"..hit_proc_random)
    if hit_proc_random<=hit_proc then
        --print("命中率高")
        local skill_proc = Attributes.get_skill_proc(tower:get_skill_proc(),monster.skill_proc,tower:getBuffs(),monster:getBuffs())
        --local skill_proc = 0.3

        --local skill_proc_random = math.random(100)/100
        local skill_proc_random = CPlusMathUtil:getRandomNum(100)/100

        local ifSkill = nil
        if skill_proc_random < skill_proc then
            ifSkill = true
            --print("技能概率",skill_proc,skill_proc_random,tower:getSkillId(),"触发",tower:get_skill_proc(),monster:get_skill_proc())
        else
            ifSkill = false
            --print("技能概率",skill_proc,skill_proc_random,tower:getSkillId(),"未触发",tower:get_skill_proc(),monster:get_skill_proc())
        end


        --ifSkill = false
        
        --local frames,animation = AnimationFactory.getTowerAttackAnimation(tower:getTowerType(),tower:getTowerLevel(),frontOrBack,leftOrRight)
        
        if tower.forbidSkill then
            ifSkill = false
        end

        tower.hero:stopAllActions()
        if ifSkill then
            --print("技能攻击")
            tower:addSkillWord(function()
                local attackFrames,attackAnimation = AnimationFactory.getHeroAnimation(tower:getHeroId(),"attack",frontOrBack)
                if leftOrRight=="r" then
                    tower.hero:setFlipX(false)
                else
                    tower.hero:setFlipX(true)
                end
                transition.execute(tower.hero,CCAnimate:create(attackAnimation),{
                onComplete = function()
                    tower:doHeroStandAnimation(frontOrBack)
                    if tolua.isnull(monster) then
                        tower:resume()
                    else
                        local checkSpecialZorder = function(sid)
                            local skills = {"41009","41002","41003","41007","41008","41010","41011","41012","41015","41016","42016","42019"}
                            for i=1,#skills do
                                if tostring(sid)==skills[i] then
                                    return true
                                end
                            end
                            return false
                        end
                        local zorder = nil
                        if checkSpecialZorder(tower:getSkillId()) then
                            zorder = 5000
                        elseif tostring(tower:getSkillId())=="42017" then
                            zorder = 1
                        elseif tostring(tower:getSkillId())=="42002" then
                            zorder = GameInstance:sharedInstance():zorderForPosition(ccp(monster:getPositionX(),monster:getPositionY()))
                        else
                            zorder = GameInstance:sharedInstance():zorderForPosition(ccp(monster:getPositionX(),monster:getPositionY()-30))
                        end
                        self.gameScene:addChild(HeroSkill.new(self.gameScene,tower:getSkillId(),tower,monster),zorder)
                    end
                end})
            end)
        else
            --print("普通攻击才计算伤害值，技能攻击分别计算")
            local crit_proc = Attributes.get_crit_proc(tower:get_crit_proc(),monster.necrit_proc,tower:getBuffs(),monster:getBuffs())--爆击率设定
            --print("crit_proc is"..crit_proc)
            local if_crit = false--是否爆击状态标志
            --local crit_proc_random = math.random(100)/100
            local crit_proc_random = CPlusMathUtil:getRandomNum(100)/100
            if crit_proc_random <= crit_proc then
                if_crit = true
            else
            end
            local damageArgs = {
                atker_atk_power = tower:get_attack_power(),
                atker_sunder = tower:get_sunder(),
                atker_damage_lower_proc = tower:get_damage_lower_proc(),
                atker_crit_mul = tower:get_crit_mul(),
                atker_lvl = tower:getHeroLevel(),
                atker_class = tower:getHeroClass(),
                atker_skill_id = ifSkill and tower:getSkillId() or nil,
                dfcer_armor = monster.armor,
                dfcer_resist = monster.resist,
                dfcer_necrit_mul = monster.necrit_mul,
                is_crit = if_crit,
                atker_buffs = tower:getBuffs(),
                dfcer_buffs = monster:getBuffs()
            }
            
            local oldDamage = Attributes.get_damage(damageArgs)
            local damage =  oldDamage*tower:getDamageDegree()*2*(tower.enhanceDamageDegree)--普通攻击硬加2倍
            
            local attackFrames,attackAnimation = AnimationFactory.getHeroAnimation(tower:getHeroId(),"attack",frontOrBack)
            if leftOrRight=="r" then
                tower.hero:setFlipX(false)
            else
                tower.hero:setFlipX(true)
            end
            transition.execute(tower.hero,CCAnimate:create(attackAnimation),{
                onComplete = function()
                    tower:doHeroStandAnimation(frontOrBack)
                    if tolua.isnull(monster) then
                        --print("找到的目标已经死了,从新寻找攻击范围内得目标")
                        tower:resume()
                    else
                        local bullet = BulletFactory.getBullet(tower,monster,if_crit)
                        local zorder = GameInstance:sharedInstance():zorderForPosition(ccp(tower:getPositionX(),tower:getPositionY()))
                        bullet.damage = damage
                        self.gameScene:addChild(bullet,zorder)
                    end
                end
            })
        end
        --return ifSkill,damage
    else
        --print("missing 没有打中")
        monster:addMissingLabel()
    end
end

return BattleStoryGameLogic