
local DebugUtil = require("app.common.DebugUtil")
local CommUtil = require("app.common.CommUtil")
local HeroSkill = require("app.bullet.HeroSkill")
local AnimationFactory = require("app.factory.AnimationFactory")
local BulletFactory = require("app.factory.BulletFactory")
local scheduler = require(cc.PACKAGE_NAME .. ".scheduler")
local MathUtil = require("app.common.MathUtil")
local Rect = require("app.quadTree.Rect")
local BattleUtil = require("app.new.BattleUtil")

local GameLogic = class("GameLogic",function()
        return display.newNode()
end)

function GameLogic:ctor(gameScene)
    self:schedule(handler(self,self.onSchedule),0.5)
    self.gameScene = gameScene
    self.gameOver = false
    self.countTime = 0
end

function GameLogic:checkWinOrLose()
    if totalHP:get()<=0 then     --失败
        self.gameOver = true
        self.gameScene.parent:displayLosePanel()
        return
    end
    if theLastMonster and GameInstance:sharedInstance().monsters:count()<=0 then--胜利
        self.gameOver = true
        self.gameScene.parent:displayWinPanel()
        return
    end
end

function GameLogic:onSchedule()
    if pauseStatus or self.gameOver or self.gameScene.quadTree==nil then return end
    self:checkWinOrLose()
    self.countTime = self.countTime + 0.5
    local t1,t2 = math.modf(self.countTime/2)
    if t2==0 then
        for i=0,GameInstance:sharedInstance().monsters:count()-1 do
            local monster = GameInstance:sharedInstance().monsters:objectAtIndex(i)
            monster:updateGridIndex()
        end
    end
    for i=0,GameInstance:sharedInstance().towers:count()-1 do
        local tower = GameInstance:sharedInstance().towers:objectAtIndex(i)
        if tower:getHeroId()~=nil and (not tower.damaged) and (not tower.cooldown) then
            --print("gameScene.rects的大小是",#self.gameScene.rects)
            local targetMonster = self:getTargetForTower(tower)
            if targetMonster~=nil then
                self:playAttackAnimation(tower,targetMonster)
            else
                --print("没有找到目标")
            end
        else
            --print("cd时间中")
        end
    end
end

function GameLogic:playAttackAnimation(tower,targetMonster)
    tower.target = targetMonster
    local frontOrBack = nil
    local leftOrRight = nil

    if targetMonster:getPositionX()<tower:getPositionX() then
        if targetMonster:getPositionY()<tower:getPositionY() then
            frontOrBack = "front"
            leftOrRight = "l"
        else
            frontOrBack = "back"
            leftOrRight = "l"
        end
    else
        if targetMonster:getPositionY()<tower:getPositionY() then
            frontOrBack = "front"
            leftOrRight = "r"
        else
            frontOrBack = "back"
            leftOrRight = "r"
        end
    end
    --local attack_speed = Attributes.get_atk_spd(tower:getAttackSpeed(),true)
    local attack_speed = BattleUtil.getAttackSpeed(tower)
    self:fire(tower,targetMonster,attack_speed,frontOrBack,leftOrRight)
    local imageName = "hero_"..tower:getHeroId().."_".."attack".."_"..frontOrBack.."_1.png"
    local spriteFrame = display.newSpriteFrame(imageName)
    tower.hero:setDisplayFrame(spriteFrame)
    if leftOrRight=="r" then
        tower.hero:setFlipX(false)
    else
        tower.hero:setFlipX(true)
    end
end

function GameLogic:fire(tower,monster,attack_speed,frontOrBack,leftOrRight)
    tower:coolDown(attack_speed)


    tower.hero:stopAllActions()


    local crit_proc = BattleUtil.getCrit(tower,monster)--爆击率设定
    local if_crit = false--是否爆击状态标志
    local crit_proc_random = CPlusMathUtil:getRandomNum(100)/100
    if crit_proc_random <= crit_proc then
        if_crit = true
        monster:addCritLabel()
    else
    end

    local angerNum = 15
    local damage = BattleUtil.getDamage(tower,monster)--爆击率设定
    if if_crit then damage=damage*2 angerNum=25 end
    --print("普通伤害",damage)
            
    local attackFrames,attackAnimation = AnimationFactory.getHeroAnimation(tower:getHeroId(),"attack",frontOrBack)
    if leftOrRight=="r" then
        tower.hero:setFlipX(false)
    else
        tower.hero:setFlipX(true)
    end


    --[[
    if tower.angerNum>=100 then
        transition.execute(tower.hero,CCAnimate:create(attackAnimation),{
        onComplete = function()
            tower:doHeroStandAnimation(frontOrBack)
             tower:resetAnger()
            if tolua.isnull(monster) then
                --print("找到的目标已经死了,从新寻找攻击范围内得目标")
                tower:resume()
            else
                self.gameScene:addChild(HeroSkill.new(self,tower:getSkillId(),ccp(monster:getPositionX(),monster:getPositionY())),5000)
            end
        end
        })
    else]]
        transition.execute(tower.hero,CCAnimate:create(attackAnimation),{
        onComplete = function()
            tower:doHeroStandAnimation(frontOrBack)
            tower:addAnger(angerNum)
            if tolua.isnull(monster) then
                --print("找到的目标已经死了,从新寻找攻击范围内得目标")
                tower:resume()
            else
                local bullet = BulletFactory.getBullet(tower,monster,false)
                local zorder = GameInstance:sharedInstance():zorderForPosition(ccp(tower:getPositionX(),tower:getPositionY()))
                bullet.damage = damage
                self.gameScene:addChild(bullet,zorder)
            end
        end
        })

    --end
end


function GameLogic:getTargetForTower(tower)
    --print("tower的rangeIndex是",#tower.gridIndexs)
    local everTarget = tower.target
    if not tolua.isnull(everTarget) and everTarget~=nil and not everTarget.dead then
        local curx,cury,size = everTarget:getMonsterPositionAfterTowerAnimation(0.4)
        local distance = ccpDistance(ccp(curx,cury),ccp(tower:getPositionX(),tower:getPositionY()))
        local range = tower:getRange()
        if distance<range then
            return everTarget
        end
    end
    local targetMonster = nil
    local lastDistance = 0--此前的最短距离
    local num = 0
    for i=1,#tower.gridIndexs do
        local gridIndex = tower.gridIndexs[i]
        local monsters = self.gameScene.quadTree.grids[gridIndex]
        for k, v in ipairs(monsters) do
            num = num + 1 
            if not tolua.isnull(v) and v~=nil and not v.dead and self:towerCanAttackMonster(tower,v) then
                local curx,cury,size = v:getMonsterPositionAfterTowerAnimation(0.4)
                if targetMonster~=nil then
                    local distance = ccpDistance(ccp(curx,cury),ccp(tower:getPositionX(),tower:getPositionY()))
                    local range = tower:getRange()
                    if distance<range then
                        local dist1 = v.curWaypoint
                        local dist2 = targetMonster.curWaypoint
                        if dist1 > dist2 then
                            targetMonster = v
                            lastDistance = distance
                        end 
                    end
                else
                    local distance = ccpDistance(ccp(curx,cury),ccp(tower:getPositionX(),tower:getPositionY()))
                    local range = tower:getRange()
                    if distance<range then
                        targetMonster = v
                        lastDistance = distance
                    end
                end
            end
        end
    end
    --print("检测了多少个怪物",num,GameInstance:sharedInstance().monsters:count())
    return targetMonster
end


--[[
function GameLogic:getTargetForTower(tower)
    --print("tower的rangeIndex是",#tower.gridIndexs)
    local targetMonster = nil
    local lastDistance = 0--此前的最短距离
    for i = 0, GameInstance:sharedInstance().monsters:count()-1 do
    --for i = 1, #tower.targets do
        local v = GameInstance:sharedInstance().monsters:objectAtIndex(i)
        --local v = tower.targets[i]
        if not v:ifDead() and self:towerCanAttackMonster(tower,v) then
            local curx,cury,size = v:getMonsterPositionAfterTowerAnimation(0.4)
            if targetMonster~=nil then
                local distance = MathUtil.ccpDistance(ccp(curx,cury),ccp(tower:getPositionX(),tower:getPositionY()))
                local range = tower:getRange()
                if distance<range*range then
                    local dist1 = v.curWaypoint
                    local dist2 = targetMonster.curWaypoint
                    if dist1 > dist2 then
                        targetMonster = v
                        lastDistance = distance
                    end 
                end
            else
                local distance = MathUtil.ccpDistance(ccp(curx,cury),ccp(tower:getPositionX(),tower:getPositionY()))
                local range = tower:getRange()
                if distance<range*range then
                    targetMonster = v
                    lastDistance = distance
                end
            end
        end
    end
    return targetMonster
end]]


function GameLogic:towerCanAttackMonster(tower,monster)
    local canAttack = true
    if monster.air then
        if tower:canAttackAir() then
            canAttack = true
        else
            canAttack = false
            return canAttack
        end
    end
    if monster.underground then
        if tower:canAttackUnderground() then
            canAttack = true
        else
            canAttack = false
            return canAttack
        end
    end
    if monster.stealth then
        if tower:canAttackStealth() then
            canAttack = true
        else
            canAttack = false
            return canAttack
        end
    end
    return canAttack
end

return GameLogic