
local DebugUtil = require("app.common.DebugUtil")
local CommUtil = require("app.common.CommUtil")
local HeroSkill = require("app.bullet.HeroSkill")
local AnimationFactory = require("app.factory.AnimationFactory")
local BulletFactory = require("app.factory.BulletFactory")
local scheduler = require("framework.scheduler")


local MonsterInfoLayer = class("MonsterInfoLayer",function()
    local layer = display.newLayer()
    return layer
end)

function MonsterInfoLayer:ctor(parent)
    self.infoBars = {}
    if DEBUG==1 then
        --print("呵呵呵") 
        --return 
    end
	self.gameScene = parent
    self:setTouchEnabled(true)
    self:registerScriptTouchHandler(handler(self,self.onTouch))
end

function MonsterInfoLayer:onTouch(event, x, y)
    if event == "began" then
        if CCDirector:sharedDirector():isPaused() then
            return false
        end
        if self.gameScene.skillPanel and self.gameScene.skillPanel.skillId~=-1 then return false end
        local monster = self:getOneMonsterInTouchPoint(x,y)
        if monster~=nil then
            --print("怪物的属性",monster.attackerProperties.body_id,monster.attackerProperties.id,monster.attackerProperties.scale)
            if self.gameScene.canFocus then
                if self.gameScene.focusMonster and not tolua.isnull(self.gameScene.focusMonster) then
                    self.gameScene.focusMonster:cleanFocusFlag()
                    self.gameScene.focusMonster = nil
                end
                self.gameScene.focusMonster = monster
                self.gameScene:startFocusFireOneTime()
                monster:addFocusFlag()
                if self.gameScene.hand and (gBattle.data.home_pass_id==gPlayer.data.pve_pass_id and gBattle.data.home_pass_id==60010) then
                    scheduler.unscheduleGlobal(self.gameScene.handler)--释放之后取消
                    self.gameScene.hand:removeSelf()
                    self.gameScene.focusLightingSprite:removeSelf()
                    self.gameScene.hand = nil
                end
            else
                --self:displayMonsterInfo(monster)
            end
        else
            if self.gameScene.canFocus then
                local frames,animation = require("app.factory.AnimationFactory").getFingerAniamtion()
                local quanquan = display.newSprite(frames[1],x,y):addTo(display.getRunningScene(),50000)
                quanquan:setScale(0.4)
                transition.execute(quanquan,CCAnimate:create(animation), {
                    onComplete = function()
                        quanquan:removeSelf()
                    end,
                })
            end
        end

        --[[
        for i = 0, self.gameScene.golds:count()-1 do
            local gold = tolua.cast(self.gameScene.golds:objectAtIndex(i), "CCSprite")
            if (gold:getCascadeBoundingBox():containsPoint(ccp(x,y))) and gold.cantouch then
                local endX,endY = self.gameScene.gameHud:getGoldPosition()
                local distance = ccpDistance(ccp(gold:getPositionX(),gold:getPositionY()),ccp(endX,endY))
                local time = distance/600
                gold.cantouch = false
                transition.execute(gold, CCMoveTo:create(time,ccp(endX,endY)),{
                        onComplete = function()
                            CommUtil.playSound("music/copper.mp3")
                            self.gameScene.golds:removeObject(gold)
                            gold:removeSelf()
                            self.gameScene.gameHud:updateGoldNumText()
                        end,
                })
            end
        end
        if self.gameScene.golds:count()>0 then
            return true
        else
            return false
        end]]
    elseif event == "moved" then
        --[[
        for i = 0, self.gameScene.golds:count()-1 do
            local gold = tolua.cast(self.gameScene.golds:objectAtIndex(i), "CCSprite")
            if (gold:getCascadeBoundingBox():containsPoint(ccp(x,y))) and gold.cantouch then
                local endX,endY = self.gameScene.gameHud:getGoldPosition()
                local distance = ccpDistance(ccp(gold:getPositionX(),gold:getPositionY()),ccp(endX,endY))
                local time = distance/600
                gold.cantouch = false
                transition.execute(gold, CCMoveTo:create(time,ccp(endX,endY)),{
                        onComplete = function()
                            CommUtil.playSound("music/copper.mp3")
                            self.gameScene.golds:removeObject(gold)
                            gold:removeSelf()
                            self.gameScene.gameHud:updateGoldNumText()
                        end,
                })
            end
        end]]

    end
end

function MonsterInfoLayer:getOneMonsterInTouchPoint(x,y)
    for i=0,GameInstance:sharedInstance().monsters:count()-1 do
        local monster = GameInstance:sharedInstance().monsters:objectAtIndex(i)
        if monster~=nil and not tolua.isnull(monster) and (monster.mainSprite:getCascadeBoundingBox():containsPoint(ccp(x,y))) then
           return monster
        end
    end
    return nil
end

function MonsterInfoLayer:cleanMonsterInfoBars()
    for k, v in ipairs(self.infoBars) do
        if v~=nil then
            v:removeSelf()
            table.remove(self.infoBars,k)
        end
    end
end


function MonsterInfoLayer:displayMonsterInfo(monster)

    self:cleanMonsterInfoBars()

    self.gameScene:cleanTowerProperties()

    local bg = nil
    local defence = nil

    if monster.armor>monster.resist then
        bg = display.newSprite("ui/home/home_tower_state_1.png"):pos(display.cx,display.top-20):addTo(display.getRunningScene(),8000)
        defence = monster.armor
    else
        bg = display.newSprite("ui/home/home_tower_state_4.png"):pos(display.cx,display.top-20):addTo(display.getRunningScene(),8000)
        defence = monster.resist
    end



    self.infoBars[#self.infoBars+1] = bg
                
    ui.newTTFLabel({
        text = monster:getName().."",
        font = "Marker Felt",
        x=60,
        y=25,
        color = display.COLOR_WHITE,
        size = 15,
        align = ui.TEXT_ALIGN_CENTER
        }):addTo(bg)

    ui.newTTFLabel({
        text = defence.."",
        font = "Marker Felt",
        x=170,
        y=25,
        color =  display.COLOR_WHITE,
        size = 15,
        align = ui.TEXT_ALIGN_CENTER
    }):addTo(bg)
                
    ui.newTTFLabel({
        text = monster.fullHp.."",
        font = "Marker Felt",
        x=270,
        y=25,
        color =  display.COLOR_WHITE,
        size = 15,
        align = ui.TEXT_ALIGN_CENTER
    }):addTo(bg)
                
    ui.newTTFLabel({
        text = monster.attackerProperties.move_speed.."",
        font = "Marker Felt",
        x=375,
        y=25,
        color = display.COLOR_WHITE,
        size = 15,
        align = ui.TEXT_ALIGN_CENTER
    }):addTo(bg)


    bg:performWithDelay(function()
        bg:removeSelf()
        table.remove(self.infoBars,#self.infoBars)
    end,4)

end


return MonsterInfoLayer