require("app.data.NPCData")
local NpcUtil = require("app.common.NpcUtil")
local npc_words = require("app.client.desc.npc_words")
local NPC = require("app.actor.NPC")


local NPCManager = class("NPCManager",function()
		return display.newNode()
end)



function NPCManager:ctor(gameScene)
	self.gameScene = gameScene


    self.stop = false

    gameScene.npcs = {}
    self.time = 0

    self:produceNpc()
    self:produceNpc()

    --self:schedule(handler(self, self.produceNpc),7)
    --self:schedule(handler(self, self.addBubble),15)
    --self:schedule(handler(self, self.npcLogic),2)
    --self:schedule(handler(self, self.updateNPCZorder),1)


    self.countTime=0
    self:schedule(handler(self, self.onSchedule),1)

    self.heroIndex = -1 --控制哪个英雄的转向
    self.addMoneyFlag = true

end

function NPCManager:onSchedule()
    if self.stop then
        for k, v in ipairs(self.gameScene.npcs) do
            v:clean()
        end
        return 
    end
    self.countTime = self.countTime + 1

    self:updateNPCZorder()

    local t1,t2 = math.modf(self.countTime/7)

    if t2==0 then
        self:produceNpc()
        self:importTip()
    end
    
    local t3,t4 = math.modf(self.countTime/5)
    if t4==0 then
        self:addBubble()
    end

    local t5,t6 = math.modf(self.countTime/2)

    if t6==0 then
        self:npcLogic()
        self:changeHeroAnimation()
    end

end

--改变主城塔上英雄的动画
function NPCManager:changeHeroAnimation()
    local towers = {}
    for k, v in ipairs(self.gameScene.towerFactory.towers) do
        local tower = v
        if tower.towerInfo~=nil then
            towers[#towers+1] = tower
        end
    end
    local getTowerIndex = function()
        local towerIndex = math.random(1,#towers)
        local i=1
        while i<3 and self.heroIndex==towerIndex do
            towerIndex = math.random(1,#towers)
            i = i + 1
        end
        self.heroIndex = towerIndex
        return towerIndex
    end
    local towerIndex = getTowerIndex()
    local tower = towers[towerIndex]
    if tower and tower.hero~=nil then
        if math.random(1,2)==1 then
            tower.hero:setFlipX(true)
        else
            tower.hero:setFlipX(false)
        end
        if tower.hero.state=="attack" then
            tower:doHeroStandAnimation("front")
        else
            tower:doHeroAttackAnimation("front")
        end
        
    end
    --print("towers的大小是",#towers,#self.gameScene.towerFactory.towers)
end

function NPCManager:updateNPCZorder()
    for k, v in ipairs(self.gameScene.npcs) do
        local npc = v
        local x,y,size = npc:getTextureCenterPointInWorld(npc:getPositionX(),npc:getPositionY())
        local zorder = GameInstance:sharedInstance():zorderForPosition(ccp(x,y))
        self.gameScene:reorderChild(npc,zorder)
    end
end

function NPCManager:npcLogic()
    for k, v in ipairs(self.gameScene.npcs) do
        local npc = v
        if npc:getBodyId()=="npc_1017" then
            --print("这个是货郎")
            --CCUserDefault:sharedUserDefault():getIntegerForKey("onLineTime")
            local ifTodayFirstLogin = CCUserDefault:sharedUserDefault():getBoolForKey("todayFirstLogin")
            local onLineTime = CCUserDefault:sharedUserDefault():getIntegerForKey("onLineTime")

            local t1,t2 = math.modf(onLineTime/(5*60))

            if t2 <=  3/(5*60) and self.addMoneyFlag then
                CCUserDefault:sharedUserDefault():setIntegerForKey("timesEvery5Minutes",CCUserDefault:sharedUserDefault():getIntegerForKey("timesEvery5Minutes")+1)
                --print("送钱了啊",t1,t2,math.floor(t2),onLineTime)
                self.addMoneyFlag = false
                self:performWithDelay(function()
                    self.addMoneyFlag = true
                end,2)
            end

            if CCUserDefault:sharedUserDefault():getIntegerForKey("getMoneyTimes")<CCUserDefault:sharedUserDefault():getIntegerForKey("timesEvery5Minutes") and (not npc.addDingMoney) then
                npc:addMoneySprite()
            end
        elseif npc:getBodyId()=="npc_1001" then
            --print("这个是打更哥")
            local word = NpcUtil.getAdvanceContent()
            if word~=nil and word~="" then
                if npc.state=="walk" then
                    if (not npc.coolDown) and not self.gameScene.towerFactory:isBuilding() then
                        npc.coolDown = true
                        npc:stopAllActions()
                        npc.mainSprite:stopAllActions()
                        npc:addBubbleAndWords(word)
                        npc:performWithDelay(function()
                            npc.state="walk"
                            npc:cleanBubble()
                            local waypoint = tolua.cast(npc._waypoints:objectAtIndex(npc.curWaypoint),"WayPoint")
                            if waypoint.underSea then
                                npc:doWalkAnimation(waypoint.direction)
                            else
                                npc:doWalkAnimation(waypoint.direction)
                            end
                            npc:walk()
                        end,1)
                        npc:performWithDelay(function()
                            npc.coolDown = false
                        end,npc:getCoolDownTime())
                    end
                end
            end
        elseif npc:getBodyId()=="npc_1003" then
            local gift2Cat = NpcUtil.catGetCookieNumber()
            if gift2Cat>=10 and npc.state~="giveGift" and (not npc.addDingGift) then npc.state="giveGift" npc:addGiftSprite() end
        end
    end
end

function NPCManager:getDaGengBrother()--如果打更哥在场景中，返回打更哥
    for k, v in ipairs(self.gameScene.npcs) do
        local npc = v
        if npc:getBodyId()..""=="npc_1001" then
            return npc
        end
    end
    return nil
end

function NPCManager:importTip()
    local npc = self:getDaGengBrother()
    if npc==nil then return end
    if npc.state=="walk" then
        if not self.gameScene.towerFactory:isBuilding() then
            npc:stopAllActions()
            npc.mainSprite:stopAllActions()
            npc:addBubbleAndWords(nil)
            npc.coolDown = true

            if (npc:getParent()) then
                npc:performWithDelay(function()
                    npc.state="walk"
                    npc:cleanBubble()
                    local waypoint = tolua.cast(npc._waypoints:objectAtIndex(npc.curWaypoint),"WayPoint")
                    if waypoint.underSea then
                        npc:doWalkAnimation(waypoint.direction)
                    else
                        npc:doWalkAnimation(waypoint.direction)
                    end
                    npc:walk()
                end,2)
                    npc:performWithDelay(function()
                    npc.coolDown = false
                end,npc:getCoolDownTime())
            end
        end
    else
        npc:cleanBubble()
    end
end

function NPCManager:addBubble()
    local npc = nil 
    local findTime = 0
    while npc==nil do
        npc = self:getOneNPC()
        findTime = findTime + 1
        if findTime>2 then
            --print("当前没有npc可以使用")
            return 
        end
    end
    if npc.state=="walk" then
        if not self.gameScene.towerFactory:isBuilding() then
            npc:stopAllActions()
            npc.mainSprite:stopAllActions()
            npc:addBubbleAndWords(nil)
            npc.coolDown = true

            if (npc:getParent()) then
                npc:performWithDelay(function()
                    npc.state="walk"
                    npc:cleanBubble()
                    local waypoint = tolua.cast(npc._waypoints:objectAtIndex(npc.curWaypoint),"WayPoint")
                    if waypoint.underSea then
                        npc:doWalkAnimation(waypoint.direction)
                    else
                        npc:doWalkAnimation(waypoint.direction)
                    end
                    npc:walk()
                end,1)
                    npc:performWithDelay(function()
                    npc.coolDown = false
                end,npc:getCoolDownTime())
            end
        end
    else
        npc:cleanBubble()
    end
end

function NPCManager:getOneNPC()
    if #self.gameScene.npcs==0 then return nil end
    local npcIndex = math.random(1,#self.gameScene.npcs)-1
    --print("getOneNpc",npcIndex)
    if npcIndex<1 then npcIndex=1 end
    if npcIndex>(#self.gameScene.npcs) then npcIndex=(#self.gameScene.npcs) end
    local npc = self.gameScene.npcs[npcIndex]
    if not npc.coolDown and npc.curWaypoint >3 then
        return npc
    end
    return nil
end

function NPCManager:ifExistSameNPC(body_id)
    for k, v in ipairs(self.gameScene.npcs) do
        local npc = v
        if npc:getBodyId()..""==body_id.."" then
            return true
        end
    end
    return false
end

function NPCManager:getNPCData()
    for i=1,3 do
        if not self:ifExistSameNPC(npcTypeList[i].body_id) then
            return npcTypeList[i]
        end
    end
    return nil
end

function NPCManager:produceNpc()
    if GameInstance:sharedInstance().paths:count()<=0 then return end
    local npcData = self:getNPCData()
    if npcData==nil then return end
    
    local path,fPathIndex = self.gameScene:getNPCWalkPath()
    if path==nil or path:count()<=0 then return end
    local spawnPoint = tolua.cast(path:objectAtIndex(0),"WayPoint")--出生点

    local offset = 20*npcData.scale
    
    local xPos = spawnPoint.position.x + CommUtil.getBlankWidth()
    local yPos = spawnPoint.position.y - offset
    

    local word = nil
    if npcData.body_id=="npc_1001" then
        local time=os.time()
        local tab=os.date("*t")
        local hour = tab.hour
        word=npc_words[npcData.body_id][""..hour]
        if word==nil or #word<=0 then
            word={
                "呵呵",
            }
        end
    else
        word = npc_words[npcData.body_id]
    end

    local npcProperties = {
        npcType = npcData.npcType,
        move_speed = npcData.move_speed,
        body_id = npcData.body_id,
        position = ccp(xPos,yPos),
        move_speed = npcData.move_speed,
        walk_animation_time = npcData.walk_animation_time,
        scale = npcData.scale,
        offset = offset,
        words = word,
        coolDownTime = npcData.coolDownTime,
    }
    
    local zorder = GameInstance:sharedInstance():zorderForPosition(ccp(xPos,yPos))
    local npc = NPC.new(self.gameScene,npcProperties,path,true)
    self.gameScene:addChild(npc,zorder)
    self.gameScene.npcs[#self.gameScene.npcs + 1] = npc
    --GameInstance:sharedInstance().npcs:addObject(npc
        npc:walk()

end

function NPCManager:clean()
    self.stop = true
    for k, v in ipairs(self.gameScene.npcs) do
        v:clean()
    end
end

return NPCManager