
local DebugUtil = require("app.common.DebugUtil")
local CommUtil = require("app.common.CommUtil")
local HeroSkill = require("app.bullet.HeroSkill")
local AnimationFactory = require("app.factory.AnimationFactory")
local BulletFactory = require("app.factory.BulletFactory")
local NpcUtil = require("app.common.NpcUtil")
local NPC = require("app.actor.NPC")


local NPCTouchManager = class("NPCTouchManager",function()
    local layer = display.newLayer()
    return layer
end)

function NPCTouchManager:ctor(parent)
	self.gameScene = parent
    self.canTouch = true
    self:setTouchEnabled(true)
    self:registerScriptTouchHandler(handler(self,self.onTouch))
end

function NPCTouchManager:enableTouch()
    self.canTouch = true
end

function NPCTouchManager:disableTouch()
    self.canTouch = false
end

function NPCTouchManager:onTouch(event, x, y)
    --if event == "began" then
    --print("self.canTouch",self.canTouch)
    if event == "began" and self.canTouch then
        --print("npc可以点击",self.canTouch)
        local npc = self:getOneNPCInTouchPoint(x,y)
        if npc~=nil then
            --print("点击了一个npc")
            if npc:ifCanGiveMoney() then
                --print("送钱了",npc.state)
                NpcUtil.npcGetCopper(CCUserDefault:sharedUserDefault():getIntegerForKey("timesEvery5Minutes"),function()
                    CCUserDefault:sharedUserDefault():setIntegerForKey("getMoneyTimes",CCUserDefault:sharedUserDefault():getIntegerForKey("timesEvery5Minutes"))
                    npc:removeMoneySprite()
                end)
            elseif npc:ifCanGiveGift() then
                --print("送礼物了",npc.state)
                NpcUtil.catUseCookie()
                npc:removeGiftSprite()
            elseif npc.state=="walk" then
                if not self.gameScene.towerFactory:isBuilding() then
                    npc:stopAllActions()
                    npc.mainSprite:stopAllActions()
                    npc:addBubbleAndWords(nil)
                    npc:performWithDelay(function()
                        npc.state="walk"
                        npc:cleanBubble()
                        local waypoint = tolua.cast(npc._waypoints:objectAtIndex(npc.curWaypoint),"WayPoint")
                        if waypoint.underSea then
                            npc:doWalkAnimation(waypoint.direction)
                        else
                            npc:doWalkAnimation(waypoint.direction)
                        end
                        npc:startWalk()
                    end,2)
                end
            else
                npc:cleanBubble()
            end
        end
        return false
    end
end

function NPCTouchManager:getOneNPCInTouchPoint(x,y)
    for k, v in ipairs(self.gameScene.npcs) do
        local npc = v
        local nodePoint = npc.mainSprite:convertToNodeSpace(ccp(x,y))
        local mainRect = npc.mainSprite:getCascadeBoundingBox()
        if mainRect:containsPoint(ccp(x,y)) then
            return npc
        end
    end
end


return NPCTouchManager