--
-- Author: hejiangcheng
-- Date: 2014-10-28 21:20:37
--
local SkillLvlClass = class("SkillLvlClass")
SkillLvlClass.__index = SkillLvlClass

function SkillLvlClass:ctor()
	--self.data[skill_id] = lvl
	self.data = {}
	self.qianKunBagData = {}
end

function SkillLvlClass:initServData(data)
	if type(data) ~= "table" or #data < 1 then return end
	for i, v in pairs(data) do
		self.data[toint(v.skill_id)] = toint(v.skill_lvl)
	end
end

--修改技能等级
function SkillLvlClass:modifySkillLvl(skill_id, lvl)
	assert(lvl-1 == self:getSkillLvlBySkillId(skill_id),"修改本地技能等级错误 skill_id = %d, newLvl = %d, oldLvl = %d",skill_id,lvl,self:getSkillLvlBySkillId(skill_id))
	self.data[toint(skill_id)] = lvl
end

--返回技能等级
function SkillLvlClass:getSkillLvlBySkillId(skill_id)
	return self.data[toint(skill_id)] or 1
end

--返回技能列表
function SkillLvlClass:getSkillTable()
	local tb = {}
	for i, v in pairs(gBeauty:getBeautySkills()) do
		table.insert(tb, {v, self:getSkillLvlBySkillId(v)})
	end

	local lead_hero_skill_id = 0
	for i, v in pairs(gHero:getHeroBag()) do
		if i == 1 then 
			lead_hero_skill_id = v.skill_id 
		else
			if v.quality:get() ~= 1 then
				table.insert(tb, {v.skill_id, self:getSkillLvlBySkillId(v.skill_id)})
			end
		end
	end
	table.sort(tb, function(a,b) return a[1] < b[1] end)
	table.insert(tb,1,{lead_hero_skill_id, self:getSkillLvlBySkillId(lead_hero_skill_id)})
	return tb
end


-----------------------------------------------------------
--初化乾坤袋数据
function SkillLvlClass:getQianKuanServData(callback)
	local function func(cmd ,body)
		self.qianKunBagData = {}
		if body.ret == '0' then
			if type(body.UniverseItem) ~= "table" or #body.UniverseItem < 1 then
				callback()
				return
			end
			for i, v in pairs(body.UniverseItem) do
				if toint(v.stuff_cnt) > 0 then
					table.insert(self.qianKunBagData, {object_id = toint(v.object_id), num = Number.new(toint(v.stuff_cnt))})
				end
			end
			callback()
		else
			CommUtil.errService(display.getRunningScene())
			callback()
		end
	end
    require("app.sock.SockNode").new({
	        cmd = 304,
	        body = {
    			player_id  = gPlayer.data.player_id, -- 玩家id
	        },
	        freeze = true,
	        callback = func,
        }):addTo(display.getRunningScene(),SECOND_MENU_Z_ORDER )
end

--合成物品并修改乾坤袋

--[[
local para = {}
para.result_item
para.part1_id
para.part1_num
para.stuff_cnt --校验数量
para.callback(FormulaCombineSuccessFlag)

gSkillLvl:combineItem(para)
]]
function SkillLvlClass:combineItem(para)
	local function func(cmd ,body)
		if body.ret == '0' then
			para.callback(tonumber(body.FormulaCombineSuccessFlag))
		else
			para.callback(0) --失败
			-- CommUtil.errService(display.getRunningScene())
		end
	end
    require("app.sock.SockNode").new({
	        cmd = 308,
	        body = {
    			player_id  = gPlayer.data.player_id, -- 玩家id
    			result_item	= para.result_item,
    			part1_id	= para.part1_id,
    			part1_num	= para.part1_num,
    			stuff_cnt	= para.stuff_cnt,
	        },
	        freeze = true,
	        callback = func,
        }):addTo(display.getRunningScene(),SECOND_MENU_Z_ORDER )
end

--通过object_id返回数量
function SkillLvlClass:getNumById(object_id)
	if toint(object_id) < 39000 then
		-- print("在传统背包中取得数量")
		return gItem:getItemNum(toint(object_id))
	end
	-- print("在乾坤袋中取得数量")
	if #self.qianKunBagData < 1 then return 0 end
	for i, v in pairs(self.qianKunBagData) do
		if v.object_id == object_id then return v.num:get() end
	end
	return 0
end

--减去道具
function SkillLvlClass:cutItemNum(object_id, num)
	if toint(num) <= 0 then return end
	if toint(object_id) <= 0 then return end

	if toint(object_id) < 39000 then
		gItem:CutItemNum(object_id, num)
		return
	end
	-- print("在乾坤袋中减去数量")
	for i, v in pairs(self.qianKunBagData) do
		if v.object_id == object_id then 
			if v.num:get() < num then echoError("减去乾坤袋中数据错误,object_id=%d,num=%d,cut_num=%d", object_id, v.num:get(), num) end
			if v.num:get() == num then
				table.remove(self.qianKunBagData, i)
			else
				v.num:set(v.num:get() - num)
			end
			return
		end
	end
end

--加上道具
function SkillLvlClass:addItemNum(object_id, num)
	if toint(num) <= 0 then return end
	if toint(object_id) <= 0 then return end

	if toint(object_id) < 39000 then
		gItem:insertItem(object_id, num)
		return
	end
	-- print("在乾坤袋中加上数量")
	if #self.qianKunBagData > 0 then
		for i, v in pairs(self.qianKunBagData) do
			if v.object_id == object_id then 
				v.num:set(v.num:get() + num)
				return
			end
		end
	end
	table.insert(self.qianKunBagData, {object_id=object_id, num=Number.new(num)})
end

--
function SkillLvlClass:getIndexById(object_id)
	if #self.qianKunBagData < 1 then return 0 end
	for i, v in pairs(self.qianKunBagData) do
		if v.object_id == object_id then
			return i
		end
	end
	return 0
end

-----------------------------------------------------------
function SkillLvlClass:init4PlayersServData(data)
	-- print('+++++++++++++')
	-- DebugUtil.print_table2(data)
	-- print('+++++++++++++')
	if type(data) ~= "table" or #data < 1 then return end
	local tb = {}
	for i, v in pairs(data) do
		table.insert(tb, {
			player_id = toint(v.player_id),
			all_power = toint(v.all_power),
			resist_power = toint(v.resist_power),
			name = string.urldecode(v.name),
			lvl = toint(v.level),
		})
	end
	return tb
end

return SkillLvlClass