local MathUtil = require("app.common.MathUtil")

local AnimationFactory = {}


function AnimationFactory.getHeroRideShadowAnimation(type)
    if type=="hunter" then
        local imageName = "tower_hunter_wheels_%d.png"
        local frames = display.newFrames(imageName, 1,8)
        local animation = display.newAnimation(frames,2/8)
        return frames,animation
    elseif type=="warrior" then
        local imageName = "shadow_warrior_sword.png"
        --local frames = display.newFrames(imageName, 1,1)
        --local animation = display.newAnimation(frames,1)
        return imageName
    elseif type=="wizard" then
        local imageName = "shadow_wizard_crane_%d.png"
        local frames = display.newFrames(imageName, 1,8)
        local animation = display.newAnimation(frames,1/8)
        return frames,animation
    elseif type=="mage" then
        local imageName = "shadow_mage_lotus_%d.png"
        local frames = display.newFrames(imageName,1,8)
        local animation = display.newAnimation(frames,3/8)
        return frames,animation
    end
end

function AnimationFactory.getHeroRideAnimation(type)
    if type=="hunter" then
        local imageName = "tower_hunter_wheels_%d.png"
        local frames = display.newFrames(imageName, 1,8)
        local animation = display.newAnimation(frames,1/8)
        return frames,animation
    elseif type=="warrior" then
        local imageName = "tower_warrior_lv%d.png"
        local frames = display.newFrames(imageName, 1,1)
        local animation = display.newAnimation(frames,1)
        return frames,animation
    elseif type=="wizard" then
        local imageName = "tower_wizard_crane_%d.png"
        local frames = display.newFrames(imageName, 1,8)
        local animation = display.newAnimation(frames,1/8)
        return frames,animation
    elseif type=="mage" then
        local imageName = "tower_mage_lotus_%d.png"
        local frames = display.newFrames(imageName,1,8)
        local animation = display.newAnimation(frames,1/8)
        return frames,animation
    end
end

function AnimationFactory.getHeroAnimation(heroId,attackOrStand,frontOrBack)
    local totalTime = 0.5
    if attackOrStand=="stand" then
        totalTime = 1
    end
    local imageName = "hero_"..heroId.."_"..attackOrStand.."_"..frontOrBack.."_%d.png"
    local frames = display.newFrames(imageName, 1,8)
    local animation = display.newAnimation(frames, totalTime/8)
    return frames,animation
end


function AnimationFactory.getMonsterAnimation(direction,body_id,walk_animation_time)
    local totalFrames = 8
    if direction==true then
        local imageName = "monster_"..body_id.."_walk_front_%d.png"
        local frames = display.newFrames(imageName, 1,totalFrames)
        local animation = display.newAnimation(frames, walk_animation_time / totalFrames)
        return animation
    else
        local imageName = "monster_"..body_id.."_walk_back_%d.png"
        local frames = display.newFrames(imageName, 1,totalFrames)
        local animation = display.newAnimation(frames, walk_animation_time / totalFrames)
        return animation
    end
end


function AnimationFactory.getSwimMonsterAnimation(underSea,direction,body_id,walk_animation_time)
    local totalFrames = 8
    if underSea then
        if direction then
            local imageName = "monster_"..body_id.."_swimming_front_%d.png"
            local frames = display.newFrames(imageName, 1,totalFrames)
            local animation = display.newAnimation(frames, walk_animation_time / totalFrames)
            return animation
        else
            local imageName = "monster_"..body_id.."_swimming_back_%d.png"
            local frames = display.newFrames(imageName, 1,totalFrames)
            local animation = display.newAnimation(frames, walk_animation_time / totalFrames)
            return animation
        end
    else
        if direction==true then
            local imageName = "monster_"..body_id.."_walk_front_%d.png"
            local frames = display.newFrames(imageName, 1,totalFrames)
            local animation = display.newAnimation(frames, walk_animation_time / totalFrames)
            return animation
        else
            local imageName = "monster_"..body_id.."_walk_back_%d.png"
            local frames = display.newFrames(imageName, 1,totalFrames)
            local animation = display.newAnimation(frames, walk_animation_time / totalFrames)
            return animation
        end
    end
end


function AnimationFactory.getMonsterAnimation_bak(underSea,direction,body_id,walk_animation_time)
    local totalFrames = 8
    if body_id==70053 or  body_id==70042 then
        totalFrames = 16
    end
    if underSea then
        if direction then
            local imageName = "monster_"..body_id.."_swimming_front_%d.png"
            local frames = display.newFrames(imageName, 1,totalFrames)
            local animation = display.newAnimation(frames, walk_animation_time / totalFrames)
            return animation
        else
            local imageName = "monster_"..body_id.."_swimming_back_%d.png"
            local frames = display.newFrames(imageName, 1,totalFrames)
            local animation = display.newAnimation(frames, walk_animation_time / totalFrames)
            return animation
        end
    else
        if direction==true then
            local imageName = "monster_"..body_id.."_walk_front_%d.png"
            local frames = display.newFrames(imageName, 1,totalFrames)
            local animation = display.newAnimation(frames, walk_animation_time / totalFrames)
            return animation
        else
            local imageName = "monster_"..body_id.."_walk_back_%d.png"
            local frames = display.newFrames(imageName, 1,totalFrames)
            local animation = display.newAnimation(frames, walk_animation_time / totalFrames)
            return animation
        end
    end
end

function AnimationFactory.getBossAnimation(direction,body_id,walk_animation_time)--获取陆地行走boss动画
    local totalFrames = 16
    if direction==true then
        local imageName = "monster_"..body_id.."_walk_front_%d.png"
        local frames = display.newFrames(imageName, 1,totalFrames)
        local animation = display.newAnimation(frames, walk_animation_time / totalFrames)
        return animation
    else
        local imageName = "monster_"..body_id.."_walk_back_%d.png"
        local frames = display.newFrames(imageName, 1,totalFrames)
        local animation = display.newAnimation(frames, walk_animation_time / totalFrames)
        return animation
    end
end

function AnimationFactory.getSkillCastAnimation(skill_id,frameNO,time)
    local castAnimation = display.getAnimationCache("cast_skill_"..skill_id)
    if castAnimation~=nil then return castAnimation end
    local frames = display.newFrames("cast_"..skill_id.."_%02d.png", 1,frameNO)
    local animation = display.newAnimation(frames,time/frameNO)
    display.setAnimationCache("cast_skill_"..skill_id,animation)
    return animation
end

function AnimationFactory.getSkillChannelAnimation(skill_id,frameNO,time)
    local channelAnimation = display.getAnimationCache("channel_skill_"..skill_id)
    if channelAnimation~=nil then return channelAnimation end
    local frames = display.newFrames("channel_"..skill_id.."_%02d.png", 1,frameNO)
    local animation = display.newAnimation(frames, time/frameNO)
    display.setAnimationCache("channel_skill_"..skill_id,animation)
    return animation
end



function AnimationFactory.getBossAttackAnimation(direction,body_id,walk_animation_time)--获取陆地行走boss动画
    local totalFrames = 16
    if direction==true then
        local imageName = "monster_"..body_id.."_Remove_front_%d.png"
        local frames = display.newFrames(imageName, 1,totalFrames)
        local animation = display.newAnimation(frames, walk_animation_time / totalFrames)
        return animation
    else
        local imageName = "monster_"..body_id.."_remove_back_%d.png"
        local frames = display.newFrames(imageName, 1,totalFrames)
        local animation = display.newAnimation(frames, walk_animation_time / totalFrames)
        return animation
    end
end

function AnimationFactory.getBossStandAnimation(direction,body_id)--获取boss静止站立动画
    local totalFrames = 16
    if direction==true then
        local imageName = "monster_"..body_id.."_stand_front_%d.png"
        local frames = display.newFrames(imageName, 1,totalFrames)
        local animation = display.newAnimation(frames, 1 / totalFrames)
        return animation
    else
        local imageName = "monster_"..body_id.."_stand_back_%d.png"
        local frames = display.newFrames(imageName, 1,totalFrames)
        local animation = display.newAnimation(frames, 1 / totalFrames)
        return animation
    end
end

function AnimationFactory.getBossCallAnimation(direction,body_id)--获取boss静止站立动画
    local totalFrames = 16
    if direction==true then
        local imageName = "monster_"..body_id.."_call_front_%d.png"
        local frames = display.newFrames(imageName, 1,totalFrames)
        local animation = display.newAnimation(frames, 1 / totalFrames)
        return animation
    else
        local imageName = "monster_"..body_id.."_call_back_%d.png"
        local frames = display.newFrames(imageName, 1,totalFrames)
        local animation = display.newAnimation(frames, 1 / totalFrames)
        return animation
    end
end

function AnimationFactory.getMonsterLoadingAnimation(underSea,direction,body_id,walk_animation_time)
    --[[
    if body_id==70061 then
        local imageName = "monster_"..body_id.."_walk_%d.png"
        local frames = display.newFrames(imageName, 1,8)
        local animation = display.newAnimation(frames, walk_animation_time / 8)
        return animation
    end]]
    local totalFrames = 8
    if body_id==70053 then
        totalFrames = 16
    end
    if underSea then
        if direction == "back" then
            local imageName = "monster_"..body_id.."_swimming_back_%d.png"
            local frames = display.newFrames(imageName, 1,totalFrames)
            local animation = display.newAnimation(frames, walk_animation_time / totalFrames)
            return animation,frames
        elseif direction == "front" then
            local imageName = "monster_"..body_id.."_swimming_front_%d.png"
            local frames = display.newFrames(imageName, 1,totalFrames)
            local animation = display.newAnimation(frames, walk_animation_time / totalFrames)
            return animation,frames
        end
    else
        if direction == "back" then
            local imageName = "monster_"..body_id.."_walk_back_%d.png"
            local frames = display.newFrames(imageName, 1,totalFrames)
            local animation = display.newAnimation(frames, walk_animation_time / totalFrames)
            return animation,frames
        elseif direction == "front" then
            local imageName = "monster_"..body_id.."_walk_front_%d.png"
            local frames = display.newFrames(imageName, 1,totalFrames)
            local animation = display.newAnimation(frames, walk_animation_time / totalFrames)
            return animation,frames
        end
    end
end

function AnimationFactory.getTowerFriendEffectAnimation()
    local frames = display.newFrames("tower_friend_%d.png", 1,28)
    local animation = display.newAnimation(frames,2/28)
    return frames,animation
end

function AnimationFactory.getBuildAnimation()
    local frames = display.newFrames("build_%02d.png", 1,11)
    local animation = display.newAnimation(frames,2/11)
    return frames,animation
end

function AnimationFactory.getWorkerAnimation()
    local frames = display.newFrames("npc_1018_build_%d.png", 1, 8)
    local animation = display.newAnimation(frames, 0.5 / 8)
    return frames,animation
end

function AnimationFactory.getBehitPhysicalAnimation()
    local frames = display.newFrames("physical_%02d.png", 1,3)
    local animation = display.newAnimation(frames,0.3/3)
    return frames,animation,35
end

function AnimationFactory.getBehitMagicAnimation()
    local frames = display.newFrames("spell_%02d.png", 1,3)
    local animation = display.newAnimation(frames,1/3)
    return frames,animation,35
end

function AnimationFactory.getSkillReleaseAnimation()
    local frames = display.newFrames("release_%02d.png", 1,4)
    local animation = display.newAnimation(frames,0.3/4)
    return frames,animation
end

--取得塔攻击动画
--front:1,2
--back:3,4顺序为先front 后back 从左到右边
function AnimationFactory.getTowerAttackAnimation(towerType,towerLevel,frontOrBack,leftOrRight)
    --print("取得塔攻击动画",frontOrBack,leftOrRight)
    local imageName = string.format("%s_v%d_tower_%s_%s_",towerType,towerLevel,frontOrBack,leftOrRight)
    local frames = display.newFrames(imageName.."%d.png", 1,8)        
    local animation = display.newAnimation(frames, 0.7/8)
    return frames,animation
end

function AnimationFactory.getNpcAnimation(direction,body_id,walk_animation_time)
    if direction then
        local imageName = body_id.."_walk_front_%d.png"
        local frames = display.newFrames(imageName, 1,8)
        local animation = display.newAnimation(frames, walk_animation_time / 8)
        return animation
    else
        local imageName = body_id.."_walk_back_%d.png"
        local frames = display.newFrames(imageName, 1,8)
        local animation = display.newAnimation(frames, walk_animation_time / 8)
        return animation
    end
end

function AnimationFactory.getNpcStandAnimation(direction,body_id,walk_animation_time)
    if direction then
        local imageName = body_id.."_stand_front_%d.png"
        local frames = display.newFrames(imageName, 1,8)
        local animation = display.newAnimation(frames, walk_animation_time / 8)
        return animation
    else
        local imageName = body_id.."_stand_back_%d.png"
        local frames = display.newFrames(imageName, 1,8)
        local animation = display.newAnimation(frames, walk_animation_time / 8)
        return animation
    end
end

function AnimationFactory.getDeadAnimation(death_effect)

    --print("获取死亡效果",death_effect)

    local deadName = nil
    
    if death_effect == 0 then
        deadName = "blood"
    elseif death_effect == 1 then
        deadName = "blood"
    elseif death_effect == 2 then
        deadName = "demon"
    elseif death_effect == 3 then
        deadName = "dust"
    elseif death_effect == 4 then
        deadName = "mound"
    elseif death_effect == 5 then
        deadName = "sewage"
    elseif death_effect == 6 then
        deadName = "smoke"
    else
    end
    
    local offset = 0
    if deadName == "blood" then
        local frames = display.newFrames(deadName.."_%02d.png", 1,3)
        local animation = display.newAnimation(frames,0.3/3)
        offset = 35
        return frames,animation,offset
    elseif deadName == "demon" then
        local frames = display.newFrames(deadName.."_%02d.png", 1,8)
        local animation = display.newAnimation(frames,0.3/8)
        offset = 35
        return frames,animation,offset
    elseif deadName == "dust" then
        local frames = display.newFrames(deadName.."_%02d.png", 1,10)
        local animation = display.newAnimation(frames,0.3/10)
        offset = 35
        return frames,animation,offset
    elseif deadName == "mound" then
        local frames = display.newFrames(deadName.."_%02d.png", 1,8)
        local animation = display.newAnimation(frames,0.3/8)
        offset = 35
        return frames,animation,offset
    elseif deadName == "sewage" then
        local frames = display.newFrames(deadName.."_%02d.png", 1,7)
        local animation = display.newAnimation(frames,0.3/7)
        offset = 35
        return frames,animation,offset
    elseif deadName == "smoke" then
        local frames = display.newFrames(deadName.."_%02d.png",1,4)
        local animation = display.newAnimation(frames,0.3/4)
        offset = 35
        return frames,animation,offset
    else
    end
end

function AnimationFactory.getBuffAnimation(effectName)
    if effectName=="armor" then
        local frames = display.newFrames("armor_%02d.png", 1,8)
        local animation = display.newAnimation(frames,1/8)
        return frames,animation,35
    elseif effectName=="bleed" then
        local frames = display.newFrames("bleed_%02d.png", 1,7)
        local animation = display.newAnimation(frames,0.3/7)
        return frames,animation,35
    elseif effectName=="flame" then
        local frames = display.newFrames("flame_%02d.png", 1,9)
        local animation = display.newAnimation(frames,0.4/9)
        return frames,animation,35
    elseif effectName=="resist" then
        local frames = display.newFrames("resist_%02d.png", 1,8)
        local animation = display.newAnimation(frames,0.4/8)
        return frames,animation,35
    elseif effectName=="slow" then
        local frames = display.newFrames("slow_%02d.png", 1,8)
        local animation = display.newAnimation(frames,1/8)
        return frames,animation,35
    elseif effectName=="vertigo" then
        local frames = display.newFrames("vertigo_%02d.png", 1,9)
        local animation = display.newAnimation(frames,0.4/9)
        return frames,animation,35
    elseif effectName=="winding" then 
        local frames = display.newFrames("winding_%02d.png", 1,8)
        local animation = display.newAnimation(frames,0.4/8)
        return frames,animation,35
    end
end

--通天塔给妖妓加血特效
function AnimationFactory.getAddBloodAnimation()
    local frames = display.newFrames("peach_%02d.png", 1,7)
    local animation = display.newAnimation(frames,1/7)
    return frames,animation
end

function AnimationFactory.getHeroSkillEffectAnimation(index)
    local frames = display.newFrames("hero_skill_effect_"..index.."_%02d.png", 1,8)
    local animation = display.newAnimation(frames,0.3/8)
    return frames,animation
end

function AnimationFactory.getHeroStartAnimation()
    local frames = display.newFrames("start%d.png", 1,3)
    local animation = display.newAnimation(frames,0.3/3)
    return frames,animation
end


function AnimationFactory.getMonsterAppearAnimation()
    local frames = display.newFrames("home_monster_appear_%d.png", 1,12)
    local animation = display.newAnimation(frames,1/16)
    return frames,animation
end

function AnimationFactory.getMonsterDeadAnimation()
    local frames = display.newFrames("dead_%d.png", 1,3)
    local animation = display.newAnimation(frames,0.3/3)
    return frames,animation
end

function AnimationFactory.getOneToTwoAnimation()
    local frames = display.newFrames("%d.png", 1,9)
    local animation = display.newAnimation(frames,0.2/9)
    return frames,animation
end

function AnimationFactory.getFingerAniamtion()
    --[[
    local frames = display.newFrames("quanquan_%d.png", 1,14)
    local animation = display.newAnimation(frames,1/14)
    return frames,animation]]
    local frames = display.newFrames("PendingAperture_%d.png", 1,9)
    local animation = display.newAnimation(frames,0.5/9)
    return frames,animation
end

function AnimationFactory.getCircleAniamtion()
    local frames = display.newFrames("PendingAperture_%d.png", 1,9)
    local animation = display.newAnimation(frames,1.5/9)
    return frames,animation
end

function AnimationFactory.getSkillSelectAniamtion()
    local frames = display.newFrames("Skill_Box_Selected_Ani_%d.png", 1,9)
    local animation = display.newAnimation(frames,1.5/9)
    return frames,animation
end

function AnimationFactory.getTowerFlagAniamtion()
    local frames = display.newFrames("taji_call_%d.png", 1,8)
    local animation = display.newAnimation(frames,1/8)
    return frames,animation
end

function AnimationFactory.getMageExplodeAniamtion()
    local frames = display.newFrames("mage_explode_%d.png", 1,4)
    local animation = display.newAnimation(frames,0.4/4)
    return frames,animation
end

function AnimationFactory.getWarriorExplodeAniamtion()
    local frames = display.newFrames("warrior_explode_%d.png", 1,3)
    local animation = display.newAnimation(frames,0.3/3)
    return frames,animation
end



function AnimationFactory.getDogShoutAniamtion(frontOrBack)
    local frames = display.newFrames("npc_1022_call_"..frontOrBack.."_%d.png", 1,8)
    local animation = display.newAnimation(frames,1/8)
    return frames,animation
end

function AnimationFactory.getDogSitAniamtion(frontOrBack)
    local frames = display.newFrames("npc_1022_sitting_"..frontOrBack.."_%d.png", 1,8)
    local animation = display.newAnimation(frames,1/8)
    return frames,animation
end

function AnimationFactory.getDogStandAniamtion(frontOrBack)
    local frames = display.newFrames("npc_1022_stand_"..frontOrBack.."_%d.png", 1,8)
    local animation = display.newAnimation(frames,0.5/8)
    return frames,animation
end

function AnimationFactory.getDeleteTowerAnimation()
    local frames = display.newFrames("build_%02d.png", 1,11)
    local animation = display.newAnimation(frames,1/11)
    return frames,animation
end

function AnimationFactory.getFrozenTowerAnimation()
    local frames = display.newFrames("seal_%02d.png", 1,10)
    local animation = display.newAnimation(frames,1/10)
    return frames,animation
end

function AnimationFactory.getWaterFlowAnimation()
    local frames = display.newFrames("water_flow_%02d.png", 1,11)
    local animation = display.newAnimation(frames,1/10)
    return frames,animation
end

function AnimationFactory.getWaterExpAnimation()
    local frames = display.newFrames("water_exp_%02d.png", 1,7)
    local animation = display.newAnimation(frames,1/7)
    return frames,animation
end

function AnimationFactory.getBirdFlyAnimation()
    local frames = display.newFrames("crane_%d.png", 1,8)
    local animation = display.newAnimation(frames,0.5/8)
    return frames,animation
end

function AnimationFactory.getGoldAnimation()
    local frames = display.newFrames("gold_%d.png", 1,16)
    local animation = display.newAnimation(frames,1/16)
    return frames,animation
end

function AnimationFactory.getFocusAnimation()
    local frames = display.newFrames("aim_%d.png", 1,8)
    local animation = display.newAnimation(frames,0.5/8)
    return frames,animation
end

function AnimationFactory.getNpcRunAnimation(direction,body_id,walk_animation_time)
    if direction then
        local imageName = body_id.."_run_front_%d.png"
        local frames = display.newFrames(imageName, 1,8)
        local animation = display.newAnimation(frames, walk_animation_time / 8)
        return animation
    else
        local imageName = body_id.."_run_back_%d.png"
        local frames = display.newFrames(imageName, 1,8)
        local animation = display.newAnimation(frames, walk_animation_time / 8)
        return animation
    end
end


return AnimationFactory
