local DebugUtil = require("app.common.DebugUtil")
local CommUtil = require("app.common.CommUtil")
local Rect = require("app.quadTree.Rect")


--local QuadTree =class ("QuadTree")
--QuadTree.__index = QuadTree

local QuadTree = class("QuadTree",function()
    local node = display.newNode()
    return node
end)

function QuadTree:ctor()
	--self.gameScene = gameScene
    --print("四叉树初始化开始")
    local rectWidth = display.width/gridColumn
    local rectHeight = display.height/gridRow

    self.rects = {}
    
    for i=1,gridRow do
        for j=1,gridColumn do
            --print("i_j",i,j,rectWidth,rectHeight)
            local index = (i-1)*gridColumn + j
            if gridDebug then
                local rect = display.newRect(rectWidth,rectHeight)
                rect:pos((j-1)*rectWidth+rectWidth/2,(i-1)*rectHeight+rectHeight/2)
                rect:addTo(self)
                --[[
                local label = ui.newTTFLabel({
                    text = ""..i.."_"..j.."_"..index,
                    --text = index,
                    font = "Arial",
                    size = 16,
                    color = ccc3(255, 0, 0),
                })
                :pos((j-1)*rectWidth+rectWidth/2,(i-1)*rectHeight+rectHeight/2)
                :addTo(self)]]
            end
            self.rects[#self.rects+1] = CCRect((j-1)*rectWidth,(i-1)*rectHeight,rectWidth,rectHeight)
        end
    end
    self.grids = {}
    for i=1,#self.rects do
        self.grids[#self.grids+1] = {}
    end

    print("grid_number_is_",#self.grids)

    for i=0,GameInstance:sharedInstance().towers:count()-1 do
        local tower = GameInstance:sharedInstance().towers:objectAtIndex(i)
        if tower:ifBuild() then
            tower.gridIndexs = {}
            local towerRect,offset,centerPoint,size = tower:getRectAndOffset()
            if gridDebug then display.newRect(size.width-offset*2,size.height-offset*2):pos(centerPoint.x,centerPoint.y):addTo(self) end
            for i=1,#self.rects do
                local rect = self.rects[i]
                if rect:intersectsRect(towerRect) then
                    tower.gridIndexs[#tower.gridIndexs+1] = i
                end
            end
        end
    end
end

function QuadTree:updateTowerRect(tower)
        if not tower:ifBuild() then
            return
        end
        tower.gridIndexs = {}
        local towerRect,offset,centerPoint,size = tower:getRectAndOffset()
        if gridDebug then display.newRect(size.width-offset*2,size.height-offset*2):pos(centerPoint.x,centerPoint.y):addTo(self) end
        for i=1,#self.rects do
            local rect = self.rects[i]
            if rect:intersectsRect(towerRect) then
                tower.gridIndexs[#tower.gridIndexs+1] = i
            end
        end
end

--[[
    4象限如此划分 逆时针
    1 0 
    2 3
]]
function QuadTree:split()
    local subWidth = self.bounds.width/2
    local subHeight = self.bounds.height/2
    local x = self.bounds.x
    local y = self.bounds.y
    self.nodes[1] = require("app.quadTree.QuadTree").new(self.level+1,Rect.new(x+subWidth,y+subHeight,subWidth,subHeight),self,0,self.gameScene):addTo(self.gameScene)
    self.nodes[2] = require("app.quadTree.QuadTree").new(self.level+1,Rect.new(x,y+subHeight,subWidth,subHeight),self,1,self.gameScene):addTo(self.gameScene)
    self.nodes[3] = require("app.quadTree.QuadTree").new(self.level+1,Rect.new(x,y,subWidth,subHeight),self,2,self.gameScene):addTo(self.gameScene)
    self.nodes[4] = require("app.quadTree.QuadTree").new(self.level+1,Rect.new(x+subWidth,y,subWidth,subHeight),self,3,self.gameScene):addTo(self.gameScene)
end

function QuadTree:getIndex(rect)
    rect.topQuadrant = rect.y > self.yMidPoint
    rect.bottomQuadrant = (rect.y+ rect.height) < self.yMidPoint
    rect.leftQuadrant = self.xMidPoint and rect.x + rect.width < self.xMidPoint
    rect.rightQuadrant = rect.x > self.xMidPoint
    rect.centerQuadrant = not topQuadrant and (not bottomQuadrant) and (not rightQuadrant) and (not leftQuadrant)
            
    local index = -1
    if rect.leftQuadrant then
        if rect.topQuadrant then
            index = 1
        elseif rect.bottomQuadrant then
            index = 2
        end
    elseif rect.rightQuadrant then
        if rect.topQuadrant then
            index = 0
        elseif rect.bottomQuadrant then
            index = 3
        end
    end
    return index--等于-1 是矩形刚好 移动到 中心线的边上 不在四块子区域内部
end

function QuadTree:insert(rect)
    if self.nodes[1]~=nil then
        local index = self:getIndex(rect)
        if index~=-1 then
            self.nodes[index+1]:insert(rect)
            return
        end
    end
    self.objects[#self.objects+1] = rect
    rect.parent = self

    if #self.objects > self.MAX_OBJECTS and self.level < self.MAX_LEVELS then
        if self.nodes[1] == nil then self:split() end
        local i = 1
        while i<#self.objects+1 do
            index = self:getIndex(self.objects[i])
            if index~=-1 then
                --self.nodes[index].insert(objects.splice(i, 1)[0])
                local object = self.objects[i]
                table.remove(self.objects,i)
                self.nodes[index+1]:insert(object)
            else
                i = i + 1
            end
        end
    end
end


function QuadTree:retrive(rect)
    --[[]]
    local returnObjects = {}
    local q = rect.parent
    while q do
        table.merge(returnObjects,q.objects)
        q = q.parent
    end
    return returnObjects
    --[[
    local returnObjects = {}
    table.merge(returnObjects,self.objects)
    local index = self:getIndex(rect)
    if (index ~= -1 and self.nodes[1] ~= nil) then
        --print("index是",index)
        local innerObjects = self.nodes[index+1]:retrive(rect)
        print("innerObjects",#innerObjects)
        table.merge(returnObjects,innerObjects)
    end
    return returnObjects]]
end
--[[
    首先 getIndex会返回节点的位置 0,1,2,3 用二进制则是 00 01 10 11
    从根节点开始 其次 根节点的子节点 一直到 rect所处的节点 也就是getIndex返回-1的时候
    每次返回的结果都<<2 加上上次保存的 那么就可以得到它所在的节点
    比如 10, 00 , 11 保存后则是 100011 如果上次的节点位置 和此次计算不同 则说明需要重新插入节点]]
function QuadTree:calIndex(rect)
    local index = self:getIndex(rect)
    if index~=-1 and self.nodes[1]~=nil then
        rect.level = rect.level + 1
        rect.nowIndex =  GameInstance:sharedInstance():moveLeft(rect.nowIndex,2) + index
        --print("索引是",index+1)
        self.nodes[index+1]:calIndex(rect)
        return
    end
    if rect.nowIndex~=rect.lastIndex then
        self:modifyIndex(rect)
    end
    rect.lastIndex = rect.nowIndex
end

function QuadTree:modifyIndex(rect)
    local ifExists = function()
        for key, value in ipairs(rect.parent.objects) do
            if value==rect then
                return true,key
            end            
        end
        return false,-1
    end

    local ifExistsInParentObjects,key = ifExists()
    if ifExistsInParentObjects then
        table.remove(rect.parent.objects,key)
        self:insert(rect)
    end
end

function QuadTree:newInsert(rect)
    if self.nodes[1] ~=nil then
        local index = GameInstance:sharedInstance():bitAnd(GameInstance:sharedInstance():moveRight(rect.nowIndex,(rect.level-1)*2),3)
        rect.level = rect.level - 1
        if rect.level > -1 then
            self.nodes[index+1].newInsert(rect)
            return
        end
    end   
    self.objects[#self.objects+1] = rect
    rect.parent = self

    if #self.objects > self.MAX_OBJECTS and self.level < self.MAX_LEVELS then
        if self.nodes[1] == nil then
            self:split()
        end
        local i = 1
        while i< #self.objects do
            index = self:getIndex(self.objects[i])
            if index~=-1 then
                --nodes[index].insert(objects.splice(i, 1)[0])
                local object = self.objects[i]
                table.remove(self.objects,i)
                self.nodes[index+1]:insert(object)
            else
                i = i + 1
            end
        end
    end
end

function QuadTree:clear()
    --print("四叉树清理")
    --[[
    for i=1,#self.nodes do
        local node = self.nodes[i]
        node:removeSelf()
    end
    self.objects = {}
    self.nodes = {}]]
    self.grids = {}
    for i=1,#self.rects do
        self.grids[#self.grids+1] = {}
    end
end

function QuadTree:removeMonsterFromGrid(monster,gridIndex)
    local monsters = self.grids[gridIndex]
    if monsters==nil or #monsters==0 then return end
    for k, v in ipairs(monsters) do
        if v==monster then
            table.remove(monsters,k)
            return
        end
    end
end

function QuadTree:addMonesterInGrid(monster,gridIndex)
    self.grids[gridIndex][#self.grids[gridIndex]+1] = monster
end

return QuadTree