local scheduler = require("framework.scheduler")
local socketTip = require("app.client.page_ini.socketTip")
local ThirdWindow = import("app.ui.WindowThird")

local HttpNode = class("HttpNode", function()
    local node = display.newNode()
    node:setTouchEnabled(true)
    node:addTouchEventListener(function(event, x, y)
        return true
    end)
    node:registerScriptHandler(function(event)
        if event == "enter" then
            node:onEnter()
        elseif event == "exit" then
            node:onExit()
        end
    end)
    
    return node
end)

--[[
args
    --url链接地址
    --mode连接方式,GET/POST
    --callback(ret, body)返回处理ret = 0(正常)/-1(通信异常)
    --freeze是否冻结页面
    ----------------------以下为可选参数
    --enable 数据是否加密且request/response同时采用,true为加密,仅在post模式下使用
    --compress 暂时未使用
    --req_header请求头
    --cookie 设置cookie
    --post_value = {key = , value = } 仅在post模式下使用;
    --post_data  尽在post模式下使用
]]
function HttpNode:ctor(args)
    --CommUtil.guideHide()

    self.config = totable(args)

    self.reConnTime = 0

    self:requestHttp()
end

function HttpNode:onRequestFinished(event)
    --local starttime = os.time()
    
    local ok = (event.name == "completed")
    local request_ = event.request

    if not ok then
        -- 请求失败，显示错误代码和错误消息
        print("HttpNode:", "request state = " .. event.name)
        print("HttpNode:", "request error code = " .. request_:getErrorCode())
        print("HttpNode:", "request error message = " .. request_:getErrorMessage())
        if (self.config) then
            self.config.callback(-1)
        end
        return
    end

    local function checkResponse(response)
        local response_ = json.decode(response)
        return response_.ret
    end

    if request_:getResponseStatusCode() == 200 then
        --if checkResponse(request_:getResponseString()) == '0' then
            local responseStr = request_:getResponseString()
            if self.config.enable then
                responseStr = self:decryptData(responseStr)
            end
            self.config.callback(0, json.decode(responseStr))
        --else
        --    print("HttpNode: server except")
        --end

        self:exitNode()
    elseif request_:getResponseStatusCode() == 408 then   --请求超时
        self:expire()

        return
    else
        print("HttpNode:communication error, status code = " .. request_:getResponseStatusCode())
        
        CommUtil.toast(display.getRunningScene(), '出现异常')

        self.config.callback(-1)

        self:exitNode()
    end
    
    --print("HttpNode:", "协议处理耗时 = " .. (os.time() - starttime))
end

function HttpNode:requestHttp()
    self.reConnTime = self.reConnTime + 1


    local func = function(result)
        --没有网络支持
        if (result == "-1") then
            self:noNetwork()
            return
        end

        if (not self.config.mode) or (self.config.mode ~= 'GET' and self.config.mode ~= 'POST') then
            print("HttpNode:link mode error = " .. self.config.mode)
            return
        end

        if not self.config.callback then
            print("HttpNode:callback is nil")
            return   
        end

        --通信,默认超时时间10s
        local request = network.createHTTPRequest(handler(self, self.onRequestFinished), self.config.url, self.config.mode)
        --request:setTimeout(30)

        if self.config.req_header then
            request:addRequestHeader(self.config.req_header)
        end

        if self.config.cookie then
            request:setCookieString(self.config.cookie)
        end

        if self.config.post_value and self.config.mode == 'POST' then
            if self.config.enable then
                self.config.post_value.value = self:encryptData(self.config.post_value.value)
            end
            request:addPOSTValue(self.config.post_value.key, self.config.post_value.value)
        end

        if self.config.post_data and self.config.mode == 'POST' then
            if self.config.enable then
                self.config.post_data = self:encryptData(self.config.post_data)
            end
            request:setPOSTData(self.config.post_data)
        end

        request:start()
        
        --冻结页面
        if (self.config.freeze == true and self.reConnTime == 1) then
            self:freeze()
        end
    end
    
    DeviceUtil.getNetWorkStatus(func)
end

function HttpNode:freeze()
    CommUtil.addBlankLayer(self)
    CommUtil.addFreeze(self)
end

function HttpNode:noNetwork()
    ThirdWindow.new({
        title = "提示",
        content = socketTip.tips["2"],
        leftBtn = {
            normal = "res/ui/public/public_definite_normal.png",
            pressed = "res/ui/public/public_definite_pressed.png"},
        onConfirm = function()   --退出
            CommUtil.reLoginGame()
        end
    }):addTo(self)
end

function HttpNode:expire()
    ThirdWindow.new({
        title = '提示',
        content = socketTip.tips["3"],
        leftBtn = {
            normal = "res/ui/public/public_to_withdraw_btn_normal.png",
            pressed = "res/ui/public/public_to_withdraw_btn_pressed.png"},
        rightBtn = {
            normal = "res/ui/public/public_reconnection_btn_normal.png",
            pressed = "res/ui/public/public_reconnection_btn_pressed.png"},
        onConfirm = function()   --退出
            CommUtil.reLoginGame()
        end,
        onCancel = function()    --重连
            self:requestHttp()
        end
    }):addTo(self)
end

function HttpNode:decryptData(data)
    local decryptStr = crypto.decodeBase64(data)
    return crypto.decryptXXTEA(decryptStr, MOBI_PRIVATE_KEY)
end

function HttpNode:encryptData(data)
    local encryptStr = crypto.encryptXXTEA(data, MOBI_PRIVATE_KEY)
    return crypto.encodeBase64(encryptStr)
end

function HttpNode:exitNode()
    local closeFunc = function()
        if not tolua.isnull(self) then
            self:removeFromParentAndCleanup(true)
        end
    end

    scheduler.performWithDelayGlobal(closeFunc, 0.2)
end

function HttpNode:onExit()
    -- body
end

return HttpNode