local DragView = class("DragView",function(args)
	local node = display.newNode()
	node:setNodeEventEnabled(true)
	require("framework.api.EventProtocol").extend(node)
	return node
end)
local DragViewItem = import(".DragViewItem")
local Text = import(".Text")

DragView.CLICKED_EVENT = "CLICKED_EVENT"
DragView.SWAPED_EVENT = "SWAPED_EVENT"
function DragView:ctor( args )
--[[
args:
	cell     number,  每行或每列显示的item个数 vertical为false时为每行个数，反之为每列个数
	nums     table,   item元素个数
	size 	 table,   显示大小 { height = 400, width = 500}
	pos      table,   显示点坐标 { x = 200, y = 100 }
	interval table,   横纵item间隔 { x = 10, y = 10 }
	vertical boolean, 显示方式，默认为false
]]
	assert(type(args.cell) == "number")
	assert(type(args.nums) == "number")
	assert(type(args.size) == "table")
	assert(type(args.pos) == "table")
	assert(type(args.interval) == "table")
	assert(type(args.vertical) == "boolean" or args.vertical == nil)

	self.DRAG_ITEM_ZORDER = 5 --把拖动ITEM的层级设高
	self.DEFAULT_ITEM_ZORDER = 1
	self.SWAP_DONE = true --指示拖动动画完成

	self.vertical_ = (args.vertical == nil) and false or args.vertical
	self.cell_ = args.cell
	self.nums_ = args.nums
	self.content_size_ = args.size
	self.interval_ = args.interval
	self.pos_ = args.pos
	self.rows_, self.cols_ = self:getRowCols()
	self.cell_size_ = {
			width = math.floor((self.content_size_.width - self.interval_.x*(self.cols_-1)) / self.cols_),
			height = math.floor((self.content_size_.height - self.interval_.y*(self.rows_-1)) / self.rows_)
		}

	self.layer = display.newLayer():addTo(self)
	self.layer:setTouchEnabled(true)
	self.layer:registerScriptTouchHandler(handler(self,self.onTouch))

	self.drag_index_ = 0
	self.offset_ = { x = 0, y = 0 }
	self.startP_ = { x = 0, y = 0 }
end

function DragView:getRowCols()
--[[ 计算并返回排列items所需的行列数
]]
	local rows, cols
	if self.vertical_ then
		rows = self.cell_
		cols = math.ceil(self.nums_/rows)
	else
		cols = self.cell_
		rows = math.ceil(self.nums_/cols)
	end
	return rows, cols
end

function DragView:alignItems( nodes )
--[[ 排列items
]]
	local rect = CCRect(self.pos_.x, self.pos_.y, self.content_size_.width, self.content_size_.height)
	self.clipnode_ = display.newClippingRegionNode(rect):addTo(self)
	self.drag_node_ = display.newNode():addTo(self.clipnode_)

	local pos_x = self.pos_.x
	local pos_y = self.pos_.y + self.content_size_.height - self.cell_size_.height

	if self.vertical_ then -- 垂直排列
		for index, node in ipairs(nodes) do
			if index > 1 and index%self.rows_ == 1 then
				pos_x = pos_x + self.cell_size_.width + self.interval_.x
				pos_y = self.pos_.y + self.content_size_.height - self.cell_size_.height
			end
			local item =  DragViewItem.new({
					index = index,
					node  = node,
					size  = self.cell_size_,
				}):addTo(self.drag_node_, self.DEFAULT_ITEM_ZORDER):pos(pos_x, pos_y)
			local text = Text.new({
				text         = tostring(index),
				x            = pos_x + self.cell_size_.width/2,
				y            = pos_y + self.cell_size_.height/2,
				size         = 70,
				align        = ui.TEXT_ALIGN_CENTER,
				color        = ccc3(250, 250, 250),
				shadowColor  = ccc3(0,0, 0)
				}):addTo(self.drag_node_, -1)
			if self.items_ == nil then self.items_ = {} end
			table.insert(self.items_,item)
			pos_y = pos_y - self.cell_size_.height - self.interval_.y
		end
	else -- 水平排列
		for index, node in ipairs(nodes) do
			if index > 1 and index%self.cols_ == 1 then
				pos_x = self.pos_.x
				pos_y = pos_y - self.cell_size_.height - self.interval_.y
			end
			local item =  DragViewItem.new({
					index = index,
					node  = node,
					size  = self.cell_size_,
				}):addTo(self.drag_node_, self.DEFAULT_ITEM_ZORDER):pos(pos_x, pos_y)
			local text = Text.new({
				text         = tostring(index),
				x            = pos_x + self.cell_size_.width/2,
				y            = pos_y + self.cell_size_.height/2,
				size         = 70,
				align        = ui.TEXT_ALIGN_CENTER,
				color        = ccc3(250, 250, 250),
				shadowColor  = ccc3(0,0, 0)
				}):addTo(self.drag_node_, -1)
			if self.items_ == nil then self.items_ = {} end
			table.insert(self.items_,item)
			pos_x = pos_x + self.cell_size_.width + self.interval_.x
		end
	end
end

function DragView:onTouch( event, x, y )
	if self.SWAP_DONE ~= true then return end
	if event == "began" then
		return self:onTouchBegan(x, y)
	elseif event == "moved" then
		return self:onTouchMoved(x, y)
	elseif event == "ended" then
		return self:onTouchEnded(x, y)
	end
end

function DragView:onTouchBegan( x, y )
--[[ 触摸开始时 item弹起 边框白色虚线
]]
	if self:isOutSide(x,y) then return end
	self.startP_.x = x
	self.startP_.y = y
	for i,v in ipairs(self.items_) do
		if self:containsPoint(i,x,y) then
			-- 正在移动第i个item
			self.drag_index_ = i
			local pos_x, pos_y = self:calPosition(i)
			self.offset_.x = x - pos_x
			self.offset_.y = y - pos_y
			self.drag_node_:reorderChild(self.items_[i], self.DRAG_ITEM_ZORDER)
			self.items_[i]:select()
			return true
		end
	end
end

function DragView:onTouchMoved( x, y )
--[[ 移动时item跟随
TODO: 改造交换方式 移动时交换
]]
	if self:isOutSide(x,y) then return end
	self.items_[self.drag_index_]:setPosition(x - self.offset_.x, y - self.offset_.y)
end

function DragView:onTouchEnded( x, y )
--[[ 触摸结束时判断坐标 再进行交换
]]
	if math.abs(x-self.startP_.x)<5 and math.abs(y-self.startP_.y)<5 then
		-- 点击事件
		self.drag_node_:reorderChild(self.items_[self.drag_index_], self.DEFAULT_ITEM_ZORDER)
		self:restoreItem()
		self:dispatchEvent({name = DragView.CLICKED_EVENT, index = self.drag_index_, x = x, y = y})
		return
	end
	if self:containsPoint(self.drag_index_,x,y) then
		-- 没移动出自身范围
		self:restoreItem()
		return
	end
	for i,v in ipairs(self.items_) do
		if self:containsPoint(i,x,y) then
			-- 移动到位置 i
			self:swapItem(self.drag_index_, i)
			self.drag_node_:reorderChild(self.items_[i], self.DEFAULT_ITEM_ZORDER)
			self.items_[self.drag_index_]:unSelect()
			return
		end
	end
	-- 超出范围 还原该item
	self:restoreItem()
end

function DragView:restoreItem()
--[[ 将拖动的item还原
]]
	local src_x, src_y = self:calPosition(self.drag_index_)
	transition.execute(self.items_[self.drag_index_], CCMoveTo:create(0.2, CCPoint(src_x, src_y)), {
		easing = "backout"
	})
	self.items_[self.drag_index_]:unSelect()
end

function DragView:isOutSide( x, y )
--[[ 判断点是否在显示范围外
]]
	local x_, y_ = self.pos_.x, self.pos_.y
	if x < x_ or x > x_ + self.content_size_.width or
		y < y_ or y > y_ + self.content_size_.height then
		return true
	end
	return false
end

function DragView:containsPoint( index, x, y )
--[[ 判断坐标点是否在item显示范围内
]]
	local src_x, src_y = self:calPosition(index)
	if (x >= src_x and x < src_x+self.cell_size_.width) and
		(y >= src_y and y < src_y+self.cell_size_.height) then
		return true
	else
		return false
	end	
end

function DragView:calPosition( index )
--[[ 根据索引计算坐标
]]
	local row_n, col_n
	local x, y
	if self.vertical_ then
		col_n = math.floor((index-1)/self.rows_) + 1
		row_n = index - (col_n-1) * self.rows_
	else
		row_n = math.floor((index-1)/self.cols_) + 1
		col_n = index - (row_n-1) * self.cols_
	end
	x = self.pos_.x + (col_n-1) * self.cell_size_.width + self.interval_.x*(col_n-1)
	y = self.pos_.y + self.content_size_.height - row_n * self.cell_size_.height - self.interval_.y*(row_n-1)
	return x, y
end

function DragView:swapItem( src, dest )
--[[ 交换item
]]
	self.SWAP_DONE = false
	local src_x, src_y = self:calPosition(src)
	local dest_x, dest_y = self:calPosition(dest)
	
	self.items_[src]:setPosition(dest_x, dest_y)
	transition.execute(self.items_[dest], CCMoveTo:create(0.2, CCPoint(src_x, src_y)), {
		easing = "backout",
		onComplete = function()
			local tmp = self.items_[dest]
			local idx = self.items_[dest].index_
			self.items_[dest] = self.items_[src]
			self.items_[dest].index_ = self.items_[src].index_
			self.items_[src] = tmp
			self.items_[src].index_ = idx
			self.SWAP_DONE = true
			self:dispatchEvent({name = DragView.SWAPED_EVENT, src = src, dest = dest})
		end,
	})

end

function DragView:showFrame( true_or_false )
--[[ 设置边框显示隐藏
]]
	for _, item in ipairs(self.items_) do
		item:setFrameVisible(true_or_false)
	end
end

function DragView:setClippingRegion( scale )
--[[ 父节点缩放后需调用该方法 否则会引发裁切问题
]]
	local rect = self.clipnode_:getClippingRegion()
	local new_rect = CCRect(rect.origin.x, rect.origin.y, rect.size.width*scale, rect.size.height*scale)
	self.clipnode_:setClippingRegion(new_rect)
end

function DragView:setTouchEnabled( true_or_false )
--[[ 设置是否相应触摸
]]
	self.layer:setTouchEnabled(true_or_false)
end
return DragView