local DragViewItem = class("DragViewItem",function()
	return display.newNode()
end)

--[[
index: 保存当前索引
node: 用于扩展item中显示的对象
size: item显示区域大小 非item的大小
	  若size小于真实大小 则会根据宽高比缩放自动缩放 
	  默认值为node的大小

DragViewItem.new({
	index = 1,
	node  = display.newNode(),
	size  = { width = 10, height = 10 },
})
]]

function DragViewItem:ctor( args )
	assert(type(args) == "table", "The type of param #1 args should be a table")
	assert(type(args.node) == "userdata", "The type of args.node should be a userdata")
	assert(type(args.size) == "table", "The type of args.size should be a table")

	self.index_ = args.index
	local real_size = args.node:getCascadeBoundingBox().size
	local scale = self:getScale(args.size, real_size)
	self.node_ = args.node
	self.node_:addTo(self)
	self.node_:setScale(scale)
	self.node_:setAnchorPoint(CCPoint(0, 0))

	self.frame_ = display.newRect(args.size.width, args.size.height):addTo(self)
		:pos(args.size.width/2, args.size.height/2)
	self.frame_:setLineColor(ccc4f(0.0, 1.0, 0.0, 1.0))
	self.frame_:setLineWidth(2)
	self.frame_:setVisible(false)
end

function DragViewItem:getScale( size, real_size )
--[[ 计算使node能完全显示的scale值
]]
	local width_scale  = size.width / real_size.width
	local height_scale = size.height / real_size.height
	if (size.height / size.width) >= (real_size.height / real_size.width) then
		return width_scale
	else
		return height_scale
	end
end

function DragViewItem:select()
--[[ 选中时
]]
	self:setScale(1.05)
	self.frame_:setLineColor(ccc4f(1.0, 0.0, 0.0, 1.0))
	self.frame_:setLineWidth(4)
end

function DragViewItem:unSelect()
--[[ 取消选中时
]]
	self:setScale(1)
	self.frame_:setLineColor(ccc4f(0.0, 1.0, 0.0, 1.0))
	self.frame_:setLineWidth(2)
end

function DragViewItem:setFrameVisible( is_visible )
	self.frame_:setVisible(is_visible)
end

return DragViewItem