local ProgressBar = class("ProgressBar", function(args)
	local node = display.newNode()
	return node
end)
--[[
	length 	: 自定义长度
	act 	: 动画模式
	increase: 动画模式下 进度条增长方式
	velocity: 单位长度动画执行时间 默认值1

	ProgressBar.new({length=700,act=true,increase=true,velocity=2})
]]
function ProgressBar:ctor(args)
	local params = totable(args)
	assert(type(params.length)=="number" and params.length>0,"param length error")
	self.act = params.act or false
	self.increase = params.increase or false
	self.velocity = params.velocity and params.velocity/2000 or 1/2000

	--UI图片自身的长度
	local len = 44
	local con_len = 31
	local bg_len = 296
	local prgs_len = 260
	--length最小值为70
	if params.length < 2*len then
		self.length = 70
	else
		self.length = params.length
	end

	local bg_scaleX = (self.length-16) / bg_len
	local bg = display.newSprite("ui/public/public_progressbar_grounding.png"):pos(10,8):addTo(self,1)
	bg:setScaleX(bg_scaleX)
	bg:setAnchorPoint(CCPoint(0,0))

	self.prgs = display.newSprite("ui/public/public_progressbar_graduation.png"):pos(9,8):addTo(self,2)
	self.prgs:setAnchorPoint(CCPoint(0,0))
	self.prgs:setScaleX(0)

	self.prgs_effect = display.newSprite("ui/public/public_progressbar_effect.png"):pos(-10,-3):addTo(self,4)
	self.prgs_effect:setAnchorPoint(CCPoint(0,0))

	local frame_l = display.newSprite("ui/public/public_progressbar_frame.png"):pos(0,0):addTo(self,3)
	frame_l:setAnchorPoint(CCPoint(0,0))
	local frame_r = display.newSprite("ui/public/public_progressbar_frame.png"):pos(self.length-len,0):addTo(self,3)
	frame_r:setAnchorPoint(CCPoint(0,0))
	frame_r:setFlipX(true)
	local m_scaleX = (self.length - 2*len) / con_len
	local frame_m = display.newSprite("ui/public/public_progressbar_connect.png"):pos(len,5):addTo(self,3)
	frame_m:setAnchorPoint(CCPoint(0,0))
	frame_m:setScaleX(m_scaleX)

	self.prgs_scaleX = (self.length-18) / prgs_len
	self.percentage = 0
end

function ProgressBar:setPercentage(percentage)
	assert(type(percentage)=="number","percentage must be number")
	assert(percentage>=0 and percentage<=100,"percentage must between 0 and 100")
	local prgs_scale = self.prgs_scaleX * percentage / 100

	--计算进度点x坐标
	local effect_start_x = -10
	local effect_end_x = effect_start_x+self.length-25
	local effect_len = effect_end_x - effect_start_x
	local effect_pos_x = effect_len * percentage / 100 + effect_start_x

	if self.act then
		if self.increase and (self.percentage > percentage) then
			--满进度后再重新增长
			local interval1 = (effect_end_x - self.prgs_effect:getPositionX()) * self.velocity
			local interval2 = (effect_pos_x - effect_start_x) * self.velocity
			transition.execute(self.prgs_effect, CCMoveTo:create(interval1, CCPoint(effect_end_x, 0)), {
				onComplete = function()
					self.prgs_effect:setPositionX(-10)
					transition.moveTo(self.prgs_effect, {x = effect_pos_x, time = interval2})
				end,
			})
			transition.execute(self.prgs, CCScaleTo:create(interval1, self.prgs_scaleX, 1), {
				onComplete = function()
					self.prgs:setScaleX(0)
					transition.scaleTo(self.prgs, {scaleX  = prgs_scale, time = interval2})
				end,
			})
		elseif self.percentage ~= percentage then
			--直接增减
			local interval = math.abs(effect_pos_x - self.prgs_effect:getPositionX()) * self.velocity
			transition.scaleTo(self.prgs, {scaleX = prgs_scale, time = interval})
			transition.moveTo(self.prgs_effect, {x = effect_pos_x, time = interval})
		end
	else
		--无动画效果
		self.prgs:setScaleX(prgs_scale)
		self.prgs_effect:setPositionX(effect_pos_x)
	end

	self.percentage = percentage
end

function ProgressBar:getPercentage()
	return self.percentage
end

function ProgressBar:setAction(true_or_flase)
	self.act = true_or_flase
end

function ProgressBar:setIncrease(true_or_flase)
	self.increase = true_or_flase
end

function ProgressBar:setVelocity(time_per_unit_length)
	self.velocity = time_per_unit_length / 2000
end

return ProgressBar