
local ScrollView = class("ScrollView", function(rect)
    local abcrect
    if not rect then 
        rect = CCRect(0, 0, 0, 0) 
   
    end
    
    local node = display.newClippingRegionNode(rect)
    node:setNodeEventEnabled(true)
    node:registerScriptHandler(function(event)
        if event == "enter" then
            node:onEnter()
        elseif event == "exit" then
            node:onExit()
        end
    end)
    require("framework.api.EventProtocol").extend(node)
    return node
end)

ScrollView.DIRECTION_VERTICAL   = 1
ScrollView.DIRECTION_HORIZONTAL = 2
SCROLL_DRAG_TAP = false
SCROLL_CAN_TOUCH = true


function ScrollView:ctor(rect, direction)

    assert(direction == ScrollView.DIRECTION_VERTICAL or direction == ScrollView.DIRECTION_HORIZONTAL,
           "ScrollView:ctor() - invalid direction")

    self.dragThreshold = 10                 --拖动在一起范围内不算点击事件
    self.bouncThreshold = 140
    self.defaultAnimateTime = 0.5
    self.defaultAnimateEasing = "ELASTIC"

    self.direction = direction
    self.touchRect = rect
    self.offsetX = 0
    self.offsetY = 0
    self.cells = {}
    self.currentIndex = 0
    --触摸
    self.touchStartX = 0
    self.touchReleaseX = 0
    self.touchMoveX = 0
    self.currCellIndex = 1

    self.touchStartY = 0
    self.touchReleaseY = 0
    self.touchMoveY = 0

    ---------------
    self.drag = {
        currentOffsetX = self.offsetX,
        currentOffsetY = self.offsetY,
        startX = 0,
        startY = 0,
        isTap = true,
    }
    ---------
    self:registerScriptHandler(function(event)
        if event == "enter" then
            self:onEnter()
        elseif event == "exit" then
            self:onExit()
        end
    end)

    -- create container layer
    self.view = display.newLayer()
    self.view:addTouchEventListener(function(event, x, y)
        if SCROLL_CAN_TOUCH == true then
            return self:onTouch(event, x, y)
        else
            SCROLL_DRAG_TAP = true
        end 
    end)
    self:addChild(self.view)
end

function ScrollView:getCurrentCell()
    if self.currentIndex > 0 then
        return self.cells[self.currentIndex]
    else
        return nil
    end
end

function ScrollView:getCurrentIndex()
    return self.currentIndex
end

function ScrollView:setCurrentIndex(index)
    self:scrollToCell(index)
end

function ScrollView:setCurCellIndex()
    self.currCellIndex = self.currentIndex
end

function ScrollView:addCell(cell)
    self.view:addChild(cell)
    self.cells[#self.cells + 1] = cell
    self:reorderAllCells()
    self:dispatchEvent({name = "addCell", count = #self.cells})
end

function ScrollView:insertCellAtIndex(cell, index)
    self.view:addChild(cell)
    table.insert(self.cells, index, cell)
    self:reorderAllCells()
    self:dispatchEvent({name = "insertCellAtIndex", index = index, count = #self.cells})
end

function ScrollView:removeCellAtIndex(index)
    local cell = self.cells[index]
    cell:removeSelf()
    table.remove(self.cells, index)
    self:reorderAllCells()
    self:dispatchEvent({name = "removeCellAtIndex", index = index, count = #self.cells})
end

function ScrollView:getView()
    return self.view
end

function ScrollView:getTouchRect()
    return self.touchRect
end

function ScrollView:setTouchRect(rect)
    self.touchRect = rect
    self:dispatchEvent({name = "setTouchRect", rect = rect})
end

function ScrollView:getClippingRect()
    return self:getClippingRegion()
end

function ScrollView:setClippingRect(rect)
    self:setClippingRegion(rect)
    self:dispatchEvent({name = "setClippingRect", rect = rect})
end
--moveAlignRight靠边停止
function ScrollView:scrollToCell(index, animated, time, easing, moveAlignRight)
    local count = #self.cells
    if count < 1 then
        self.currentIndex = 0
        return
    end

    if index < 1 then
        index = 1
    elseif index > count then
        index = count
    end
    self.currentIndex = index

    local offset = 0
    for i = 2, index do
        local cell = self.cells[i - 1]
        local size = cell:getContentSize()
        if self.direction == ScrollView.DIRECTION_HORIZONTAL then
            offset = offset - size.width
        else
            offset = offset + size.height
        end
    end

    if (not self.LeftShiftX) then
        self.LeftShiftX = 0
    end
       if (self.MoveOffSetX) then
            local xx = self.LeftShiftX+(-self.touchMoveX)
            if (xx < 0) then
                self.LeftShiftX = 0
                self:setContentOffset(offset, animated, time, easing)
            elseif(xx >-self.MoveOffSetX) then
                 self.LeftShiftX = xx
                 if (self.MoveOffSetX) then
                    offset = self.MoveOffSetX   --纠正偏移
                end
                self.LeftShiftX = -self.MoveOffSetX
                 self:setContentOffset(offset, animated, time, easing)
            else
                self.LeftShiftX = xx
            end
            --print("self.LeftShiftX =", self.LeftShiftX )
            --print("self.MoveOffSetX =", -self.MoveOffSetX)
            --print("self.touchMoveX =", self.touchMoveX )    
    else

        self:setContentOffset(offset, animated, time, easing)
    end
    self:dispatchEvent({name = "scrollToCell", animated = animated, time = time, easing = easing})
end

function ScrollView:isTouchEnabled()
    return self.view:isTouchEnabled()
end

function ScrollView:setTouchEnabled(enabled)
    self.view:setTouchEnabled(enabled)
    SCROLL_CAN_TOUCH = true
    --self:dispatchEvent({name = "setTouchEnabled", enabled = enabled})
end

---- events

function ScrollView:onTouchBegan(x, y)
    self.drag = {
        currentOffsetX = self.offsetX,
        currentOffsetY = self.offsetY,
        startX = x,
        startY = y,
        isTap = true,
    }
    self.touchStartX = x
    self.touchStartY = y
    SCROLL_DRAG_TAP = self.drag.isTap

    local cell = self:getCurrentCell()
    cell:onTouch(event, x, y)

    return true
end

function ScrollView:onTouchMoved(x, y)
    local cell = self:getCurrentCell()
    if self.direction == ScrollView.DIRECTION_HORIZONTAL then
        if self.drag.isTap and math.abs(x - self.drag.startX) >= self.dragThreshold then
            self.drag.isTap = false
            cell:onTouch("cancelled", x, y)
        end

        if not self.drag.isTap then
            self:setContentOffset(x - self.drag.startX + self.drag.currentOffsetX)
        else
            cell:onTouch(event, x, y)
        end
    else
        if self.drag.isTap and math.abs(y - self.drag.startY) >= self.dragThreshold then
            self.drag.isTap = false
            cell:onTouch("cancelled", x, y)
        end

        if not self.drag.isTap then
            self:setContentOffset(y - self.drag.startY + self.drag.currentOffsetY)
        else
            cell:onTouch(event, x, y)
        end
    end
    SCROLL_DRAG_TAP =  self.drag.isTap
end

function ScrollView:onTouchEnded(x, y)
    self.touchReleaseX = x
    self.touchReleaseY = y---
    self.touchMoveX = self.touchReleaseX - self.touchStartX
    self.touchMoveY = self.touchReleaseY - self.touchStartY---

    if self.drag and self.drag.isTap then
        self:onTouchEndedWithTap(x, y)
    else
        self:onTouchEndedWithoutTap(x, y)
    end
    self.drag = nil

    if self.paintCell and self.hideCell then
        self:paintCell() -- add by hjc
        self:hideCell()
    end
end

function ScrollView:onTouchEndedWithTap(x, y)
    local cell = self:getCurrentCell()
    cell:onTouch(event, x, y)
    cell:onTap(x, y)
end

function ScrollView:onTouchEndedWithoutTap(x, y)
    error("ScrollView:onTouchEndedWithoutTap() - inherited class must override this method")
end

function ScrollView:onTouchCancelled(x, y)
    self.drag = nil
end

function ScrollView:onTouch(event, x, y)
    --print("event=", event)
    if self.currentIndex < 1 then return end
    if event == "began" then
        if not self.touchRect:containsPoint(CCPoint(x, y)) then return false end
        return self:onTouchBegan(x, y)
    elseif event == "moved" then
        self:onTouchMoved(x, y)
    elseif event == "ended" then
        self:onTouchEnded(x, y)
    else -- cancelled
        self:onTouchCancelled(x, y)
    end
end

---- private methods

function ScrollView:reorderAllCells()
    local count = #self.cells
    local x, y = 0, 0
    local maxWidth, maxHeight = 0, 0
    for i = 1, count do
        local cell = self.cells[i]
        cell:setPosition(x, y)
        if self.direction == ScrollView.DIRECTION_HORIZONTAL then
            local width = cell:getContentSize().width
            if width > maxWidth then maxWidth = width end
            x = x + width
        else
            local height = cell:getContentSize().height
            if height > maxHeight then maxHeight = height end
            y = y - height
        end
    end

    if count > 0 then
        if self.currentIndex < 1 then
            self.currentIndex = 1
        elseif self.currentIndex > count then
            self.currentIndex = count
        end
    else
        self.currentIndex = 0
    end

    local size
    if self.direction == ScrollView.DIRECTION_HORIZONTAL then
        size = CCSize(x, maxHeight)
    else
        size = CCSize(maxWidth, math.abs(y))
    end
    self.view:setContentSize(size)
end

function ScrollView:setContentOffset(offset, animated, time, easing)
    local ox, oy = self.offsetX, self.offsetY
    local x, y = ox, oy
    if self.direction == ScrollView.DIRECTION_HORIZONTAL then
        self.offsetX = offset
        x = offset

        local maxX = self.bouncThreshold
        local minX = -self.view:getContentSize().width - self.bouncThreshold + self.touchRect.size.width
        if x > maxX then
            x = maxX
        elseif x < minX then
            x = minX
        end

    else
        self.offsetY = offset
        y = offset

        local maxY = self.view:getContentSize().height + self.bouncThreshold - self.touchRect.size.height
        local minY = -self.bouncThreshold
        if y > maxY then
            y = maxY
        elseif y < minY then
            y = minY
        end
    end
    --
    if animated  then
        transition.stopTarget(self.view)
        transition.moveTo(self.view, {
            x = x,
            y = y,
            time = time or self.defaultAnimateTime,
            easing = easing or self.defaultAnimateEasing,
        })
    else
        self.view:setPosition(x, y)
    end
end

-- START add by hjc
-- 当numPages大于3时在拉动中加载资源
-- initPageData参数 numPages：可翻页数目 func：动态加载资源回调,并回传参数需要加载页面的index
function ScrollView:initPageData(numPages, func)
    assert(type(func) == 'function', "#para error! func should be function!")
    self.numPages_ = numPages
    self.dragFunc_ = func
end

function ScrollView:paintCell()
    if not self.numPages_ then return end
    if not (self.currentIndex > 0) then return end
    if self.currentIndex >= self.numPages_ then return end

    for i = 1, 2 do --加载两个页面
        local index = i + self.currentIndex
        if index <=  self.numPages_ then
            if not self.cells[i+self.currentIndex] then
                self.dragFunc_( index )
            end
        end
    end
end

function ScrollView:scrollToSpecifiedCell(specifiedIndex)
    assert(type(specifiedIndex) == 'number', "#para error! specifiedIndex should be number!")
    
    if not self.numPages_ and specifiedIndex <= #self.cells then
        self.currentIndex = specifiedIndex
        self:setCurCellIndex()
        self:setCurrentIndex(self.currentIndex)
        return
    end

    --以下部分self.numPages_ ~= nil
    if specifiedIndex > self.numPages_ then
        return
    end

    --若当前页 > 最大页数，返回到第一页
    if self.currentIndex > self.numPages_ then
        self.currentIndex = 1
        self:setCurCellIndex()
        self:setCurrentIndex(self.currentIndex)
        return
    end

    --若specifiedIndex <= 当前页，直接返回到specifiedIndex页
    if specifiedIndex <= self.currentIndex then
        self.currentIndex = specifiedIndex
        self:setCurCellIndex()
        self:setCurrentIndex(self.currentIndex)
        return
    end

    --若specifiedIndex > 当前页，加载新newPages并定位到specifiedIndex页
    local tempIndex = self.currentIndex
    local newPages = specifiedIndex - self.currentIndex + 2
    newPages = (specifiedIndex + 2) <= self.numPages_ and newPages or (self.numPages_ - self.currentIndex)

    for i = 1, newPages do
        local index = tempIndex + i

        if index <= specifiedIndex then
            self.currentIndex = self.currentIndex + 1
        end

        if not self.cells[index] and self.dragFunc_ then            
            self.dragFunc_(index)
        end
    end

    self:setCurCellIndex()

    self:setCurrentIndex(self.currentIndex)
end

function ScrollView:hideCell()
    if #self.cells <= 5 then return end
    for i, v in pairs(self.cells) do
        if math.abs(i - self.currentIndex) > 2 then
            if v.setVisible then v:setVisible(false) end
        else
            if v.setVisible then v:setVisible(true) end
        end
    end
end
-- END

function ScrollView:onExit()
    self:removeAllEventListeners()
end

return ScrollView