local ScrollViewII = class("ScrollViewII", function()
    local node = display.newNode()
    node:setNodeEventEnabled(true)
    require("framework.api.EventProtocol").extend(node)
    return node
end)

ScrollViewII.DIRECTION_VERTICAL   = 1 --垂直
ScrollViewII.DIRECTION_HORIZONTAL = 2 --水平
SCROLL_DRAG_TAP = false
SCROLL_CAN_TOUCH = true
--[[
ctor参数：
direction, 判断滑动方向
nodesize{width,height}, node 大小
clippingnodesize{width,height} 外层clippingnode大小
cellsize{width,height} 每个cell大小
]]
function ScrollViewII:ctor(paras)
    assert(type(paras) == 'table','ScrollViewII:ctor paras type error')
    assert(paras.direction == ScrollViewII.DIRECTION_VERTICAL or paras.direction == ScrollViewII.DIRECTION_HORIZONTAL,
           "ScrollViewII:ctor() - invalid direction")
    self.nodesize = totable(paras.nodesize)
    assert(type(self.nodesize) == 'table','ScrollViewII:ctor self.nodesize type error')
    self.clippingnodesize = totable(paras.clippingnodesize)
    assert(type(self.clippingnodesize) == 'table','ScrollViewII:ctor self.clippingnodesize type error')
    self.cellsize = totable(paras.cellsize)
    assert(type(self.cellsize) == 'table','ScrollViewII:ctor self.cellsize type error')

    self:setContentSize(CCSize(self.nodesize.width, self.nodesize.height))
    self.direction = paras.direction
    self.dragThreshold = 4                 --拖动在一起范围内不算点击事件

    self.ScrollAbled = true --设置是否可滑动

    --设置滑动有效范围
    self.scrollStartX = 0
    self.scrollEndX = 0
    self.scrollStartY = 0
    self.scrollEndY = 0

    --self.touchRect = rect
    self.offsetX = 0
    self.offsetY = 0
    --self.cells = {}
    self.currentIndex = 0
    --触摸
    self.touchStartX = 0
    self.touchReleaseX = 0
    self.touchMoveX = 0

    self.transitionMove = false
    --self.currCellIndex = 1

    self.changeX = 0
    self.changeY = 0
    --
    self.curX = 0
    self.curY = 0

    self.touchStartY = 0
    self.touchReleaseY = 0
    self.touchMoveY = 0

    -- create container layer
    self.view = display.newLayer()
    self.view:setTouchEnabled(true)
    self.view:setTouchSwallowEnabled(false)
    self.view:registerScriptTouchHandler(handler(self,self.onTouch))
    self:addChild(self.view)
end
--[[
设置是否可滑动
]]
function ScrollViewII:setTouchEnabled(enabled)
    self.view:setTouchEnabled(enabled)
    SCROLL_CAN_TOUCH = true
end

--设置是否可滑动
function ScrollViewII:setScrollAbled( enabled)
    self.ScrollAbled = enabled
end

function ScrollViewII:onTouch(event, x, y)
    --print(x,y)
    -- self.scrollStartX = point.x
    -- self.scrollEndX = self.clippingnodesize.width
    -- self.scrollStartY = point.y
    -- self.scrollEndY = self.clippingnodesize.height
    -- if not SCROLL_CAN_TOUCH then return end
    if event == "began" then
        if self.scrollEndY ~= 0 then
            if y >= self.scrollStartY and y <= self.scrollEndY then
                SCROLL_DRAG_TAP = false
                return self:onTouchBegan(x ,y)
            else
                return false
            end
        else
            SCROLL_DRAG_TAP = false
            return self:onTouchBegan(x ,y)
        end
    elseif event == "moved" then
        -- return self:onTouchMoved(x ,y)
        -- return self:onTouchMoved(x ,y)
        if self.scrollEndY ~= 0 then
            if y >= self.scrollStartY and y <= self.scrollEndY then
                return self:onTouchMoved(x ,y)
            else
                return false
            end
        else
            return self:onTouchMoved(x ,y)
        end
    elseif event == "ended" then
        return self:onTouchEnded(x, y)
    end    
end

function ScrollViewII:onTouchBegan(x, y)
    --print('ScrollViewII:onTouchBegan',x,y)
    self.touchStartX = x
    self.touchStartY = y
    self.offsetX = 0
    self.offsetY = 0
    return true
end

function ScrollViewII:onTouchMoved(x, y)
    if not self.ScrollAbled then return true end
    if self.transitionMove == false then 
        self.offsetX = x - self.touchStartX
        self.offsetY = y - self.touchStartY

        if ScrollViewII.DIRECTION_VERTICAL == self.direction then 
            if self.offsetY > self.dragThreshold or self.offsetY < -self.dragThreshold then
                self.transitionMove = true
            end
        end

        if ScrollViewII.DIRECTION_HORIZONTAL == self.direction then 
            if self.offsetX  > self.dragThreshold or self.offsetX  < -self.dragThreshold then
                self.transitionMove = true
            end
        end  
    end
    if self.transitionMove == true then
        if self.offsetX == 0 then self.offsetX = x - self.curX end
        if ScrollViewII.DIRECTION_HORIZONTAL == self.direction then 
            local offsetX = self:getPositionX() + self.offsetX
            if offsetX >= 0 then offsetX = self.offsetX/3 + self:getPositionX()
            elseif offsetX + self.nodesize.width < self.clippingnodesize.width then 
                offsetX = self.offsetX/3 + self:getPositionX() 
            end
            self:setPositionX(offsetX)
        end 

        if self.offsetY == 0 then self.offsetY = y - self.curY end
        if ScrollViewII.DIRECTION_VERTICAL == self.direction then
            local offsetY = self:getPositionY() + self.offsetY
            if offsetY <= 0 then offsetY =self.offsetY/3 + self:getPositionY()
            elseif offsetY - self.nodesize.height  + self.clippingnodesize.height >=0 then
                offsetY =self.offsetY/3 + self:getPositionY()
            end
            self:setPositionY(offsetY)
        end
    end
    self.offsetX = 0
    self.offsetY = 0
    self.curX = x
    self.curY = y 
    return true
end

function ScrollViewII:onTouchEnded(x, y)
    --print('ScrollViewII:onTouchEnded self:getPositionX()',self:getPositionX())
    --print('self.clippingnodesize.width=',self.clippingnodesize.width)
    self.touchReleaseX = x
    self.touchReleaseY = y

    if self.transitionMove then
        local point
        if  ScrollViewII.DIRECTION_HORIZONTAL == self.direction then
            if self:getPositionX() >= 0 then
                point = CCPoint(0,0)
            elseif self:getPositionX() < 0 then 
                if self:getPositionX() - self.clippingnodesize.width + self.nodesize.width <= 0 then
                    if self.clippingnodesize.width > self.nodesize.width then
                        point = CCPoint(0,0)
                    else
                        point = CCPoint(self.clippingnodesize.width - self.nodesize.width, 0)
                    end
                else
                    local ind = math.floor(self:getPositionX()+ 0.5)/self.cellsize.width
                    ind = math.floor(ind + 0.5)
                    point = CCPoint(ind*self.cellsize.width, 0)
                end
            end

            transition.stopTarget(self) --解决有多个动作执行时，显示异常bug
            transition.execute(self,CCMoveTo:create(0.2,point), {
                delay = 0.1,
                easing = "ELASTIC",
                onComplete = function()
                    --print("transition.execute HORIZONTAL completed")
                end,
            })
            self.transitionMove = false
        end

        if  ScrollViewII.DIRECTION_VERTICAL == self.direction then
            if self:getPositionY() <= 0 then
                point = CCPoint(0,0)
            elseif self:getPositionY() > 0 then
                if self:getPositionY() + self.clippingnodesize.height - self.nodesize.height >= 0 then
                    if self.clippingnodesize.height > self.nodesize.height then 
                        point = CCPoint(0,0)
                    else
                        point = CCPoint(0,self.nodesize.height - self.clippingnodesize.height)
                    end
                else
                    local ind = math.floor(self:getPositionY() + 0.5)/self.cellsize.height
                    ind = math.floor(ind + 0.5)
                    point = CCPoint(0,ind*self.cellsize.height)
                end
            end

            transition.stopTarget(self)
            transition.execute(self,CCMoveTo:create(0.2,point), {
                delay = 0.1,
                easing = "ELASTIC",
                onComplete = function()
                    --print("transition.execute  VERTICAL completed")
                end,
            })
            self.transitionMove = false
        end
    else
        SCROLL_DRAG_TAP = true
        self:onClicked(x,y)
    end
    return true    
end

function ScrollViewII:onClicked()
    -- echoError('this function should be overwrote by child')    
end

--[[
功能：滑向上一页
]]
function ScrollViewII:moveToPrev()
    local point 
    if ScrollViewII.DIRECTION_HORIZONTAL == self.direction then
        if self.nodesize.width <= self.clippingnodesize.width then return end
        if self:getPositionX() + self.clippingnodesize.width >= 0 then
            point = CCPoint(0,0)
            SCROLL_DRAG_TAP = false
            transition.execute(self,CCMoveTo:create(0.1,point), {
                delay = 0.1,
                easing = "ELASTIC",
                onComplete = function()
                    SCROLL_DRAG_TAP = true
                end,
            })
        else
            local xpos = math.floor((self:getPositionX() + self.clippingnodesize.width)/self.cellsize.width + 0.5)*self.cellsize.width
            point = CCPoint(xpos,0)
            SCROLL_DRAG_TAP = false
            transition.execute(self,CCMoveTo:create(0.2,point), {
                delay = 0.1,
                easing = "ELASTIC",
                onComplete = function()
                    SCROLL_DRAG_TAP = true
                end,
            })            
        end
    elseif ScrollViewII.DIRECTION_VERTICAL == self.direction then
        if self.nodesize.height <= self.clippingnodesize.height then return end
        --TODO
    end    
end

--[[
功能：滑向下一页
]]
function ScrollViewII:moveToNext()
    local point
    if ScrollViewII.DIRECTION_HORIZONTAL == self.direction then
        if self.nodesize.width <= self.clippingnodesize.width then return end
        if self.nodesize.width - (self.clippingnodesize.width - self:getPositionX()) <= self.clippingnodesize.width then
            point = CCPoint(self.clippingnodesize.width - self.nodesize.width ,0)
            SCROLL_DRAG_TAP = false
            transition.execute(self,CCMoveTo:create(0.1,point), {
                delay = 0.1,
                easing = "ELASTIC",
                onComplete = function()
                    SCROLL_DRAG_TAP = true
                end,
            })
        else
            local xpos = math.floor((self:getPositionX() - self.clippingnodesize.width)/self.cellsize.width + 0.5)*self.cellsize.width
            point = CCPoint(xpos,0)
            SCROLL_DRAG_TAP = false
            transition.execute(self,CCMoveTo:create(0.2,point), {
                delay = 0.1,
                easing = "ELASTIC",
                onComplete = function()
                    SCROLL_DRAG_TAP = true
                end,
            })
        end
    elseif ScrollViewII.DIRECTION_VERTICAL == self.direction then
        if self.nodesize.height <= self.clippingnodesize.height then return end
        --TODO
    end        
end

--[[
功能：设置移动某个cell到靠近中间的位置
参数：cell 的索引 index:1,2,....
]]
function ScrollViewII:moveToCenter(index)
    local all_cell_num, cell_num_per_page = math.floor(self.nodesize.width/self.cellsize.width), math.floor(self.clippingnodesize.width/self.cellsize.width)
    -- assert(index >= 1 and index <= all_cell_num)
    if index < 1 or index > all_cell_num then return end

    local num_of_center = math.floor(cell_num_per_page/2 + 0.5)
    local point_x = 0

    if ScrollViewII.DIRECTION_HORIZONTAL == self.direction then
        if all_cell_num <= cell_num_per_page then return end

        SCROLL_DRAG_TAP = false --动作期间不可点击
        if index <= num_of_center then
            point_x = 0
        elseif index >= all_cell_num - num_of_center + 1 then
            point_x = self.clippingnodesize.width - self.nodesize.width 
        else
            point_x = - (index - num_of_center) * self.cellsize.width
        end
    else
        --TODO
        return
    end

    if self:getPositionX() == point_x then return end --如果当前位置就是选中位置

    transition.execute(self,CCMoveTo:create(0.2,ccp(point_x,self:getPositionY())), {
        delay = 0.1,
        easing = "ELASTIC",
        onComplete = function()
            SCROLL_DRAG_TAP = true
        end,
    })
end

--[[
设置滑动范围
参数originx,originy为clippingnode左下角坐标
]]
function ScrollViewII:setScrollPosition(originx,originy)
    --assert(originx >=0 and originx <= display.width)
    --assert(originy >=0 and originy <= display.height)

    self.scrollStartX = originx
    self.scrollEndX = originx + self.clippingnodesize.width
    self.scrollStartY = originy
    self.scrollEndY = originy + self.clippingnodesize.height
end

function ScrollViewII:onExit()
    --print('ScrollViewII:onExit 执行')
end

return ScrollViewII