local UIScrollView = class("UIScrollView", function()
	return display.newNode()
end)

local UIScrollViewItem = import(".UIScrollViewItem")
local UIScrollViewIndicator = import(".UIScrollViewIndicator")

-- UIScrollView.SCROLLED_EVENT = "SCROLLED_EVENT"
-- UIScrollView.OVERBORDER_EVENT = "OVERBORDER_EVENT"
UIScrollView.CLICKED_EVENT = "CLICKED_EVENT"

function UIScrollView:ctor( args )
--[[
args:
	row	      number,  显示的行数
	col	      number,  显示的列数
	nums      table,   item元素个数
	size      table,   显示大小 { height = 400, width = 500 }
	interval  table,   横纵item间隔 { x = 10, y = 10 }
	mode      number,  滑动模式(1/2) 默认为1 即连续滑动
	direction number,  滑动方向(1/2) 默认为1 即水平拖动
]]
	assert(type(args.row) == "number")
	assert(type(args.col) == "number")
	assert(type(args.nums) == "number")
	assert(type(args.size) == "table")
	assert(type(args.interval) == "table")
	assert(type(args.mode) == "number" or args.mode == nil)
	assert(type(args.direction) == "number" or args.direction == nil)
	self.args_ = args
	self.args_.border = { x = self.args_.interval.x/2, y = self.args_.interval.y/2}  -- 边框间隔
	self:setMode(args.mode)
	self:setDirection(args.direction)
	self.frame_visible_ = false

	-- 保存滑动数据
	self.startP = { x = 0, y = 0 } --记录起始点坐标
	self.curP   = { x = 0, y = 0 } --记录当前点坐标
	self.moveP  = { x = 0, y = 0 } --记录滑动相对坐标

	cc.GameObject.extend(self)
	self:addComponent("components.behavior.EventProtocol"):exportMethods()
	self:addScriptEventListener(cc.Event.EXIT_SCENE, function()
		self:removeAllEventListeners()
	end)

	-- 设置自身可触摸并响应触摸事件
	self:setTouchEnabled(true)
	self:addTouchEventListener(handler(self, self.onTouch_))
	-- self.layer = display.newLayer():addTo(self)
	-- self.layer:setTouchEnabled(true)
	-- self.layer:addTouchEventListener(handler(self, self.onTouch_))--, false, -1, true)
end

--=============================
-- 触摸响应
--=============================
function UIScrollView:isOutSide( x, y )
--[[ 在显示区域才能响应触摸
]]
	local x_, y_ = self:getPosition()
	if x < x_ or x > x_ + self.args_.size.width or
		y < y_ or y > y_ + self.args_.size.height then
		return true
	end
	return false
end

function UIScrollView:onTouch_( event, x, y )
--[[ 触摸响应事件
]]
	local p = { x = x, y = y, startX = self.startP.x, startY = self.startP.y }
	if event == "began" then
		return self:onTouchBegan_(p)
	elseif event == "moved" then
		return self:onTouchMoved_(p)
	elseif event == "ended" then
		return self:onTouchEnded_(p)
	end
end

function UIScrollView:onTouchBegan_( event )
--[[ 
]]
	local x, y = event.x, event.y
	if self:isOutSide(x, y) then return false end

	-- 记录点击起始点
	self.startP.x = x
	self.startP.y = y
	-- 保存当前点坐标
	self.curP.x = x
	self.curP.y = y

	return true
end

function UIScrollView:onTouchMoved_( event )
--[[ 
]]
	local x, y = event.x, event.y
	local startX, startY = event.startX, event.startY
	-- 当前点或者起始点在外面 不做操作
	if self:isOutSide(x, y) or self:isOutSide(startX, startY) then return end

	-- 控制显示节点
	self:setVisibleItem()

	if self.direction_ == 1 then
		local changeX = self.curP.x - x  --计算拖动距离
		self.curP.x = x
		local moveP_x = self.moveP.x - changeX
		if moveP_x > self.min_.x or moveP_x < -self.max_.x then
			--超出边缘时 滑动距离缩小2倍
			changeX = changeX / 3
		end
		self.moveP.x = self.moveP.x - changeX
		self.page_node_:setPositionX(self.moveP.x)
	else
		local changeY = self.curP.y - y
		self.curP.y = y
		local moveP_y = self.moveP.y - changeY
		if moveP_y > self.min_.y or moveP_y < -self.max_.y then
			--超出边缘时 滑动距离缩小2倍
			changeY = changeY / 3
		end
		self.moveP.y = self.moveP.y - changeY
		self.page_node_:setPositionY(self.moveP.y)
	end
end

function UIScrollView:calMoveDistance()
--[[ 保证item完全显示 计算动画移动距离
]]
	local delta, length
	if self.direction_ == 1 then
		delta = self.page_node_:getPositionX()
		length = self.cell_size_.width + self.args_.interval.x
	else
		delta = self.page_node_:getPositionY()
		length = self.cell_size_.height + self.args_.interval.y
	end
	local d = math.abs(delta) % length
	if d*2 <= length and delta >= 0 then
		return -d
	elseif d*2 <= length and delta < 0 then
		return d
	elseif d*2 > length and delta >= 0 then
		return length - d
	elseif d*2 > length and delta < 0 then
		return -(length - d)
	else
		return 0
	end
end

function UIScrollView:onTouchEnded_( event )
--[[ 
]]
	local x, y = event.x, event.y
	local startX, startY = event.startX, event.startY
	-- 判断起始点是不是在显示范围外
	-- 不加当前点判断的原因是避免从显示范围内部滑动到外部之后不做操作
	if self:isOutSide(startX, startY) then return end

	if self.direction_ == 1 then
		if self.moveP.x > self.min_.x then
			transition.moveTo(self.page_node_, {x = self.min_.x, time = 0.1, onComplete = function()
				self.moveP.x = self.min_.x
				self:setVisibleItem()
			end})
		elseif self.moveP.x < -self.max_.x then
			transition.moveTo(self.page_node_, {x = -self.max_.x, time = 0.1, onComplete = function()
				self.moveP.x = -self.max_.x
				self:setVisibleItem()
			end})
		else
			if self.mode_ == 1 then
				local d = self:calMoveDistance()
				transition.moveBy(self.page_node_, {x = d, time = 0.1, onComplete = function()
					self.moveP.x = self.moveP.x + d
					self:setVisibleItem()
				end})
			else
				if self:canTurningPage(x, y) then
					self:pageTurning(x - self.startP.x)
				else
					transition.moveBy(self.page_node_, {x = self.startP.x - x, time = 0.1})
					self.moveP.x = self.moveP.x + self.startP.x - x
				end
			end
		end
	else
		if self.moveP.y > self.min_.y then
			transition.moveTo(self.page_node_, {y = self.min_.y, time = 0.1, onComplete = function()
				self.moveP.y = self.min_.y
				self:setVisibleItem()
			end})
		elseif self.moveP.y < -self.max_.y then
			transition.moveTo(self.page_node_, {y = -self.max_.y, time = 0.1, onComplete = function()
				self.moveP.y = -self.max_.y
				self:setVisibleItem()
			end})
		else
			if self.mode_ == 1 then
				local d = self:calMoveDistance()
				transition.moveBy(self.page_node_, {y = d, time = 0.1, onComplete = function()
					self.moveP.y = self.moveP.y + d
					self:setVisibleItem()
				end})
			else
				if self:canTurningPage(x, y) then
					self:pageTurning(y - self.startP.y)
				else
					transition.moveBy(self.page_node_, {y = self.startP.y - y, time = 0.1})
					self.moveP.y = self.moveP.y + self.startP.y - y
				end
			end
		end
	end

end

function UIScrollView:setVisibleItem()
--[[ 控制显示的item  节省资源
]]
	if self.mode_ == 1 then
		if self.direction_ == 1 then
			for i,v in ipairs(self.items_) do
				local x = gPVPMenu:round(v:getPositionX() + self.moveP.x, 2)
				if x > -self.cell_size_.width and x < self.args_.size.width then
					v:setVisible(true)
				else
					v:setVisible(false)
				end
			end
		else
			for i,v in ipairs(self.items_) do
				local y = gPVPMenu:round(v:getPositionY() + self.moveP.y, 2)
				if y > -self.cell_size_.height and y < self.args_.size.height then
					v:setVisible(true)
				else
					v:setVisible(false)
				end
			end
		end
	else --TODO 翻页滑动还可以改进
		if self.direction_ == 1 then
			for i,v in ipairs(self.items_) do
				local x = gPVPMenu:round(v:getPositionX() + self.moveP.x, 2)
				if x > -self.args_.size.width and x < self.args_.size.width*2 then
					v:setVisible(true)
				else
					v:setVisible(false)
				end
			end
		else
			for i,v in ipairs(self.items_) do
				local y = gPVPMenu:round(v:getPositionY() + self.moveP.y, 2)
				if y > -self.args_.size.height and y < self.args_.size.height*2 then
					v:setVisible(true)
				else
					v:setVisible(false)
				end
			end
		end
	end
end

function UIScrollView:onItemClicked_( event )
--[[ item点击事件
]]
	-- 点击的item在显示范围外时不处理
	if self:isOutSide(event.x, event.y) then return end

	local select_item = self.items_[event.index]
	for _, item in ipairs(self.items_) do
		item:unSelect()
	end
	select_item:select()
	self:dispatchEvent({ name = UIScrollView.CLICKED_EVENT, index = event.index, x = event.x, y = event.y})
end

function UIScrollView:canTurningPage( x, y )
--[[ 移动一小段时 不翻页
]]
	if math.abs(self.startP.x - x) < self.args_.size.width/8 and self.direction_ == 1 then
		return false
	elseif math.abs(self.startP.y - y) < self.args_.size.height/8 and self.direction_ == 2 then
		return false
	end
	return true
end

function UIScrollView:pageTurning( distance )
--[[ 翻页
]]
	if self.direction_ == 1 then
		if distance >= 0 then
			if self.page_index_ ~= 1 then
				self.page_index_ = self.page_index_ - 1
			end
		else
			if self.page_index_ ~= self.pages_ then
				self.page_index_ = self.page_index_ + 1
			end
		end
		self.moveP.x = (1 - self.page_index_) * self.args_.size.width
		transition.stopTarget(self.page_node_)
		transition.moveTo(self.page_node_, {x = self.moveP.x, time = 0.2, onComplete = function()
			self.indicator_:setIndex(self.page_index_)
			self:setVisibleItem()
		end})
	else
		if distance >= 0 then
			if self.page_index_ ~= self.pages_ then
				self.page_index_ = self.page_index_ + 1
			end
		else
			if self.page_index_ ~= 1 then
				self.page_index_ = self.page_index_ - 1
			end
		end
		self.moveP.y = (self.page_index_ - 1) * self.args_.size.height
		transition.stopTarget(self.page_node_)
		transition.moveTo(self.page_node_, {y = self.moveP.y, time = 0.2, onComplete = function()
			self.indicator_:setIndex(self.page_index_)
			self:setVisibleItem()
		end})
	end
end

function UIScrollView:turnToPageHead()
--[[ 自动滑动到开始
]]
	for i,v in ipairs(self.items_) do
		v:setVisible(true)
	end
	if self.direction_ == 1 then
		self.moveP.x = self.max_.x
		transition.moveTo(self.page_node_, {x = self.min_.x, time = 0.3, onComplete = function()
			self:setVisibleItem()
		end})
	else
		self.moveP.y = self.min_.y
		transition.moveTo(self.page_node_, {y = self.max_.y, time = 0.3, onComplete = function()
			self:setVisibleItem()
		end})
	end
end

function UIScrollView:turnToPageEnd( skipAction )
--[[ 自动滑动到末尾
]]
	for i,v in ipairs(self.items_) do
		v:setVisible(true)
	end
	if self.direction_ == 1 then
		self.moveP.x = self.max_.x
		if skipAction then
			self.page_node_:setPositionX(self.max_.x)
			self:setVisibleItem()
		else
			transition.moveTo(self.page_node_, {x = self.max_.x, time = 0.3, onComplete = function()
				self:setVisibleItem()
			end})
		end
	else
		self.moveP.y = self.min_.y
		if skipAction then
			self.page_node_:setPositionY(self.min_.y)
			self:setVisibleItem()
		else
			transition.moveTo(self.page_node_, {y = self.min_.y, time = 0.3, onComplete = function()
				self:setVisibleItem()
			end})
		end
	end
end


--=============================
-- 属性设置
--=============================
function UIScrollView:setMode( mode )
--[[ 设置滑动方式
mode:
	1 -> 连续滑动
	2 -> 翻页滑动
]]
	assert(mode == 1 or mode == 2, "设置滑动方式出错 - 参数错误")
	self.mode_ = mode
	if mode == 2 then
		self.page_index_ = 1
	end
end

function UIScrollView:setDirection( direction )
--[[ 设置滑动方向
direction:
	1 -> 水平
	2 -> 垂直
]]
	assert(direction == 1 or direction == 2, "设置滑动方向出错 - 参数错误")
	self.direction_ = direction
end

function UIScrollView:showFrame( true_or_false )
--[[ 设置边框显示隐藏
]]
	self.frame_visible_ = true_or_false
	for _, item in ipairs(self.items_) do
		item:setFrameVisible(true_or_false)
	end
	self.frame_:setVisible(true_or_false)
end

function UIScrollView:showIndicator( true_or_false )
	if self.mode_ == 1 then return end
	self.indicator_:setVisible(true_or_false)
end

function UIScrollView:setClippingRegion( scaleX, scaleY )
--[[ 父节点缩放后需调用该方法 否则会引发裁切问题
]]
	scaleY = scaleY or scaleX
	local rect = self.clipnode_:getClippingRegion()
	local new_rect = CCRect(rect.origin.x, rect.origin.y, rect.size.width*scaleX, rect.size.height*scaleY)
	self.clipnode_:setClippingRegion(new_rect)
end

function UIScrollView:touchEnabled( true_or_false )
--[[ 设置是否响应触摸
]]
	self:setTouchEnabled(true_or_false)
	for i,v in ipairs(self.items_) do
		v:setTouchEnabled(true_or_false)
	end
end

function UIScrollView:setIndex( page_index )
--[[ 翻页模式 设置索引
]]
	if self.mode_ ~= 2 then return end
	self.page_index_ = page_index
	if self.direction_ == 1 then
		self.moveP.x = (1 - self.page_index_) * self.args_.size.width
		self.page_node_:setPositionX(self.moveP.x)
	else
		self.moveP.y = (self.page_index_ - 1) * self.args_.size.height
		self.page_node_:setPositionY(self.moveP.y)
	end
	self.indicator_:setIndex(self.page_index_)
	self:setVisibleItem()
end
--=============================
-- Item排列
--=============================
function UIScrollView:alignItems( nodes )
--[[ 根据排列方式 排列item
]]
	assert(type(nodes) == "table", "设置ITEM出错 - 参数类型错误")
	self.nodes_ = nodes

	local rect = CCRect(0, 0, self.args_.size.width, self.args_.size.height)
	self.clipnode_ = display.newClippingRegionNode(rect):addTo(self)
	self.page_node_ = display.newNode():addTo(self.clipnode_)

	self.frame_ = display.newRect(self.args_.size.width, self.args_.size.height):addTo(self)
		:pos(self.args_.size.width/2, self.args_.size.height/2)
	self.frame_:setLineColor(ccc4f(0.0, 0.0, 1.0, 1.0))
	self.frame_:setLineWidth(2)
	self.frame_:setVisible(self.frame_visible_)

	self:getData(self.args_.nums)

	-- 获取scrollView的坐标
	local pos_t = { x = self:getPositionX(), y = self:getPositionY() }
	-- 起始坐标
	local pos_x = self.args_.border.x
	local pos_y = self.args_.size.height - self.args_.border.y - self.cell_size_.height

	if self.direction_ == 1 then
		-- 水平模式 item从上至下排列
		for index, node in ipairs(self.nodes_) do
			if index > 1 and (index%self.rows_per_page_ == 1 or self.rows_per_page_ == 1) then
				pos_x = pos_x + self.cell_size_.width + self.args_.interval.x
				pos_y = self.args_.size.height - self.args_.border.y - self.cell_size_.height
			end
			local item = self:makeItem({
					index = index,
					node  = node,
				}):addTo(self.page_node_):pos(pos_x, pos_y)
			if self.items_ == nil then self.items_ = {} end
			table.insert(self.items_, item)
			pos_y = pos_y - self.cell_size_.height - self.args_.interval.y
		end
	else
		-- 垂直模式 item 从左至右排列
		for index, node in ipairs(self.nodes_) do
			if index > 1 and (index%self.cols_per_page_ == 1 or self.cols_per_page_ == 1)then
				pos_x = self.args_.border.x
				pos_y = pos_y - self.args_.interval.y - self.cell_size_.height
			end
			local item = self:makeItem({
					index = index,
					node  = node,
				}):addTo(self.page_node_):pos(pos_x, pos_y)
			if self.items_ == nil then self.items_ = {} end
			table.insert(self.items_, item)
			pos_x = pos_x + self.cell_size_.width + self.args_.interval.x
		end
	end

	if self.mode_ == 2 then
		-- 添加指示器
		self.indicator_ = UIScrollViewIndicator.new({
				nums = self.pages_,
				width = self.direction_ == 1 and self.args_.size.width or self.args_.size.height,
				direction = self.direction_ ,
				index = 1,
			}):addTo(self, 2)
		-- 是否让指示器显示到外部 不遮挡item
		local size = self.indicator_:getCascadeBoundingBox().size
		local move = self.direction_ == 1 and -size.height or -size.width
		local x, y = self.indicator_:getPosition()
		x = self.direction_ == 1 and x or x-size.width
		y = self.direction_ == 1 and y-size.height or y
		self.indicator_:setPosition(x, y)
	end
	-- 构造完毕后设置显示
	self:setVisibleItem()
end

function UIScrollView:getData( nums )
--[[ 根据元素个数计算并返回各种数据
]]
	-- 	每页行列数 页数
	self.rows_per_page_ = self.args_.row
	self.cols_per_page_ = self.args_.col
	self.nums_per_page_ = self.rows_per_page_ * self.cols_per_page_
	self.pages_ = math.ceil(nums / self.nums_per_page_)

	-- 实际行列数
	if self.direction_ == 1 then
		self.rows_ = self.rows_per_page_
		self.cols_ = math.ceil(nums / self.rows_)
	else
		self.cols_ = self.cols_per_page_
		self.rows_ = math.ceil(nums / self.cols_)
	end

	-- 计算每个item显示大小
	self.cell_size_ = {
		width = (self.args_.size.width - 2*self.args_.border.x - self.args_.interval.x * (self.cols_per_page_-1)) / self.cols_per_page_,
		height = (self.args_.size.height - 2*self.args_.border.y - self.args_.interval.y * (self.rows_per_page_-1)) / self.rows_per_page_
	}

	-- 计算整个控件的长宽
	if self.mode_ == 1 then
		-- 连续滑动时 根据行列数来计算总长宽
		self.width_ = self.cols_ * self.cell_size_.width + (self.cols_ - 1) * self.args_.interval.x
		self.height_ = self.rows_ * self.cell_size_.height + (self.rows_ - 1) * self.args_.interval.y

		self.min_ = { x = 0, y = self.height_ - self.args_.size.height + 2*self.args_.border.y }
		self.max_ = { x = self.width_ - self.args_.size.width + 2*self.args_.border.x, y = 0 }
		if self.min_.y < 0 then self.min_.y = 0 end
	else
		-- 按页滚动时 根据页面数来计算总长宽
		if self.direction_ == 1 then
			self.width_ = self.pages_ * self.args_.size.width
			self.height_ = self.args_.size.height
		else
			self.width_ = self.args_.size.width
			self.height_ = self.pages_ * self.args_.size.height
		end
		self.min_ = { x = 0, y = self.height_ - self.args_.size.height }
		self.max_ = { x = self.width_ - self.args_.size.width, y = 0 }
	end

	if self.pages_ <= 1 then
		self.max_.x = self.min_.x
		self.max_.y = self.min_.y
	end

end

--=======================
-- 元素的增删改查
--=======================
function UIScrollView:removeItem( index )
--[[ 删除Item
]]
	assert(type(index)=="number", "参数index错误 - 必须是数值类型")
	assert(index > 0 and index <= #self.items_, "参数index错误 - 索引越界")

	if self.items_[index] ~= nil then
		-- 移动  后一个元素设置为前一个元素的位置
		for i=#self.items_, index+1, -1 do
			local x, y = self.items_[i-1]:getPosition()
			self.items_[i]:setPosition(x, y)
			-- 改变index
			self.items_[i].index_ = self.items_[i].index_ - 1
		end
		-- 删除
		self.items_[index]:removeFromParentAndCleanup(true)
		table.remove(self.items_, index)

		table.remove(self.nodes_, index)
	end

	local x, y = self.page_node_:getPosition()
	local pages = self.pages_
	-- 删除元素之后 控件相关值有变更
	self:getData(#self.items_)
	
	if self.mode_ == 1 then
		if self.direction_ == 1 and math.abs(x)+self.args_.size.width > self.width_ and self.width_ > self.args_.size.width then
			local d = self.cell_size_.width + self.args_.interval.x
			transition.moveBy(self.page_node_, {x = d, time = 0.1})
			self.moveP.x = self.moveP.x + d
		elseif self.direction_ == 2 and math.abs(y)+self.args_.size.height > self.height_  and self.height_ > self.args_.size.height then
			local d = self.cell_size_.height + self.args_.interval.y
			transition.moveBy(self.page_node_, {y = -d, time = 0.1})
			self.moveP.y= self.moveP.y - d
		end
	else
		-- 该页元素被删除 滑动到上页
		if self.direction_ == 1 and math.abs(x)+self.args_.size.width > self.width_ then
			self:pageTurning(1)
		elseif self.direction_ == 2 and math.abs(y)+self.args_.size.height > self.height_ then
			self:pageTurning(-1)
		end
		if pages ~= self.pages and self.indicator_:isVisible() then
			-- 重设指示器
			self.indicator_:removeFromParentAndCleanup(true)
			self.indicator_ = nil
			self.indicator_ = UIScrollViewIndicator.new({
					nums = self.pages_,
					width = self.direction_ == 1 and self.args_.size.width or self.args_.size.height,
					direction = self.direction_ ,
					index = self.page_index_,
				}):addTo(self, 2)
		end
	end

	self:setVisibleItem()
end

function UIScrollView:addItem( index, node )
--[[ 增加Item
]]
	assert(type(index)=="number", "参数index错误 - 必须是数值类型")
	assert(index > 0, "参数index错误 - 索引越界")

	if index > #self.items_ then
		index = #self.items_ + 1
	end

	local item = self:makeItem({
			index = index,
			node = node,
		})

	if index <= #self.items_ then -- index 小于等于已有item个数
		local x, y = self.items_[index]:getPosition()
		item:addTo(self.page_node_):pos(x, y)

		--从index开始的所有元素后移
		local size = #self.items_
		for i=index, size-1 do
			local x, y = self.items_[i+1]:getPosition()
			self.items_[i]:setPosition(x, y)
			-- 改变index
			self.items_[i].index_ = self.items_[i].index_ + 1
		end
		--最后的元素
		x, y = self:calNextPosition(#self.items_)
		self.items_[size]:setPosition(x, y)
		self.items_[size].index_ = self.items_[size].index_ + 1
	else
		--索引不在范围内  直接追加至末尾
		local x, y = self:calNextPosition(index-1)
		item:addTo(self.page_node_):pos(x, y)
	end

	-- 增加item 和 node
	table.insert(self.items_, index, item)
	table.insert(self.nodes_, index, node)
	
	local pages = self.pages_
	-- 添加元素之后 控件相关值有变更
	self:getData(#self.items_)

	if self.mode_ == 2 and pages ~= self.pages and self.indicator_:isVisible() then
		-- 重设指示器
		self.indicator_:removeFromParentAndCleanup(true)
		self.indicator_ = nil
		self.indicator_ = UIScrollViewIndicator.new({
				nums = self.pages_,
				width = self.direction_ == 1 and self.args_.size.width or self.args_.size.height,
				direction = self.direction_ ,
				index = self.page_index_,
			}):addTo(self, 2)
	end
	self:setVisibleItem()
end

function UIScrollView:calNextPosition( index )
--[[ 计算该index的下一个元素坐标
]]
	assert(index <= #self.items_, "参数index错误 - 索引越界")
	if index == 0 then
		local x = self.args_.border.x
		local y = self.args_.size.height - self.args_.border.y - self.cell_size_.height
		return x, y
	end
	local x, y = self.items_[index]:getPosition()
	if self.direction_ == 1 then
		if (index+1) > 1 and ((index+1)%self.rows_per_page_ == 1 or self.rows_per_page_ == 1) then
			x = x + self.cell_size_.width + self.args_.interval.x
			y = self.args_.size.height - self.args_.border.y - self.cell_size_.height
		else
			y = y - self.cell_size_.height - self.args_.interval.y
		end
	else
		if (index+1) > 1 and ((index+1)%self.cols_per_page_ == 1 or self.cols_per_page_ == 1)then
			x = self.args_.border.x
			y = y - self.args_.interval.y - self.cell_size_.height
		else
			x = x + self.cell_size_.width + self.args_.interval.x
		end
	end
	return x, y
end

function UIScrollView:makeItem( args )
--[[ 工厂创建Item并增加监听
]]
	local item =  UIScrollViewItem.new({
			index = args.index,
			node  = args.node,
			size  = self.cell_size_,
			parent_size = self.args_.size,
			parent_pos = { x = self:getPositionX(), y = self:getPositionY() },
		})
	item:addEventListener(UIScrollView.CLICKED_EVENT, handler(self, self.onItemClicked_))
	item:addEventListener("BEGAN_EVENT", handler(self, self.onTouchBegan_))
	item:addEventListener("MOVED_EVENT", handler(self, self.onTouchMoved_))
	item:addEventListener("ENDED_EVENT", handler(self, self.onTouchEnded_))
	item:addScriptEventListener(cc.Event.EXIT_SCENE, function()
		item:removeAllEventListeners()
	end)
	item:setFrameVisible(self.frame_visible_)
	return item
end

return UIScrollView