local UIScrollViewItem = class("UIScrollViewItem",function()
	return display.newNode()
end)

local scheduler = CCDirector:sharedDirector():getScheduler()
--[[
index: 保存当前索引
node: 用于扩展item中显示的对象
size: item显示区域大小 非item的大小
	  若size小于真实大小 则会根据宽高比缩放自动缩放 
	  默认值为sprite的大小

UIScrollViewItem.new({
	index = 1,
	node  = display.newNode(),
	size  = { width = 10, height = 10 },
	parent_size = { height = 400, width = 500 },
	parent_pos = { x = 0, y = 0 }
})
]]

function UIScrollViewItem:ctor( args )
	assert(type(args) == "table", "The type of param #1 args should be a table")
	assert(type(args.node) == "userdata", "The type of args.node should be a userdata")
	assert(type(args.size) == "table", "The type of args.size should be a table")
	assert(type(args.parent_size) == "table", "The type of args.parent_size should be a table")
	assert(type(args.parent_pos) == "table", "The type of args.parent_pos should be a table")

	self.index_ = args.index
	local real_size = args.node:getCascadeBoundingBox().size
	local scale = self:getScale(args.size, real_size)
	self.node_ = args.node
	self.node_:addTo(self)
	self.node_:setScale(scale)
	self.node_:setAnchorPoint(CCPoint(0, 0))

	self.parent_pos_ = args.parent_pos
	self.parent_size_ = args.parent_size

	self.frame_ = display.newRect(args.size.width, args.size.height):addTo(self, 100)
		:pos(args.size.width/2, args.size.height/2)
	self.frame_:setLineColor(ccc4f(0.0, 1.0, 0.0, 1.0))
	self.frame_:setLineWidth(2)
	self.frame_:setVisible(false)

	self:setTouchEnabled(true)
	self:addTouchEventListener(handler(self, self.onTouch_))
	
	cc.GameObject.extend(self)
	self:addComponent("components.behavior.EventProtocol"):exportMethods()

	self.time_ = 0
	self.startP = { x = 0, y = 0 }
end

-- function UIScrollViewItem:isOutSide( x, y )
-- --[[ 在显示区域才能响应触摸
-- ]]
-- 	local x_, y_ = self.parent_pos_.x, self.parent_pos_.y
-- 	if x < x_ or x > x_ + self.parent_size_.width or
-- 		y < y_ or y > y_ + self.parent_size_.height then
-- 		return true
-- 	end
-- 	return false
-- end

function UIScrollViewItem:onTouch_( event, x, y )
--[[ 分发item点击事件
]]
	-- if self:isOutSide(x, y) then return end
	if event == "began" then
		self:dispatchEvent({ name = "BEGAN_EVENT", x = x, y = y})
		self.startP.x, self.startP.y = x, y
		-- 点击开始时 计时
		self.handler = scheduler:scheduleScriptFunc(function( dt )
			self.time_ = self.time_ + dt
			-- 按住时间大于0.4秒 表示选中
			if self.time_ > 0.4 then
				self:dispatchEvent({ name = "CLICKED_EVENT", index = self.index_, x = x, y = y})
				if self.handler ~= nil then
					scheduler:unscheduleScriptEntry(self.handler)
					self.handler = nil
					self.time_ = 0
				end
			end
		end, 0, false)
		return true
	elseif event == "moved" then
		self:dispatchEvent({ name = "MOVED_EVENT", x = x, y = y, startX = self.startP.x, startY = self.startP.y})
		if self.handler == nil then	return end
		if math.abs(x-self.startP.x)>5 or math.abs(y-self.startP.y)>5 then
			scheduler:unscheduleScriptEntry(self.handler)
			self.handler = nil
			self.time_ = 0
		end
	elseif event == "ended" then
		self:dispatchEvent({ name = "ENDED_EVENT", x = x, y = y, startX = self.startP.x, startY = self.startP.y})
		if self.handler ~= nil then
			if math.abs(x-self.startP.x)<=5 and math.abs(y-self.startP.y)<=5 then
				self:dispatchEvent({ name = "CLICKED_EVENT", index = self.index_, x = x, y = y})
			end
			scheduler:unscheduleScriptEntry(self.handler)
			self.handler = nil
		end
		self.time_ = 0
	end
end

function UIScrollViewItem:getScale( size, real_size )
--[[ 计算使node能完全显示的scale值
]]
	local width_scale  = size.width / real_size.width
	local height_scale = size.height / real_size.height
	if (size.height / size.width) >= (real_size.height / real_size.width) then
		return width_scale
	else
		return height_scale
	end
end

function UIScrollViewItem:select()
--[[ 选中时
]]
	self.selected_ = true
	self.frame_:setLineColor(ccc4f(1.0, 0.0, 0.0, 1.0))
	self.frame_:setLineWidth(4)
end

function UIScrollViewItem:unSelect()
--[[ 取消选中时
]]
	self.selected_ = false
	self.frame_:setLineColor(ccc4f(0.0, 1.0, 0.0, 1.0))
	self.frame_:setLineWidth(2)
end

function UIScrollViewItem:setFrameVisible( is_visible )
	self.frame_:setVisible(is_visible)
end

return UIScrollViewItem