local secondMenuCell = require("app.views.secondMenu")
local PageControl = require("app.ui.PageControl")
local SecondMenu = class("SecondMenu",PageControl)

SecondMenu.INDICATOR_MARGIN = 30

function SecondMenu:ctor(rect,data)

    SecondMenu.super.ctor(self, rect, PageControl.DIRECTION_HORIZONTAL)

    local bg = display.newSprite("box_Bg.png",rect.size.width/2,rect.size.height/2)
    local bgSize = bg:getContentSize()
    local scaleX = rect.size.width/bgSize.width
    local scaleY = rect.size.height/bgSize.height
    bg:setScaleX(scaleX)
    bg:setScaleY(scaleY)
    self:addChild(bg,-10)
    --
    -- add cells
    local rows, cols = 1, 6
    
    local numPages = math.ceil(#data / (rows * cols))
    
    for pageIndex = 1, numPages do
        local cellData = {}
        for i=1,6 do 
            local index=i+(pageIndex-1)*cols
            if data[index] then
                cellData[i] = data[index]
            else
                break
            end
        end
        local cell = secondMenu.new(CCSize(display.width, rect.size.height), rows, cols,cellData)
        cell:addEventListener("onClickItem", function(event) return self:onClickItem(event) end)
        self:addCell(cell)
    end

    -- add indicators
    local x = (self:getClippingRect().size.width - SecondMenu.INDICATOR_MARGIN * (numPages - 1)) / 2
    local y = self:getClippingRect().origin.y + 50

    self.indicator_ = display.newSprite("cellSelected.png")
    self.indicator_:setPosition(x, y)
    self.indicator_.firstX_ = x

    for pageIndex = 1, numPages do
        local icon = display.newSprite("cellIndicator.png")
        icon:setPosition(x, y)
        self:addChild(icon)
        x = x + SecondMenu.INDICATOR_MARGIN
    end
    self:addChild(self.indicator_)
end

function SecondMenu:scrollToCell(index, animated, time)
    SecondMenu.super.scrollToCell(self, index, animated, time)
    transition.stopTarget(self.indicator_)
    local x = self.indicator_.firstX_ + (self:getCurrentIndex() - 1) * SecondMenu.INDICATOR_MARGIN
    if animated then
        time = time or self.defaultAnimateTime
        transition.moveTo(self.indicator_, {x = x, time = time / 2})
    else
        self.indicator_:setPositionX(x)
    end
end

function SecondMenu:onClickItem(event)
    self:dispatchEvent({name = "onClickItem", itemData = event.itemData})
end

return SecondMenu
